@@ -1,6 +1,6 @@
 /*
  * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2008
+ *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
  *
  * SARG donations:
@@ -24,14 +24,14 @@
  */
 
 #include "include/conf.h"
+#include "include/defs.h"
 
-void authfail_report()
+void authfail_report(void)
 {
 
    FILE *fp_in = NULL, *fp_ou = NULL;
       
    char url[MAXLEN];
-   char html2[MAXLEN];
    char authfail_in[MAXLEN];
    char per[MAXLEN];
    char report[MAXLEN];
@@ -46,6 +46,7 @@ void authfail_report()
    char *str;
    int  z=0;
    int  count=0;
+   int  cstatus;
 
    if(strlen(DataFile) > 0) return;
 
@@ -62,24 +63,31 @@ void authfail_report()
    sprintf(per,"%s/sarg-period",dirname);
    sprintf(report,"%s/authfail.html",dirname);
 
-   sprintf(csort,"sort -b -T %s -k 3,3 -k 5,5 -o '%s' '%s'", TempDir, authfail_in, tmp4);
-   system(csort);
-   unlink(tmp4);
-
    if ((fp_in = fopen(per, "r")) == 0) {
       fprintf(stderr, "SARG: (authfail) %s: %s\n",text[45],per);
       exit(1);
    }
-
-   fgets(period,sizeof(period),fp_in);
+   if (!fgets(period,sizeof(period),fp_in)) {
+      fprintf(stderr,"SARG: (authfail) read error in %s\n",per);
+      exit(1);
+   }
    fclose(fp_in);
 
-   if((fp_in=fopen(authfail_in,"r"))==NULL) {
-     fprintf(stderr, "SARG: (authfail) %s: %s\n",text[45],authfail_in);
-     exit(1);
+   sprintf(csort,"sort -b -T \"%s\" -k 3,3 -k 5,5 -o \"%s\" \"%s\"", TempDir, authfail_in, tmp4);
+   cstatus=system(csort);
+   if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
+      fprintf(stderr, "SARG: sort command return status %d\n",WEXITSTATUS(cstatus));
+      fprintf(stderr, "SARG: sort command: %s\n",csort);
+      exit(1);
    }
+   if((fp_in=MY_FOPEN(authfail_in,"r"))==NULL) {
+      fprintf(stderr, "SARG: (authfail) %s: %s\n",text[45],authfail_in);
+      fprintf(stderr, "SARG: sort command: %s\n",csort);
+      exit(1);
+   }
+   unlink(tmp4);
 
-   if((fp_ou=fopen(report,"w"))==NULL) {
+   if((fp_ou=MY_FOPEN(report,"w"))==NULL) {
      fprintf(stderr, "SARG: (authfail) %s: %s\n",text[45],report);
      exit(1);
    }
@@ -89,35 +97,32 @@ void authfail_report()
    fputs("</head>\n",fp_ou);
    if(strlen(FontFace) > 0) fprintf(fp_ou,"<font face=%s>\n",FontFace);
    fprintf(fp_ou,"<body bgcolor=%s text=%s background='%s'>\n",BgColor,TxColor,BgImage);
-   if(strlen(LogoImage) > 0) fprintf(fp_ou, "<center><table cellpadding=\"0\" cellspacing=\"0\">\n<tr><th class=\"logo\"><img src='%s' border=0 align=absmiddle width=%s height=%s>&nbsp;%s</th></tr>\n<tr><td height=\"5\"></td></tr>\n</table>\n",LogoImage,Width,Height,LogoText);
+   write_logo_image(fp_ou);
 
    if(strcmp(IndexTree,"date") == 0)
       show_sarg(fp_ou, "../../..");
    else
       show_sarg(fp_ou,"..");
-   fputs("<center><table cellpadding=0 cellspacing=0>\n",fp_ou);
-   sprintf(url,"<tr><th class=\"title\">%s</th></tr>\n",Title);
-   fputs(url,fp_ou);
+   fputs("<div align=\"center\"><table cellpadding=0 cellspacing=0>\n",fp_ou);
+   fprintf(fp_ou,"<tr><th class=\"title\">%s</th></tr>\n",Title);
 
-   sprintf(url,"<tr><td class=\"header\">%s: %s</td></tr>\n",text[89],period);
-   fputs(url,fp_ou);
-   sprintf(url,"<tr><th class=\"header3\">%s</th></tr>\n",text[117]);
-   fputs(url,fp_ou);
-   fputs("</table></center>\n",fp_ou);
+   fprintf(fp_ou,"<tr><td class=\"header\">%s: %s</td></tr>\n",text[89],period);
+   fprintf(fp_ou,"<tr><th class=\"header3\">%s</th></tr>\n",text[117]);
+   fputs("</table></div>\n",fp_ou);
 
-   fputs("<center><table cellpadding=0 cellspacing=2>\n",fp_ou);
+   fputs("<div align=\"center\"><table cellpadding=\"0\" cellspacing=\"2\">\n",fp_ou);
    fputs("<tr><td></td></tr>\n",fp_ou);
    fputs("<tr><td></td></tr>\n",fp_ou);
    fputs("<tr><td></td></tr>\n",fp_ou);
-   sprintf(url,"<tr><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th></tr>\n",text[98],text[111],text[110],text[91]);
-   fputs(url,fp_ou);
+   fprintf(fp_ou,"<tr><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th></tr>\n",text[98],text[111],text[110],text[91]);
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-      getword(data,buf,' ');
-      getword(hora,buf,' ');
-      getword(user,buf,' ');
-      getword(ip,buf,' ');
-      getword(url,buf,' ');
+      if (getword_multisep(data,sizeof(data),buf,' ')<0 || getword_multisep(hora,sizeof(hora),buf,' ')<0 ||
+          getword_multisep(user,sizeof(user),buf,' ')<0 || getword_multisep(ip,sizeof(ip),buf,' ')<0 ||
+          getword_multisep(url,sizeof(url),buf,' ')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",authfail_in);
+         exit(1);
+      }
 
       if((str=(char *) strstr(user, "_")) != (char *) NULL ) {
          if((str=(char *) strstr(str+1, "_")) != (char *) NULL )
@@ -125,7 +130,7 @@ void authfail_report()
       }
 
       if(strcmp(Ip2Name,"yes") == 0) 
-         ip2name(ip);
+         ip2name(ip,sizeof(ip));
 
       if(!z) {
          strcpy(ouser,user);
@@ -142,23 +147,10 @@ void authfail_report()
             strcpy(oip,ip);
       }
 
-      if(UserTabFile[0] != '\0') {
-         sprintf(warea,":%s:",user);
-         if((str=(char *) strstr(userfile,warea)) != (char *) NULL ) {
-            z1=0;
-            str2=(char *) strstr(str+1,":");
-            str2++;
-            bzero(name, MAXLEN);
-            while(str2[z1] != ':') {
-               name[z1]=str2[z1];
-               z1++;
-            }
-         } else strcpy(name,user);
-      } else strcpy(name,user);
+      get_usertab_name(user,name,sizeof(name));
 
-      if(dotinuser && strstr(name,"_")) {
-         str2=(char *)subs(name,"_",".");
-         strcpy(name,str2);
+      if(dotinuser && strchr(name,'_')) {
+         subs(name,sizeof(name),"_",".");
       }
 
      if(AuthfailReportLimit) {
@@ -172,8 +164,7 @@ void authfail_report()
             continue;
       }
 
-      sprintf(html2,"<tr><td class=\"data2\">%s</td><td class=\"data2\">%s</td><td class=\"data2\">%s-%s</td><td class=\"data2\">%s<a href=\"%s\">%s</a></td></th>\n",name,ip,data,hora,BlockImage,url,url);
-      fputs(html2,fp_ou);
+      fprintf(fp_ou,"<tr><td class=\"data2\">%s</td><td class=\"data2\">%s</td><td class=\"data2\">%s-%s</td><td class=\"data2\">%s<a href=\"%s\">%s</a></td></th>\n",name,ip,data,hora,BlockImage,url,url);
    }
 
    fputs("</table>\n",fp_ou);
