@@ -1,6 +1,6 @@
 /*
  * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2008
+ *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
  *
  * SARG donations:
@@ -24,8 +24,9 @@
  */
 
 #include "include/conf.h"
+#include "include/defs.h"
 
-void ccharset()
+void ccharset(char *CharSet)
 {
    if(strcmp(CharSet,"Latin2") == 0) strcpy(CharSet,"ISO-8859-2");
    else if(strcmp(CharSet,"Latin3") == 0) strcpy(CharSet,"ISO-8859-3");
