@@ -1,6 +1,6 @@
 /*
  * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2008
+ *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
  *
  * SARG donations:
@@ -24,17 +24,15 @@
  */
 
 #include "include/conf.h"
+#include "include/defs.h"
 
-void dansguardian_report()
+void dansguardian_report(void)
 {
 
    FILE *fp_in = NULL, *fp_ou = NULL;
       
    char url[MAXLEN];
-   char html[MAXLEN];
-   char html2[MAXLEN];
    char dansguardian_in[MAXLEN];
-   char dansguardian_ou[MAXLEN];
    char per[MAXLEN];
    char report[MAXLEN];
    char period[100];
@@ -46,7 +44,6 @@ void dansguardian_report()
    char date[15];
    char date2[15];
    char hour[15];
-   char ftime[128];
    char *str;
    int  z=0;
    int  count=0;
@@ -67,15 +64,18 @@ void dansguardian_report()
       exit(1);
    }
 
-   fgets(period,sizeof(period),fp_in);
+   if (!fgets(period,sizeof(period),fp_in)) {
+      fprintf(stderr,"SARG: (dansguardian_report) read error in %s\n",per);
+      exit(1);
+   }
    fclose(fp_in);
 
-   if((fp_in=fopen(dansguardian_in,"r"))==NULL) {
+   if((fp_in=MY_FOPEN(dansguardian_in,"r"))==NULL) {
      fprintf(stderr, "SARG: (dansguardian_report) %s: %s\n",text[8],dansguardian_in);
      exit(1);
    }
 
-   if((fp_ou=fopen(report,"w"))==NULL) {
+   if((fp_ou=MY_FOPEN(report,"w"))==NULL) {
      fprintf(stderr, "SARG: (dansguardian_report) %s: %s\n",text[8],report);
      exit(1);
    }
@@ -85,34 +85,30 @@ void dansguardian_report()
    fputs("</head>\n",fp_ou);
    if(strlen(FontFace) > 0) fprintf(fp_ou,"<font face=%s>\n",FontFace);
    fprintf(fp_ou,"<body bgcolor=%s text=%s background='%s'>\n",BgColor,TxColor,BgImage);
-   if(strlen(LogoImage) > 0) fprintf(fp_ou, "<center><table cellpadding=\"0\" cellspacing=\"0\">\n<tr><th class=\"logo\"><img src='%s' border=0 align=absmiddle width=%s height=%s>&nbsp;%s</th></tr>\n<tr><td height=\"5\"></td></tr>\n</table>\n",LogoImage,Width,Height,LogoText);
-   
+   write_logo_image(fp_ou);
+
    if(strcmp(IndexTree,"date") == 0)
       show_sarg(fp_ou, "../../..");
    else
       show_sarg(fp_ou,"..");
-   fputs("<center><table cellpadding=0 cellspacing=0>\n",fp_ou);
-   sprintf(url,"<tr><th class=\"title\">%s</th></tr>\n",Title);
-   fputs(url,fp_ou);
+   fputs("<div align=\"center\"><table cellpadding=\"0\" cellspacing=\"0\">\n",fp_ou);
+   fprintf(fp_ou,"<tr><th class=\"title\">%s</th></tr>\n",Title);
 
-   sprintf(url,"<tr><td class=\"header\">%s: %s</td></tr>\n",text[89],period);
-   fputs(url,fp_ou);
-   sprintf(url,"<tr><th class=\"header3\">%s</th></tr>\n",text[128]);
-   fputs(url,fp_ou);
-   fputs("</table></center>\n",fp_ou);
+   fprintf(fp_ou,"<tr><td class=\"header\">%s: %s</td></tr>\n",text[89],period);
+   fprintf(fp_ou,"<tr><th class=\"header3\">%s</th></tr>\n",text[128]);
+   fputs("</table></div>\n",fp_ou);
 
-   fputs("<center><table cellpadding=1 cellspacing=2>\n",fp_ou);
+   fputs("<div align=\"center\"><table cellpadding=\"1\" cellspacing=\"2\">\n",fp_ou);
    fputs("<tr><td></td></tr>\n",fp_ou);
-   sprintf(url,"<tr><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th></tr>\n",text[98],text[111],text[110],text[91],text[129]);
-   fputs(url,fp_ou);
+   fprintf(fp_ou,"<tr><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th></tr>\n",text[98],text[111],text[110],text[91],text[129]);
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-      getword(user,buf,' ');
-      getword(date2,buf,' ');
-      getword(hour,buf,' ');
-      getword(ip,buf,' ');
-      getword(url,buf,' ');
-      getword(rule,buf,'\n');
+      if (getword(user,sizeof(user),buf,' ')<0 || getword(date2,sizeof(date2),buf,' ')<0 ||
+          getword(hour,sizeof(hour),buf,' ')<0 || getword(ip,sizeof(ip),buf,' ')<0 ||
+          getword(url,sizeof(url),buf,' ')<0 || getword(rule,sizeof(rule),buf,'\n')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",dansguardian_in);
+         exit(1);
+      }
 
       if(strcmp(UserIp,"yes") == 0)
            strcpy(user,ip);
@@ -138,7 +134,7 @@ void dansguardian_report()
       }
 
       if(strcmp(Ip2Name,"yes") == 0) 
-         ip2name(ip);
+         ip2name(ip,sizeof(ip));
 
       if(!z) {
          strcpy(ouser,user);
@@ -155,23 +151,10 @@ void dansguardian_report()
             strcpy(oip,ip);
       }
 
-      if(UserTabFile[0] != '\0') {
-         sprintf(warea,":%s:",user);
-         if((str=(char *) strstr(userfile,warea)) != (char *) NULL ) {
-            z1=0;
-            str2=(char *) strstr(str+1,":");
-            str2++;
-            bzero(name, MAXLEN);
-            while(str2[z1] != ':') {
-               name[z1]=str2[z1];
-               z1++;
-            }
-         } else strcpy(name,user);
-      } else strcpy(name,user);
+      get_usertab_name(user,name,sizeof(name));
 
-      if(dotinuser && strstr(name,"_")) {
-         str2=(char *)subs(name,"_",".");
-         strcpy(name,str2);
+      if(dotinuser && strchr(name,'_')) {
+         subs(name,sizeof(name),"_",".");
       }
 
      if(DansGuardianReportLimit) {
@@ -185,8 +168,7 @@ void dansguardian_report()
             continue;
       }
 
-      sprintf(html2,"<tr><td class=\"data2\" nospaw>%s</td><td class=\"data2\" nospaw>%s</td><td class=\"data2\" nospaw>%s-%s</td><td class=\"data2\" nospaw><a href=\"http://%s\">%s</a></td><td class=\"data2\" nospaw>%s</td></tr>\n",name,ip,date,hour,url,url,rule);
-      fputs(html2,fp_ou);
+      fprintf(fp_ou,"<tr><td class=\"data2\" nospaw>%s</td><td class=\"data2\" nospaw>%s</td><td class=\"data2\" nospaw>%s-%s</td><td class=\"data2\" nospaw><a href=\"http://%s\">%s</a></td><td class=\"data2\" nospaw>%s</td></tr>\n",name,ip,date,hour,url,url,rule);
    }
 
    fputs("</table>\n",fp_ou);
