@@ -1,6 +1,6 @@
 /*
  * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2008
+ *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
  *
  * SARG donations:
@@ -24,15 +24,14 @@
  */
 
 #include "include/conf.h"
+#include "include/defs.h"
 
-void download_report()
+void download_report(void)
 {
 
    FILE *fp_in = NULL, *fp_ou = NULL;
-      
+
    char url[MAXLEN];
-   char html[MAXLEN];
-   char html2[MAXLEN];
    char report_in[MAXLEN];
    char wdirname[MAXLEN];
    char report[MAXLEN];
@@ -47,8 +46,10 @@ void download_report()
    char *str;
    int  z=0;
    int  count=0;
+   int i;
 
    ouser[0]='\0';
+   ouser2[0]='\0';
 
    sprintf(report_in,"%s/sarg/download.log",TempDir);
    if(access(report_in, R_OK) != 0)
@@ -64,65 +65,61 @@ void download_report()
       exit(1);
    }
 
-   fgets(period,sizeof(period),fp_in);
+   if (!fgets(period,sizeof(period),fp_in)) {
+      fprintf(stderr,"SARG: (download) read error in %s\n",wdirname);
+      exit(1);
+   }
    fclose(fp_in);
 
-   if((fp_in=fopen(report_in,"r"))==NULL) {
+   if((fp_in=MY_FOPEN(report_in,"r"))==NULL) {
      fprintf(stderr, "SARG: (download) %s: %s\n",text[8],report_in);
      exit(1);
    }
 
-   if((fp_ou=fopen(report,"w"))==NULL) {
+   if((fp_ou=MY_FOPEN(report,"w"))==NULL) {
      fprintf(stderr, "SARG: (download) %s: %s\n",text[8],report);
      exit(1);
    }
 
-   fputs("<html>\n",fp_ou);
-   fputs("<head>\n",fp_ou);
-   sprintf(html,"  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\">\n",CharSet);
-   fputs(html,fp_ou);
+   fprintf(fp_ou, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n<html>\n<head>\n  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\">\n",CharSet);
    css(fp_ou);
    fputs("</head>\n",fp_ou);
 
-   sprintf(url,"<body bgcolor=%s text=%s background='%s'>\n",BgColor,TxColor,BgImage);
-   fputs(url,fp_ou);
+   fprintf(fp_ou,"<body bgcolor=\"%s\" text=\"%s\" background=\"%s\">\n",BgColor,TxColor,BgImage);
 
-   if(strlen(LogoImage) > 0) fprintf(fp_ou, "<center><table cellpadding=\"0\" cellspacing=\"0\">\n<tr><th class=\"logo\"><img src='%s' border=0 align=absmiddle width=%s height=%s>&nbsp;%s</th></tr>\n<tr><td height=\"5\"></td></tr>\n</table>\n",LogoImage,Width,Height,LogoText);
+   write_logo_image(fp_ou);
 
    if(strcmp(IndexTree,"date") == 0)
       show_sarg(fp_ou, "../../..");
    else
       show_sarg(fp_ou, "..");
-      
-   fputs("<center><table cellpadding=0 cellspacing=0>\n",fp_ou);
-   sprintf(url,"<tr><th class=\"title\">%s</b></th></tr>\n",Title);
-   fputs(url,fp_ou);
 
-   sprintf(url,"<tr><td class=\"header\">%s: %s</td></tr>\n",text[89],period);
-   fputs(url,fp_ou);
-   sprintf(url,"<tr><th class=\"header3\">%s</th></tr>\n",text[125]);
-   fputs(url,fp_ou);
-   fputs("</table></center>\n",fp_ou);
+   fputs("<div align=\"center\"><table cellpadding=\"0\" cellspacing=\"0\">\n",fp_ou);
+   fprintf(fp_ou,"<tr><th class=\"title\">%s</th></tr>\n",Title);
 
-   fputs("<center><table cellpadding=0 cellspacing=2>\n",fp_ou);
+   fprintf(fp_ou,"<tr><td class=\"header\">%s: %s</td></tr>\n",text[89],period);
+   fprintf(fp_ou,"<tr><th class=\"header3\">%s</th></tr>\n",text[125]);
+   fputs("</table></div>\n",fp_ou);
+
+   fputs("<div align=\"center\"><table cellpadding=\"0\" cellspacing=\"2\">\n",fp_ou);
    fputs("<tr><td></td></tr>\n",fp_ou);
-   sprintf(url,"<tr><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th></tr>\n",text[98],text[111],text[110],text[91]);
-   fputs(url,fp_ou);
+   fprintf(fp_ou,"<tr><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th></tr>\n",text[98],text[111],text[110],text[91]);
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-      getword(data,buf,' ');
-      getword(hora,buf,' ');
-      getword(user,buf,' ');
-      getword(ip,buf,' ');
-      getword(url,buf,' ');
-  
+      if (getword(data,sizeof(data),buf,' ')<0 || getword(hora,sizeof(hora),buf,' ')<0 ||
+          getword(user,sizeof(user),buf,' ')<0 || getword(ip,sizeof(ip),buf,' ')<0 ||
+          getword(url,sizeof(url),buf,' ')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",report_in);
+         exit(1);
+      }
+
       if((str=(char *) strstr(user, "_")) != (char *) NULL ) {
          if((str=(char *) strstr(str+1, "_")) != (char *) NULL )
             fixip(user);
       }
 
-      if(strcmp(Ip2Name,"yes") == 0) 
-         ip2name(ip);
+      if(strcmp(Ip2Name,"yes") == 0)
+         ip2name(ip,sizeof(ip));
 
       if(!z) {
          strcpy(ouser,user);
@@ -133,34 +130,21 @@ void download_report()
             user[0]='\0';
          if(user[0] != '\0')
             strcpy(ouser,user);
-         if(strcmp(oip,ip) == 0) 
+         if(strcmp(oip,ip) == 0)
             ip[0]='\0';
          if(ip[0] != '\0')
             strcpy(oip,ip);
       }
 
-      if(UserTabFile[0] != '\0') {
-         sprintf(warea,":%s:",user);
-         if((str=(char *) strstr(userfile,warea)) != (char *) NULL ) {
-            z1=0;
-            str2=(char *) strstr(str+1,":");
-            str2++;
-            bzero(name, MAXLEN);
-            while(str2[z1] != ':') {
-               name[z1]=str2[z1];
-               z1++;
-            }
-         } else strcpy(name,user);
-      } else strcpy(name,user);
+      get_usertab_name(user,name,sizeof(name));
 
-      if(dotinuser && strstr(name,"_")) {
-         str2=(char *)subs(name,"_",".");
-         strcpy(name,str2);
+      if(dotinuser && strchr(name,'_')) {
+         subs(name,sizeof(name),"_",".");
       }
 
       if(DownloadReportLimit) {
          if(strcmp(ouser2,name) == 0) {
-            count++; 
+            count++;
          } else {
             count=1;
             strcpy(ouser2,name);
@@ -169,18 +153,20 @@ void download_report()
             continue;
       }
 
-      if(strlen(BlockIt) > 0)
-         sprintf(BlockImage,"<a href=\"%s%s?url=%s\"><img src=\"%s/sarg-squidguard-block.png\" border=\"0\"></a>&nbsp;",wwwDocumentRoot,BlockIt,url,ImageFile);
-      else BlockImage[0]='\0';
+      for (i=strlen(url)-1 ; i>=0 && (unsigned char)url[i]<' ' ; i--) url[i]=0;
 
-      sprintf(html2,"<tr><td class=\"data\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s-%s</td><td class=\"data2\">%s<a href=\"%s\">%s</a></td></th>\n",name,ip,data,hora,BlockImage,url,url);
-      fputs(html2,fp_ou);
+      fprintf(fp_ou,"<tr><td class=\"data\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s-%s</td><td class=\"data2\">",name,ip,data,hora);
+      if(BlockIt[0]!='\0')
+         fprintf(fp_ou,"<a href=\"%s%s?url=%s\"><img src=\"%s/sarg-squidguard-block.png\" border=\"0\"></a>&nbsp;",wwwDocumentRoot,BlockIt,url,ImageFile);
+      fprintf(fp_ou,"<a href=\"%s\">",url);
+      output_html_string(fp_ou,url);
+      fputs("</a></td></tr>\n",fp_ou);
    }
 
    fputs("</table>\n",fp_ou);
 
    show_info(fp_ou);
-   fputs("</html>\n",fp_ou);
+   fputs("</body>\n</html>\n",fp_ou);
 
    fclose(fp_in);
    fclose(fp_ou);
