@@ -24,30 +24,37 @@
  */
 
 #include "include/conf.h"
+#include "include/defs.h"
 
-void htmlrel()
+void htmlrel(void)
 {
    DIR *dirp;
-   FILE *fp_in, *fp_ou, *fp_ip, *fp_ip2, *fp_usr, *fp_tt;
+   FILE *fp_in, *fp_ou, *fp_ip, *fp_ip2, *fp_usr;
 
    struct dirent *direntp;
    long long int nnbytes=0, unbytes=0, tnbytes=0, totbytes=0, totbytes2=0;
    long long int totelap=0, totelap2=0, nnelap=0, unelap=0, tnelap=0;
    long long int incache=0, oucache=0, tnincache=0, tnoucache=0, twork=0, twork2=0;
    char arqin[MAXLEN], arqou[MAXLEN], arqper[MAXLEN], arqip[MAXLEN];
-   char nacc[20], nbytes[20], url[1024], purl[1024], tmsg[50], tmsg2[50], nelap[20], csort[MAXLEN];
+   char nacc[20], nbytes[20], url[MAXLEN], tmsg[50], nelap[20], csort[MAXLEN];
    char period[MAXLEN], usuario[MAXLEN], wusuario[MAXLEN], u2[MAXLEN], duser[MAXLEN];
    char userbytes[20], userelap[20], userurl[1024], userhora[9], userdia[9];
    char user_ip[MAXLEN], olduserip[MAXLEN], tmp2[MAXLEN], tmp3[MAXLEN], incac[20], oucac[20];
    char denied_report[255], name2[MAXLEN];
-   char ttd1[3], ttd2[3], ttd3[5], ttt1[3], ttt2[3], ttt3[3];
+   //char ttd1[3], ttd2[3], ttd3[5], ttt1[3], ttt2[3], ttt3[3];
    char *str;
-   char ftime[128], warea[MAXLEN];
+   char warea[MAXLEN];
    char wtemp[MAXLEN], totuser[8];
    long long int tnacc=0, ttnacc=0, unacc=0;
    float perc=0, perc2=0, ouperc=0, inperc=0;
    char *s;
-   int  x, count;
+   int count;
+   int cstatus;
+   const char txtext[]=".txt";
+   int dlen;
+   char href2[MAXLEN];
+   char urly[MAXLEN];
+   char siteind[MAXLEN];
 
    if(strstr(ReportType,"users_sites") == 0) return;
 
@@ -65,7 +72,10 @@ void htmlrel()
       exit(1);
    }
 
-   fgets(period,sizeof(period),fp_in);
+   if (!fgets(period,sizeof(period),fp_in)) {
+      fprintf(stderr,"SARG: (html1) read error in %s\n",arqper);
+      exit(1);
+   }
    fclose(fp_in);
 
    strcpy(arqper,dirname);
@@ -78,16 +88,22 @@ void htmlrel()
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
       if(strstr(buf,"TOTAL") == 0) {
-         getword(wtemp,buf,' ');
+         if (getword(wtemp,sizeof(wtemp),buf,' ')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",arqper);
+            exit(1);
+         }
          ttnacc+=my_atoll(buf);
-         getword(wtemp,buf,' ');
-         getword(wtemp,buf,' ');
+         if (getword(wtemp,sizeof(wtemp),buf,' ')<0 || getword(wtemp,sizeof(wtemp),buf,' ')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",arqper);
+            exit(1);
+         }
          totbytes+=my_atoll(wtemp);
-         getword(wtemp,buf,' ');
-         getword(wtemp,buf,' ');
-         getword(wtemp,buf,' ');
-         getword(wtemp,buf,' ');
-         getword(wtemp,buf,' ');
+         if (getword(wtemp,sizeof(wtemp),buf,' ')<0 || getword(wtemp,sizeof(wtemp),buf,' ')<0 ||
+             getword(wtemp,sizeof(wtemp),buf,' ')<0 || getword(wtemp,sizeof(wtemp),buf,' ')<0 ||
+             getword(wtemp,sizeof(wtemp),buf,' ')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",arqper);
+            exit(1);
+         }
          totelap+=my_atoll(wtemp);
       }
    }
@@ -96,28 +112,33 @@ void htmlrel()
 
    dirp = opendir(tmp);
    while ( (direntp = readdir( dirp )) != NULL ) {
-      if(strstr(direntp->d_name,".txt") == 0)
+      dlen=strlen(direntp->d_name)-(sizeof(txtext)-1);
+      if (dlen<0) continue;
+      if(strcmp(direntp->d_name+dlen,txtext) != 0)
          continue;
 
       count=1;
-      strcpy(usuario,direntp->d_name);
-      wusuario[0]='\0';
-   
-      striptxt:
-      getword(warea,usuario,'.');
-      strcpy(denied_report,warea);
-      strcat(wusuario,warea);
-   
-      if(strcmp(usuario,"txt") !=0) {
-         strcat(wusuario,".");
-         goto striptxt;
+      if (dlen>0) {
+         if (dlen>=sizeof(wusuario)) continue;
+         strncpy(wusuario,direntp->d_name,dlen);
+         wusuario[dlen]=0;
+      } else {
+         wusuario[0]=0;
+      }
+      str=strrchr(wusuario,'.');
+      if (str)
+         strcpy(denied_report,str+1);
+      else
+         strcpy(denied_report,wusuario);
+
+      if (snprintf(warea,sizeof(warea),"%s/%s",dirname,wusuario)>=sizeof(warea)) {
+         fprintf(stderr,"SARG: destination directory too long: %s/%s\n",dirname,wusuario);
+         exit(1);
       }
- 
-      sprintf(warea,"%s/%s",dirname,wusuario);
       mkdir(warea,0755);
 
-      report_day(wusuario, iprel, userip);
-      greport_day(wusuario, iprel, userip);
+      report_day(wusuario);
+      greport_day(wusuario);
 
       strcpy(usuario,wusuario);
       strcpy(arqin,tmp);
@@ -131,7 +152,7 @@ void htmlrel()
       strcpy(duser,arqin);
       strcat(duser,"denied_");
       strcat(arqin,direntp->d_name);
- 
+
       if((str=(char *) strstr(denied_report, "_")) != (char *) NULL ) {
          if((str=(char *) strstr(str+1, "_")) != (char *) NULL )
             fixip(denied_report);
@@ -140,115 +161,93 @@ void htmlrel()
       strcat(duser,".html");
       if(access(duser, R_OK) != 0)
          denied_report[0]='\0';
-   
+
+      strcpy(u2,usuario);
+      if(userip)
+         fixip(u2);
+      if(strcmp(Ip2Name,"yes") == 0)
+         ip2name(u2,sizeof(u2));
+
+      get_usertab_name(u2,name2,sizeof(name2));
+
+      if(dotinuser && strchr(name2,'_')) {
+         subs(name2,sizeof(name2),"_",".");
+      }
+
       if ((fp_in = fopen(arqin, "r")) == 0){
          fprintf(stderr, "SARG: (html3) %s: %s\n",text[45],arqin);
          exit(1);
       }
-   
+
       while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-         getword(wtemp,buf,' ');
+         if (getword(wtemp,sizeof(wtemp),buf,' ')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",arqin);
+            exit(1);
+         }
          tnacc+=my_atoll(wtemp);
-         getword(wtemp,buf,' ');
+         if (getword(wtemp,sizeof(wtemp),buf,' ')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",arqin);
+            exit(1);
+         }
          tnbytes+=my_atoll(wtemp);
-         getword(wtemp,buf,' ');
-         getword(wtemp,buf,' ');
-         getword(wtemp,buf,' ');
+         if (getword(wtemp,sizeof(wtemp),buf,' ')<0 || getword(wtemp,sizeof(wtemp),buf,' ')<0 ||
+             getword(wtemp,sizeof(wtemp),buf,' ')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",arqin);
+            exit(1);
+         }
          tnelap+=my_atoll(wtemp);
-         getword(wtemp,buf,' ');
+         if (getword(wtemp,sizeof(wtemp),buf,' ')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",arqin);
+            exit(1);
+         }
          tnincache+=my_atoll(wtemp);
-         getword(wtemp,buf,' ');
+         if (getword(wtemp,sizeof(wtemp),buf,' ')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",arqin);
+            exit(1);
+         }
          tnoucache+=my_atoll(wtemp);
       }
 
-      fclose(fp_in);
-   
-      if ((fp_in = fopen(arqin, "r")) == 0){
-         fprintf(stderr, "SARG: (html4) %s: %s\n",text[45],arqin);
-         exit(1);
-      }
-   
+      rewind(fp_in);
+
       if ((fp_ou = fopen(arqou, "w")) == 0){
          fprintf(stderr, "SARG: (html5) %s: %s\n",text[45],arqou);
          exit(1);
       }
-   
-      fputs("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\"",fp_ou);
-      fputs(" \"http://www.w3.org/TR/html4/loose.dtd\">\n",fp_ou);
+
+      fputs("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n",fp_ou);
       fputs("<html>\n",fp_ou);
       fputs("<head>\n",fp_ou);
-      sprintf(html,"  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\">\n",CharSet);
-      fputs(html,fp_ou);
+      fprintf(fp_ou,"  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\">\n",CharSet);
       css(fp_ou);
       fputs("</head>\n",fp_ou);
 
-      sprintf(buf,"<body style=\"font-family:%s;font-size:%s;background-color:%s; \
+      fprintf(fp_ou,"<body style=\"font-family:%s;font-size:%s;background-color:%s; \
            background-image:url(%s)\">\n",FontFace,TitleFontSize,BgColor,BgImage);
-      fputs(buf,fp_ou);
 
-   if(strlen(LogoImage) > 0) {
-      fputs("<center><table cellpadding=\"0\" cellspacing=\"0\">\n",fp_ou);
-      sprintf(html,"<tr><th class=\"logo\"><img src='%s' border=0 align=absmiddle width=%s height=%s>&nbsp;%s</th></tr>\n",LogoImage,Width,Height,LogoText);
-      fputs(html,fp_ou);
-      fputs("<tr><td height=\"5\"></td></tr>\n",fp_ou);
-      fputs("</table>\n",fp_ou);
-   }
+      write_logo_image(fp_ou);
 
       if(strcmp(IndexTree,"date") == 0)
          show_sarg(fp_ou, "../../../..");
       else
          show_sarg(fp_ou, "../..");
 
-      fputs("<center><table cellpadding=\"0\" cellspacing=\"0\">\n",fp_ou);
-      sprintf(html,"<tr><th class=\"title\" colspan=\"2\">%s</th></tr>\n",Title);
-      fputs(html,fp_ou);
+      fputs("<div align=\"center\"><table cellpadding=\"0\" cellspacing=\"0\">\n",fp_ou);
+      fprintf(fp_ou,"<tr><th class=\"title\" colspan=\"2\">%s</th></tr>\n",Title);
 
-      strcpy(u2,usuario);
-      if(userip){
-         strcpy(u2,usuario);
-         fixip(u2);
-      }
-      if(strcmp(Ip2Name,"yes") == 0)
-         ip2name(u2);
-
-      if(UserTabFile[0] != '\0') {
-         sprintf(warea,":%s:",u2);
-         if((str=(char *) strstr(userfile,warea)) != (char *) NULL ) {
-            z1=0;
-            str2=(char *) strstr(str+1,":");
-            str2++;
-            bzero(name, MAXLEN);
-            while(str2[z1] != ':') {
-               name[z1]=str2[z1];
-               z1++;
-            }
-         } else strcpy(name,u2);
-      } else strcpy(name,u2);
-
-      strcpy(name2,name);
-      if(dotinuser && strstr(name2,"_")) {
-         str2=(char *)subs(name2,"_",".");
-         strcpy(name2,str2);
-      }
-   
-      sprintf(html,"<tr><td class=\"header\" colspan=\"2\">%s:&nbsp;%s</td></tr>\n",text[89],period);
-      fputs(html,fp_ou);
-      sprintf(html,"<tr><td class=\"header\" colspan=\"2\">%s:&nbsp;%s</td></tr>\n",text[90],name2);
-      fputs(html,fp_ou);
-      sprintf(html,"<tr><td class=\"header\" colspan=\"2\">%s:&nbsp;%s, %s</td></tr>\n",text[104],UserSortField,UserSortOrder);
-      fputs(html,fp_ou);
-      sprintf(html,"<tr><td class=\"header3\" colspan=2>%s %s</td></tr>\n",text[32],text[55]);
-      fputs(html,fp_ou);
+      fprintf(fp_ou,"<tr><td class=\"header\" colspan=\"2\">%s:&nbsp;%s</td></tr>\n",text[89],period);
+      fprintf(fp_ou,"<tr><td class=\"header\" colspan=\"2\">%s:&nbsp;%s</td></tr>\n",text[90],name2);
+      fprintf(fp_ou,"<tr><td class=\"header\" colspan=\"2\">%s:&nbsp;%s, %s</td></tr>\n",text[104],UserSortField,UserSortOrder);
+      fprintf(fp_ou,"<tr><td class=\"header3\" colspan=\"2\">%s %s</td></tr>\n",text[32],text[55]);
       fputs("<tr><td></td></tr>\n",fp_ou);
- 
-      fputs("</table></center>\n",fp_ou);
-      fputs("<center><table cellpadding=\"2\" cellspacing=\"1\">\n",fp_ou);
-   
-      if(strlen(denied_report) > 0) {
-         sprintf(html,"<tr><td class=\"header\" colspan=11><a href=\"denied_%s.html\">%s</a> %s</td></tr>\n",denied_report,text[116],text[55]);
-         fputs(html,fp_ou);
+
+      fputs("</table></div>\n",fp_ou);
+      fputs("<div align=\"center\"><table cellpadding=\"2\" cellspacing=\"1\">\n",fp_ou);
+
+      if(denied_report[0]!='\0') {
+         fprintf(fp_ou,"<tr><td class=\"header\" colspan=\"11\"><a href=\"denied_%s.html\">%s</a> %s</td></tr>\n",denied_report,text[116],text[55]);
       }
-   
+
       strcpy(val3,text[92]);
       strcpy(val4,text[93]);
       sprintf(val11,"%%%s",text[93]);
@@ -256,9 +255,9 @@ void htmlrel()
       strcpy(val6,text[94]);
       strcpy(val7,text[95]);
       sprintf(val8,"%%%s",text[99]);
-      strcpy(val9,"colspan=2");
+      strcpy(val9,"colspan=\"2\"");
       bzero(val10, 255);
-      
+
       if(strstr(UserReportFields,"CONNECT") == 0)
          bzero(val3, 255);
       if(strstr(UserReportFields,"BYTES") == 0)
@@ -276,29 +275,48 @@ void htmlrel()
          bzero(val7, 255);
       if(strstr(UserReportFields,"%TIME") == 0)
          bzero(val8, 255);
-   
-      sprintf(html,"<tr><th></th><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header3\" %s>%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th></tr>\n",text[91],val3,val4,val11,val9,val5,val6,val7,val8);
-      fputs(html,fp_ou);
-  
+
+      fprintf(fp_ou,"<tr><th></th><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header3\" %s>%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th></tr>\n",text[91],val3,val4,val11,val9,val5,val6,val7,val8);
+
       if(debug) {
          if(userip) {
             strcpy(u2,usuario);
             fixip(u2);
-            sprintf(msg, "%s: %s",text[61],u2);
+            debuga("%s: %s",text[61],u2);
          } else
-            sprintf(msg, "%s: %s",text[61],usuario);
-            debuga(msg);
+            debuga("%s: %s",text[61],usuario);
       }
 
-      fscanf(fp_in,"%s",nacc);
-      fscanf(fp_in,"%s",nbytes);
-      fscanf(fp_in,"%s",url);
-      fscanf(fp_in,"%s",tmsg);
-      fscanf(fp_in,"%s",nelap);
-      fscanf(fp_in,"%s",incac);
-      fscanf(fp_in,"%s",oucac);
+      while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
+         if (getword(nacc,sizeof(nacc),buf,' ')<0) {
+            printf("SARG: Maybe you have a broken number of access in your %s file.\n",arqin);
+            exit(1);
+         }
+         if (getword(nbytes,sizeof(nbytes),buf,' ')<0) {
+            printf("SARG: Maybe you have a broken number of bytes in your %s file.\n",arqin);
+            exit(1);
+         }
+         if (getword(url,sizeof(url),buf,' ')<0) {
+            printf("SARG: Maybe you have a broken url in your %s file.\n",arqin);
+            exit(1);
+         }
+         if (getword(tmsg,sizeof(tmsg),buf,' ')<0) {
+            printf("SARG: Maybe you have a broken status in your %s file.\n",arqin);
+            exit(1);
+         }
+         if (getword(nelap,sizeof(nelap),buf,' ')<0) {
+            printf("SARG: Maybe you have a broken elapsed time in your %s file.\n",arqin);
+            exit(1);
+         }
+         if (getword(incac,sizeof(incac),buf,' ')<0) {
+            printf("SARG: Maybe you have a broken in cache column in your %s file.\n",arqin);
+            exit(1);
+         }
+         if (getword(oucac,sizeof(oucac),buf,' ')<0) {
+            printf("SARG: Maybe you have a broken not in cache column in your %s file.\n",arqin);
+            exit(1);
+         }
 
-      while(!feof(fp_in)) {
          if(strncmp(tmsg,"OK",2) != 0)
             sprintf(tmsg,"<td class=\"data\">%s</td>",text[46]);
          else bzero(tmsg, 50);
@@ -307,7 +325,7 @@ void htmlrel()
          nnelap=my_atoll(nelap);
          incache=my_atoll(incac);
          oucache=my_atoll(oucac);
-   
+
          if(nnbytes) {
             perc=nnbytes * 100;
             perc=perc / tnbytes;
@@ -320,62 +338,65 @@ void htmlrel()
             perc2=nnelap * 100;
             perc2=perc2 / tnelap;
          } else perc2=0;
- 
+
          if(incache) {
             inperc=incache * 100;
             inperc=inperc / nnbytes;
          } else inperc=0;
-   
+
          if(oucache) {
             ouperc=oucache * 100;
             ouperc=ouperc / nnbytes;
          } else ouperc=0;
-   
+
          twork=my_atoll(nacc);
-         sprintf(wwork1,"%s",fixnum(twork,1));
-         sprintf(wwork2,"%s",fixnum(nnbytes,1));
-         sprintf(wwork3,"%s",fixnum2(nnelap,1));
+         strcpy(wwork1,fixnum(twork,1));
+         strcpy(wwork2,fixnum(nnbytes,1));
+         strcpy(wwork3,fixnum2(nnelap,1));
 
          if(strcmp(LongUrl,"yes") != 0) {
-            getword(warea,url,'/');
-            sprintf(url,"%s",warea);
+            char *endofhost=strchr(url,'/');
+            if (endofhost) *endofhost='\0';
             strcpy(urly,url);
          } else {
             strcpy(urly,url);
             url_module(url, module);
-            getword(warea,url,'/');
+            if (getword(warea,sizeof(warea),url,'/')<0) {
+               printf("SARG: Maybe you have a broken record or garbage in the url of the %s file.\n",arqin);
+               exit(1);
+            }
             sprintf(url,"%s...%s",warea,module);
          }
 
          if(strcmp(Privacy,"yes") == 0)
-            sprintf(httplink,"<font color=%s><href=http://%s>%s",       \
-                    PrivacyStringColor,PrivacyString,PrivacyString);
-          else
-             strcpy(tmp6,"../../images");
-             if(strcmp(IndexTree,"date") == 0)
-                sprintf(tmp6,"../%s",ImageFile);
-             if(strlen(BlockIt) > 0)
-                sprintf(httplink,"<a href=\"%s%s?url=%s\"><img src=\"%s/sarg-squidguard-block.png\" border=\"0\"></a><a href=http://%s>&nbsp;%s</a>",wwwDocumentRoot,BlockIt,urly,tmp6,urly,urly);
-             else
-                sprintf(httplink,"<a href=http://%s title=\"%s\">%s</a>",urly,urly,url);
+            sprintf(httplink,"<span style=\"color:%s;\">%s</span>",PrivacyStringColor,PrivacyString);
+         else {
+            if(strcmp(IndexTree,"date") == 0)
+               sprintf(tmp6,"../%s",ImageFile);
+            else
+               strcpy(tmp6,"../../images");
+            if(BlockIt[0]!='\0')
+               snprintf(httplink,sizeof(httplink),"<a href=\"%s%s?url=%s\"><img src=\"%s/sarg-squidguard-block.png\" border=\"0\"></a><a href=http://%s>&nbsp;%s</a>",wwwDocumentRoot,BlockIt,urly,tmp6,urly,urly);
+            else
+               snprintf(httplink,sizeof(httplink),"<a href=\"http://%s\" title=\"%s\">%s</a>",urly,urly,url);
+         }
 
          if(strstr(ReportType,"site_user_time_date") != 0) {
-            sprintf(ltext110,"%s",text[110]);
-            if(ltext110){
-               for(s=ltext110; *s; ++s)
-                  *s=tolower(*s);
-            }
+            strcpy(ltext110,text[110]);
+            for(s=ltext110; *s; ++s)
+               *s=tolower(*s);
             strcpy(siteind,urly);
             str=siteind;
-            for(z1=0; z1<strlen(str); z1++) {
-	       if(str[z1]=='?' || str[z1]=='-' || str[z1]=='.' || str[z1]==':' || str[z1]=='/' || str[z1]=='\\')
+            for(z1=0; str[z1]; z1++) {
+               if(str[z1]=='?' || str[z1]=='-' || str[z1]=='.' || str[z1]==':' || str[z1]=='/' || str[z1]=='\\' || str[z1]=='*' ||
+                  str[z1]=='\'' || str[z1]=='\"' || str[z1]=='$')
                   str[z1]='_';
             }
-            sprintf(href2,"<a href=\"tt%s-%s.html\"><img src=\"%s/datetime.png\" border=\"0\" title=\"%s %s\"></a>",usuario,siteind,tmp6,ltext110,text[55]);
+            sprintf(href2,"<a href=\"tt%s-%s.html\"><img src=\"%s/datetime.png\" border=\"0\" title=\"%s %s\" alt=\"T\"></a>",usuario,siteind,tmp6,ltext110,text[55]);
          } else {
             bzero(href2, MAXLEN);
             bzero(ltext110, 50);
-         } 
+         }
 
          sprintf(val2,"%s",href2);
          sprintf(val3,"%3.2f%%",perc);
@@ -383,7 +404,7 @@ void htmlrel()
          sprintf(val5,"%3.2f%%",ouperc);
          sprintf(val6,"%s",buildtime(nnelap));
          sprintf(val7,"%3.2f%%",perc2);
-      
+
          if(strstr(UserReportFields,"CONNECT") == 0) {
             bzero(wwork1, 255);
             bzero(hbc1, 30);
@@ -417,79 +438,89 @@ void htmlrel()
          if(strncmp(tmsg," ",1) == 0)
             bzero(hbc9, 30);
 
-         sprintf(html,"<tr><td class=\"data\">%s</td><td class=\"data2\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s</td>%s</tr>\n",val2,httplink,wwork1,wwork2,val3,val4,val5,val6,wwork3,val7,tmsg);
-   
-         if(UserReportLimit) {
-            if(count <= UserReportLimit) {
-               fputs(html,fp_ou);
-               count++;
-            }
-         } else fputs(html,fp_ou);
+
+         if(UserReportLimit<=0 || count<=UserReportLimit) {
+            fprintf(fp_ou,"<tr><td class=\"data\">%s</td><td class=\"data2\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s</td>%s</tr>\n",val2,httplink,wwork1,wwork2,val3,val4,val5,val6,wwork3,val7,tmsg);
+            count++;
+         }
 
          if(iprel) {
             strcpy(arqip,tmp);
             strcat(arqip,"/");
             strcat(arqip,usuario);
             strcat(arqip,".ip");
-   
+
             if ((fp_ip = fopen(arqip, "r")) == 0){
                fprintf(stderr, "SARG: (html6) %s: %s\n",text[45],arqip);
                exit(1);
             }
-   
-            if ((fp_ip2 = fopen(tmp2, "a")) == 0){
+
+            if ((fp_ip2 = MY_FOPEN(tmp2, "a")) == 0){
                fprintf(stderr, "SARG: (html7) %s: %s\n",text[45],tmp2);
                exit(1);
             }
-   
+
             while(fgets(buf,sizeof(buf),fp_ip)!=NULL) {
                if(strstr(buf,url) != 0)
                   fputs(buf,fp_ip2);
             }
-   
+
             fclose(fp_ip);
             fclose(fp_ip2);
 
-            sprintf(csort,"sort -n -T %s -k 1,1 -k 5,5 -o '%s' '%s'",TempDir,tmp3,tmp2);
-            system(csort);
-   
-            if ((fp_ip = fopen(tmp3, "r")) == 0) {
+            sprintf(csort,"sort -n -T \"%s\" -k 1,1 -k 5,5 -o \"%s\" \"%s\"",TempDir,tmp3,tmp2);
+            cstatus=system(csort);
+            if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
+               fprintf(stderr, "SARG: sort command return status %d\n",WEXITSTATUS(cstatus));
+               fprintf(stderr, "SARG: sort command: %s\n",csort);
+               exit(1);
+            }
+
+            if ((fp_ip = MY_FOPEN(tmp3, "r")) == 0) {
                fprintf(stderr, "SARG: (html8) %s: %s\n",text[45],tmp3);
                exit(1);
             }
 
-            fscanf(fp_ip,"%s",user_ip);
-            fscanf(fp_ip,"%s",userurl);
-            fscanf(fp_ip,"%s",userdia);
-            fscanf(fp_ip,"%s",userhora);
-            fscanf(fp_ip,"%s",userbytes);
-            fscanf(fp_ip,"%s",userelap);
-   
-            strcpy(olduserip,user_ip);
-   
-            while(!feof(fp_ip)) {
+            olduserip[0]='\0';
+
+            while(fgets(buf,sizeof(buf),fp_ip)) {
+               if (getword(user_ip,sizeof(user_ip),buf,' ')<0) {
+                  printf("SARG: Maybe you have a broken user IP in your %s file.\n",tmp3);
+                  exit(1);
+               }
+               if (getword(userurl,sizeof(userurl),buf,' ')<0) {
+                  printf("SARG: Maybe you have a broken user URL in your %s file.\n",tmp3);
+                  exit(1);
+               }
+               if (getword(userdia,sizeof(userdia),buf,' ')<0) {
+                  printf("SARG: Maybe you have a broken day in your %s file.\n",tmp3);
+                  exit(1);
+               }
+               if (getword(userhora,sizeof(userhora),buf,' ')<0) {
+                  printf("SARG: Maybe you have a broken time in your %s file.\n",tmp3);
+                  exit(1);
+               }
+               if (getword(userbytes,sizeof(userbytes),buf,' ')<0) {
+                  printf("SARG: Maybe you have a broken size in your %s file.\n",tmp3);
+                  exit(1);
+               }
+               if (getword(userelap,sizeof(userelap),buf,' ')<0) {
+                  printf("SARG: Maybe you have a broken elapsed time in your %s file.\n",tmp3);
+                  exit(1);
+               }
                if(strcmp(user_ip,olduserip) != 0) {
                   my_lltoa(unelap,val2,0);
                   sprintf(wwork1,"%s",fixnum(unbytes,1));
-                  sprintf(html,"<tr><td></td><td class=\"data\">%s</td><td></td><td class=\"data\">%s</td><td></td><td></td><td></td><td class=\"data\">%s</td><td class=\"data\">%s</td></tr>\n",olduserip,wwork1,buildtime(unelap),val2);
-                  fputs(html,fp_ou);
-  
+                  fprintf(fp_ou,"<tr><td></td><td class=\"data\">%s</td><td></td><td class=\"data\">%s</td><td></td><td></td><td></td><td class=\"data\">%s</td><td class=\"data\">%s</td></tr>\n",olduserip,wwork1,buildtime(unelap),val2);
+
                   strcpy(olduserip,user_ip);
                   unacc=0;
                   unbytes=0;
                   unelap=0;
                }
-   
+
                unbytes=unbytes+my_atoll(userbytes);
                unelap=unelap+my_atoll(userelap);
-   
-               fscanf(fp_ip,"%s",user_ip);
-               fscanf(fp_ip,"%s",userurl);
-               fscanf(fp_ip,"%s",userdia);
-               fscanf(fp_ip,"%s",userhora);
-               fscanf(fp_ip,"%s",userbytes);
-               fscanf(fp_ip,"%s",userelap);
-   
             }
 
             fclose(fp_ip);
@@ -499,23 +530,13 @@ void htmlrel()
 
             my_lltoa(unelap,val3,0);
             sprintf(wwork1,"%s",fixnum(unbytes,1));
-            sprintf(html,"<tr><td></td><td class=\"data\">%s</td><td></td><td class=\"data\">%s</td><td></td><td></td><td></td><td class=\"data\">%s</td><td class=\"data\">%s</font></td></tr>\n",olduserip,wwork1,buildtime(unelap),val3);
-            fputs(html,fp_ou);
+            fprintf(fp_ou,"<tr><td></td><td class=\"data\">%s</td><td></td><td class=\"data\">%s</td><td></td><td></td><td></td><td class=\"data\">%s</td><td class=\"data\">%s</td></tr>\n",olduserip,wwork1,buildtime(unelap),val3);
          }
 
          unacc=0;
          unbytes=0;
          unelap=0;
 
-         fscanf(fp_in,"%s",nacc);
-         fscanf(fp_in,"%s",nbytes);
-         fscanf(fp_in,"%s",url);
-         fscanf(fp_in,"%s",tmsg);
-         fscanf(fp_in,"%s",nelap);
-         fscanf(fp_in,"%s",incac);
-         fscanf(fp_in,"%s",oucac);
-//         fscanf(fp_in,"%s",datestimes);
-
       }
 
       if(iprel)
@@ -526,22 +547,22 @@ void htmlrel()
          perc=totbytes / 100;
          perc=tnbytes / perc;
       } else perc=0;
-   
+
       if(tnelap) {
          perc2=totelap / 100;
          perc2=tnelap / perc2;
       } else perc2=0;
-   
+
       if(tnoucache) {
          ouperc=tnoucache * 100;
          ouperc=ouperc / tnbytes;
       } else ouperc=0;
-   
+
       if(tnincache) {
          inperc=tnincache * 100;
          inperc=inperc / tnbytes;
       } else inperc=0;
-   
+
       sprintf(wwork1,"%s",fixnum(tnacc,1));
       sprintf(wwork2,"%s",fixnum(tnbytes,1));
       sprintf(wwork3,"%s",fixnum2(tnelap,1));
@@ -597,12 +618,11 @@ void htmlrel()
       }
 
       if(strstr(UserReportFields,"TOTAL") != 0) {
-         sprintf(html,"<tr><th></th><th %s>%s</th><th %s>%s</th><th %s>%s</th><th %s>%s</th><th %s>%s</th><th %s>%s</th><th %s>%s</th><th %s>%s</font></th><th %s>%s</font></th></tr>\n",hbc9,text[107],hbc1,wwork1,hbc2,wwork2,hbc4,val3,hbc5,val4,hbc6,val5,hbc7,val6,hbc3,wwork3,hbc8,val7);
-         fputs(html,fp_ou);
+         fprintf(fp_ou,"<tr><th></th><th %s>%s</th><th %s>%s</th><th %s>%s</th><th %s>%s</th><th %s>%s</th><th %s>%s</th><th %s>%s</th><th %s>%s</th><th %s>%s</th></tr>\n",hbc9,text[107],hbc1,wwork1,hbc2,wwork2,hbc4,val3,hbc5,val4,hbc6,val5,hbc7,val6,hbc3,wwork3,hbc8,val7);
       }
 
       fclose(fp_in);
-  
+
       if(atoi(PerUserLimit) > 0) {
          if(tnbytes > (atoi(PerUserLimit)*1000000)) {
             limit_flag=0;
@@ -621,20 +641,17 @@ void htmlrel()
                }
                fclose(fp_usr);
             }
-  
+
             if(!limit_flag) {
                if((fp_usr = fopen(PerUserLimitFile, "a")) == 0) {
                   fprintf(stderr, "SARG: (html10) %s: %s\n",text[45],PerUserLimitFile);
                   exit(1);
                }
-               sprintf(html,"%s\n",u2);
-               fputs(html,fp_usr);
+               fprintf(fp_usr,"%s\n",u2);
                fclose(fp_usr);
-    
-               if(debug) {
-                  sprintf(msg, "%s %s %s (%s MB). %s %s",text[32],u2,text[74],PerUserLimit,text[75],PerUserLimitFile);
-                      debuga(msg);
-               }
+
+               if(debug)
+                  debuga("%s %s %s (%s MB). %s %s",text[32],u2,text[74],PerUserLimit,text[75],PerUserLimitFile);
             }
          }
       }
@@ -644,81 +661,83 @@ void htmlrel()
 //	continue;
 //      }
 
-   if (strstr(ReportType,"topuser") != 0) {
-      strcpy(arqper,dirname);
-      strcat(arqper,"/sarg-users");
- 
-      if ((fp_in = fopen(arqper, "r")) == 0){
-         fprintf(stderr, "SARG: (html11) %s: %s\n",text[45],arqper);
-         exit(1);
-      }
+      if (strstr(ReportType,"topuser") != 0) {
+         strcpy(arqper,dirname);
+         strcat(arqper,"/sarg-users");
 
-      fgets(totuser,8,fp_in);
-      fclose(fp_in);
- 
-      totbytes2=totbytes/my_atoll(totuser);
-      totelap2=totelap/my_atoll(totuser);
- 
-      if(totbytes2) {
-         perc = totbytes / 100;
-         perc = totbytes2 / perc;
-      } else perc=0;
-   
-      if(totelap2) {
-         perc2 = totelap / 100;
-         perc2 = totelap2 / perc2;
-      } else perc2=0;
-   
-      twork2=my_atoll(totuser);
-      twork=ttnacc/twork2;
-  
-      sprintf(wwork1,"%s",fixnum(twork,1));
-      sprintf(wwork2,"%s",fixnum(totbytes2,1));
-      sprintf(wwork3,"%s",fixnum2(totelap2,1));
+         if ((fp_in = fopen(arqper, "r")) == 0){
+            fprintf(stderr, "SARG: (html11) %s: %s\n",text[45],arqper);
+            exit(1);
+         }
 
-      sprintf(val6,"%s",buildtime(totelap2));
-      sprintf(val7,"%3.2f%%",perc2); 
-   
-      strcpy(hbc1,"class=\"header2\"");
-      strcpy(hbc2,"class=\"header2\"");
-      strcpy(hbc3,"class=\"header2\"");
-      strcpy(hbc4,"class=\"header2\"");
-      strcpy(hbc5,"class=\"header2\"");
-      strcpy(hbc6,"class=\"header\"");
+         if (!fgets(totuser,8,fp_in)) {
+            fprintf(stderr,"SARG: (html11) read error in %s\n",arqper);
+            exit(1);
+         }
+         fclose(fp_in);
 
-      if(strstr(UserReportFields,"CONNECT") == 0) {
-         bzero(wwork1, 255);
-         bzero(hbc1, 30);
-      }
-      if(strstr(UserReportFields,"BYTES") == 0) {
-         bzero(wwork2, 255);
-         bzero(hbc2, 30);
-      }
-      if(strstr(UserReportFields,"MILISEC") == 0) {
-         bzero(wwork3, 255);
-         bzero(hbc3, 30);
-      }
-      if(strstr(UserReportFields,"USED_TIME") == 0) { 
-         bzero(val6, 255);
-         bzero(hbc4, 30);
-      }
-      if(strstr(UserReportFields,"%TIME") == 0) {
-         bzero(val7, 255);
-         bzero(hbc5, 30);
-      }
-   
-      if(strstr(UserReportFields,"AVERAGE") != 0) {
-         sprintf(html,"<tr><th></th><th %s>%s</th><th %s>%s</th><th %s>%s</th><th></th><th></th><th></th><th %s>%s</th><th %s>%s</font></th><th %s>%s</th></tr>\n",hbc6,text[96],hbc1,wwork1,hbc2,wwork2,hbc4,val6,hbc3,wwork3,hbc5,val7);
-         fputs(html,fp_ou);
-      }
-} /* added */
+         totbytes2=totbytes/my_atoll(totuser);
+         totelap2=totelap/my_atoll(totuser);
+
+         if(totbytes2) {
+            perc = totbytes / 100;
+            perc = totbytes2 / perc;
+         } else perc=0;
+
+         if(totelap2) {
+            perc2 = totelap / 100;
+            perc2 = totelap2 / perc2;
+         } else perc2=0;
+
+         twork2=my_atoll(totuser);
+         twork=ttnacc/twork2;
+
+         strcpy(wwork1,fixnum(twork,1));
+         strcpy(wwork2,fixnum(totbytes2,1));
+         strcpy(wwork3,fixnum2(totelap2,1));
+
+         sprintf(val6,"%s",buildtime(totelap2));
+         sprintf(val7,"%3.2f%%",perc2);
+
+         strcpy(hbc1,"class=\"header2\"");
+         strcpy(hbc2,"class=\"header2\"");
+         strcpy(hbc3,"class=\"header2\"");
+         strcpy(hbc4,"class=\"header2\"");
+         strcpy(hbc5,"class=\"header2\"");
+         strcpy(hbc6,"class=\"header\"");
+
+         if(strstr(UserReportFields,"CONNECT") == 0) {
+            bzero(wwork1, 255);
+            bzero(hbc1, 30);
+         }
+         if(strstr(UserReportFields,"BYTES") == 0) {
+            bzero(wwork2, 255);
+            bzero(hbc2, 30);
+         }
+         if(strstr(UserReportFields,"MILISEC") == 0) {
+            bzero(wwork3, 255);
+            bzero(hbc3, 30);
+         }
+         if(strstr(UserReportFields,"USED_TIME") == 0) {
+            bzero(val6, 255);
+            bzero(hbc4, 30);
+         }
+         if(strstr(UserReportFields,"%TIME") == 0) {
+            bzero(val7, 255);
+            bzero(hbc5, 30);
+         }
+
+         if(strstr(UserReportFields,"AVERAGE") != 0) {
+            fprintf(fp_ou,"<tr><th></th><th %s>%s</th><th %s>%s</th><th %s>%s</th><th></th><th></th><th></th><th %s>%s</th><th %s>%s</th><th %s>%s</th></tr>\n",hbc6,text[96],hbc1,wwork1,hbc2,wwork2,hbc4,val6,hbc3,wwork3,hbc5,val7);
+         }
+      } /* added */
       tnacc=0;
       tnbytes=0;
       tnelap=0;
       tnincache=0;
       tnoucache=0;
 
-      fputs("</center></table>\n",fp_ou);
+      fputs("</table></div>\n",fp_ou);
 
       show_info(fp_ou);
 
@@ -732,6 +751,7 @@ void htmlrel()
 
    (void)rewinddir(dirp);
    (void)closedir(dirp);
+   greport_cleanup();
 
    return;
 }
