@@ -1,94 +1,133 @@
-
+#include "config.h"
 #include "info.h"
 
-#if HAVE_STDIO_H
+#ifdef HAVE_WINDOWS_H
+#include <windows.h>
+#endif
+#ifdef HAVE_STDIO_H
 #include <stdio.h>
 #endif
-#if HAVE_STDLIB_H
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#if HAVE_STRING_H
+#ifdef HAVE_STRING_H
 #include <string.h>
 #endif
-#if HAVE_STRINGS_H
+#ifdef HAVE_STRINGS_H
 #include <strings.h>
 #endif
-#if HAVE_SYS_TIME_H
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
-#if HAVE_TIME_H
+#ifdef HAVE_TIME_H
 #include <time.h>
 #endif
-#if HAVE_SYS_RESOURCE_H
+#ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
 #endif
-#if HAVE_UNISTD_H
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#if HAVE_SYS_DIRENT_H && !HAVE_DIRENT_H
+#if defined(HAVE_SYS_DIRENT_H) && !defined(HAVE_DIRENT_H)
 #include <sys/dirent.h>
 #endif
-#if HAVE_DIRENT_H
+#ifdef HAVE_DIRENT_H
 #include <dirent.h>
 #endif
-#if HAVE_SYS_SOCKET_H
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
-#if HAVE_NETDB_H
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
 #endif
-#if HAVE_TYPES_H
+#ifdef HAVE_TYPES_H
 #include <types.h>
 #endif
-#if HAVE_NETINET_IN_H
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
-#if HAVE_ARPA_INET_H
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
 #endif
-#if HAVE_SYS_STAT_H
+#ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
-#if HAVE_CTYPE_H
+#ifdef HAVE_CTYPE_H
 #include <ctype.h>
 #endif
-#if HAVE_ERRNO_H
+#ifdef HAVE_ERRNO_H
 #include <errno.h>
 #endif
-#if HAVE_GD_H
+#ifdef HAVE_SYS_WAIT_H
+#include <sys/wait.h>
+#endif
+#ifdef HAVE_GD_H
 #include <gd.h>
 #define HAVE_GD
 gdImagePtr im;
 gdPoint points[4];
 #endif
-#if HAVE_GDFONTL_H
+#ifdef HAVE_GDFONTL_H
 #include <gdfontl.h>
 #endif
-#if HAVE_GDFONTT_H
+#ifdef HAVE_GDFONTT_H
 #include <gdfontt.h>
 #endif
-#if HAVE_GDFONTS_H
+#ifdef HAVE_GDFONTS_H
 #include <gdfonts.h>
 #endif
-#if HAVE_GDFONTMB_H
+#ifdef HAVE_GDFONTMB_H
 #include <gdfontmb.h>
 #endif
-#if HAVE_GDFONTG_H
+#ifdef HAVE_GDFONTG_H
 #include <gdfontg.h>
 #endif
+#ifdef HAVE_STDARG_H
+#include <stdarg.h>
+#endif
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+#ifdef HAVE_LIMITS_H
+#include <limits.h>
+#endif
+#ifdef HAVE_WINSOCK_H
+#include <winsock.h>
+#endif
 
-#if HAVE_FOPEN64
+#if defined(HAVE_FOPEN64)
 #define _FILE_OFFSET_BITS 64
-#define MY_FOPEN fopen
+#define MY_FOPEN fopen64
 #else
 #define MY_FOPEN fopen
 #endif
 
+#if !defined(HAVE_BZERO)
+#define bzero(mem,size) memset(mem,0,size)
+#endif
+
+#if defined(IBERTY_LIB) && !defined(HAVE_MKSTEMP)
+int mkstemps(char *template, int suffixlen);
+#define mkstemp(template) mkstemps(template,0)
+#endif
+
+#if defined(__MINGW32__)
+#define mkdir(p,m) _mkdir(p)
+
+#ifndef WIFEXITED
+  #define WIFEXITED(S) 1
+#endif
+
+#ifndef WEXITSTATUS
+  #define WEXITSTATUS(S) (S)
+#endif
+
+#endif /*__MINGW32__*/
 
 #define MAXLEN 20000
 long long int my_atoll (const char *nptr);
 
-FILE *fp_tt; 
+FILE *fp_tt;
 
 char outdir[MAXLEN];
 char dirname[MAXLEN];
@@ -119,9 +158,6 @@ char href3[MAXLEN];
 char df[20];
 char day[3], month[4], year[5];
 char nmonth[30];
-char y1[5], y2[5];
-char d1[3], d2[3];
-char m1[4], m2[4];
 char ltext110[50];
 char cdfrom[30];
 char cduntil[30];
@@ -133,9 +169,9 @@ char OverwriteReport[4];
 char u2[255];
 char RecordsWithoutUser[20];
 char UseComma[4];
-char MailUtility[6];
+char MailUtility[PATH_MAX];
 char TopSitesNum[20];
-char TopUsersNum[20];
+int TopUsersNum;
 char ExcludeCodes[256];
 char TopsitesSortField[15];
 char TopsitesSortType[20];
@@ -214,7 +250,7 @@ char ShowSargLogo[5];
 char ParsedOutputLog[MAXLEN];
 char ParsedOutputLogCompress[255];
 char DisplayedValues[20];
-char HeaderFontSize[4];
+char HeaderFontSize[5];
 char TitleFontSize[5];
 char wwwDocumentRoot[MAXLEN];
 char ExternalCSSFile[MAXLEN];
@@ -238,7 +274,6 @@ char AuthName[512];
 char AuthType[255];
 char Require[512];
 char DownloadSuffix[MAXLEN];
-char *excludecode;
 char *userfile;
 char *str;
 char *str2;
@@ -261,7 +296,6 @@ char ftime[128];
 char mask[MAXLEN];
 char httplink[MAXLEN];
 char html_old[MAXLEN];
-char siteind[MAXLEN];
 char site[MAXLEN];
 char us[50];
 char email[MAXLEN];
@@ -270,8 +304,6 @@ char ouser2[255];
 char user2[MAXLEN];
 char wentp[512];
 char addr[MAXLEN];
-char suffix[10];
-char download_url[MAXLEN];
 char Ulimit[6];
 char RealtimeTypes[1024];
 char cmd[255];
@@ -280,7 +312,6 @@ char tbuf[128];
 char ip[25];
 char RealtimeUnauthRec[15];
 
-int  excode;
 int  idate;
 int  smartfilter;
 int  denied_count;
@@ -297,7 +328,7 @@ int  ttopen;
 int  ind2;
 int  sarglog;
 int  isalog;
-int  dfrom; 
+int  dfrom;
 int  duntil;
 int  dataonly;
 int  indexonly;
@@ -314,12 +345,10 @@ int  SiteUsersReportLimit;
 int  DansGuardianReportLimit;
 int  SquidGuardReportLimit;
 int  UserReportLimit;
-int  download_flag;
 int  dotinuser;
 int  realtime_refresh;
 int  realtime_access_log_lines;
 int  realt;
-int  x, y;
 int  rc;
 int  ntopsites;
 int  nsitesusers;
@@ -327,11 +356,8 @@ int  nrepday;
 int  ndownload;
 int  ntopuser;
 int  squid24;
-long l1, l2;
 float perc;
 
-long long int twork;
-long long int twork2;
 long long int nocost;
 float cost;
 
@@ -342,5 +368,3 @@ typedef struct
 
 DIR *dirp;
 struct dirent *direntp;
-
-int getnumlist( char *, numlist *, const int, const int );
