@@ -1,6 +1,6 @@
 /*
  * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2008
+ *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
  *
  * SARG donations:
@@ -24,8 +24,9 @@
  */
 
 #include "include/conf.h"
+#include "include/defs.h"
 
-void make_index()
+void make_index(void)
 {
 
    FILE *fp_ou, *fp_ou2, *fp_ou3, *fp_tmp, *fp_tmp2, *fp_tmp3;
@@ -46,10 +47,14 @@ void make_index()
    char tbytes[20];
    char media[20];
    char ftime[128];
-   char day[4], mon[4], year[6], hour[10];
+   char day[16], mon[16], year[6], hour[10];
    char h[3], m[3], s[3];
+   int cstatus;
+   char y1[5], y2[5];
+   char d1[3], d2[3];
+   char m1[4], m2[4];
 
-   if(LastLog[0] != '\0') mklastlog(outdir,debug);
+   if(LastLog[0] != '\0') mklastlog(outdir);
 
    sprintf(wdir,"%sindex.html",outdir);
    sprintf(wdir_tmp,"%sindex.unsort",outdir);
@@ -61,9 +66,9 @@ void make_index()
       return;
    }
 
-   if(debug) debuga(text[53]);
+   if(debug) debuga("%s",text[53]);
 
-  // Root dir
+   // Root dir
    dirp = opendir(outdir);
    while ((direntp = readdir( dirp )) != NULL) {
       if(strcmp(IndexTree,"date") == 0) {
@@ -104,8 +109,13 @@ void make_index()
          rename(val4,val3);
          sprintf(val5,"%s/images",val2);
          if(access(val5, R_OK) != 0) {
-            sprintf(val5,"ln -s %simages %s/images",outdir,val2);
-            system(val5);
+            sprintf(val5,"ln -s \"%simages\" \"%s/images\"",outdir,val2);
+            cstatus=system(val5);
+            if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
+               fprintf(stderr, "SARG: command return status %d\n",WEXITSTATUS(cstatus));
+               fprintf(stderr, "SARG: command: %s\n",val5);
+               exit(1);
+            }
          }
       } else {
          if(!isdigit(direntp->d_name[0]) && !isdigit(direntp->d_name[1])) continue;
@@ -122,7 +132,10 @@ void make_index()
                   bzero(newname,512);
                   strcpy(warea,direntp2->d_name);
                   if(strstr(warea,"-") != 0) {
-                     getword(m1,warea,'-');
+                     if (getword_multisep(m1,sizeof(m1),warea,'-')<0) {
+                        printf("SARG: Maybe you have a broken record or garbage in your %s directory.\n",val2);
+                        exit(1);
+                     }
                      strcpy(m2,warea);
                      conv_month_name(m1);
                      conv_month_name(m2);
@@ -133,7 +146,10 @@ void make_index()
                   }
                   strcpy(warea,direntp3->d_name);
                   if(strstr(warea,"-") != 0) {
-                     getword(d1,warea,'-');
+                     if (getword_multisep(d1,sizeof(d1),warea,'-')<0) {
+                        printf("SARG: Maybe you have a broken record or garbage in your %s directory.\n",val2);
+                        exit(1);
+                     }
                      strcpy(d2,warea);
                   } else {
                      strcpy(d1,warea);
@@ -168,7 +184,7 @@ void make_index()
    dirp = opendir(outdir);
    while ((direntp = readdir( dirp )) != NULL) {
       if(strcmp(IndexTree,"date") == 0) {
-         if(strlen(direntp->d_name) > 4 || !isdigit(direntp->d_name[0]) && !isdigit(direntp->d_name[1])) continue;
+         if(strlen(direntp->d_name) > 4 || (!isdigit(direntp->d_name[0]) && !isdigit(direntp->d_name[1]))) continue;
          fprintf(fp_tmp,"%s %s\n",direntp->d_name,get_size(outdir,direntp->d_name));
          continue;
       } else {
@@ -190,16 +206,39 @@ void make_index()
          obtuser(outdir,direntp->d_name,tuser);
          obttotal(outdir,direntp->d_name,tbytes,tuser,media);
          strcpy(html,data);
-         getword(mon,html,' ');
-         getword(mon,html,' ');
-         getword(day,html,' ');
-         getword(hour,html,' ');
-         getword(year,html,' ');
-         getword(year,html,' ');
+         if (getword_multisep(mon,sizeof(mon),html,' ')<0) {
+            printf("SARG: Maybe you have a broken week day in your %s%s/sarg-date file.\n",outdir,direntp->d_name);
+            exit(1);
+         }
+         if (getword_multisep(mon,sizeof(mon),html,' ')<0) {
+            printf("SARG: Maybe you have a broken month in your %s%s/sarg-date file.\n",outdir,direntp->d_name);
+            exit(1);
+         }
+         if (getword_multisep(day,sizeof(day),html,' ')<0) {
+            printf("SARG: Maybe you have a broken day in your %s%s/sarg-date file.\n",outdir,direntp->d_name);
+            exit(1);
+         }
+         if (getword_multisep(hour,sizeof(hour),html,' ')<0) {
+            printf("SARG: Maybe you have a broken time in your %s%s/sarg-date file.\n",outdir,direntp->d_name);
+            exit(1);
+         }
+         if (getword_multisep(year,sizeof(year),html,' ')<0) {
+            printf("SARG: Maybe you have a broken year in your %s%s/sarg-date file.\n",outdir,direntp->d_name);
+            exit(1);
+         }
          strcpy(html,hour);
-         getword(h,html,':');
-         getword(m,html,':');
-         strcpy(s,html);
+         if (getword_multisep(h,sizeof(h),html,':')<0) {
+            printf("SARG: Maybe you have a broken hour in your %s%s/sarg-date file.\n",outdir,direntp->d_name);
+            exit(1);
+         }
+         if (getword_multisep(m,sizeof(m),html,':')<0) {
+            printf("SARG: Maybe you have a broken minute in your %s%s/sarg-date file.\n",outdir,direntp->d_name);
+            exit(1);
+         }
+         if (getword_multisep(s,sizeof(s),html,0)<0) {
+            printf("SARG: Maybe you have a broken second in your %s%s/sarg-date file.\n",outdir,direntp->d_name);
+            exit(1);
+         }
          buildymd(day,mon,year,ftime);
          fprintf(fp_tmp,"%s%s%s%s;%s;%s;%s;%s;%s;%s\n",ftime, h, m, s, direntp->d_name, data, tuser, tbytes, media,newname);
          continue;
@@ -213,22 +252,31 @@ void make_index()
    }
 
    if(strcmp(IndexTree,"date") == 0) {
-      if(strcmp(IndexSortOrder,"A") == 0) sprintf(warea,"sort -k 1,1 '%s' -o '%s'", wdir_tmp, wdir_tmp2);
-      else sprintf(warea,"sort -r -k 1,1 '%s' -o '%s'", wdir_tmp, wdir_tmp2);
-      system(warea);
-      unlink(wdir_tmp);
-      if((fp_tmp=fopen(wdir_tmp2,"r"))==NULL) {
-         fprintf(stderr, "SARG: (index) %s: %s\n",text[45],wdir_tmp2);
+      if(strcmp(IndexSortOrder,"A") == 0) sprintf(warea,"sort -k 1,1 \"%s\" -o \"%s\"", wdir_tmp, wdir_tmp2);
+      else sprintf(warea,"sort -r -k 1,1 \"%s\" -o \"%s\"", wdir_tmp, wdir_tmp2);
+      cstatus=system(warea);
+      if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
+         fprintf(stderr, "SARG: sort command return status %d\n",WEXITSTATUS(cstatus));
+         fprintf(stderr, "SARG: sort command: %s\n",warea);
          exit(1);
       }
+      if((fp_tmp=fopen(wdir_tmp2,"r"))==NULL) {
+         fprintf(stderr, "SARG: (index) %s: %s\n",text[45],wdir_tmp2);
+         fprintf(stderr, "SARG: sort command: %s\n",warea);
+         exit(1);
+      }
+      unlink(wdir_tmp);
       if((fp_ou=fopen(wdir,"w"))==NULL) {
          fprintf(stderr, "SARG: (index) %s: %s\n",text[45],wdir);
          exit(1);
       }
       write_html_header(fp_ou, ".");
       fprintf(fp_ou,"<tr><th %s>%s</th><th %s>%s</th></tr>\n",hbc1,text[130],hbc1,text[132]);
-      while(fgets(wwork1,MAXLEN,fp_tmp)!=NULL) {
-         getword(tmp4,wwork1,' ');
+      while(fgets(wwork1,sizeof(wwork1),fp_tmp)!=NULL) {
+         if (getword_multisep(tmp4,sizeof(tmp4),wwork1,' ')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",wdir_tmp2);
+            exit(1);
+         }
          fprintf(fp_ou,"<tr><td class=\"data2\"><a href=\"%s\">%s</a></td><td class=\"data2\">%s</td></tr>\n",tmp4,tmp4,wwork1);
          sprintf(tmp2,"%s%s",outdir,tmp4);
          sprintf(tmp3,"%s%s/index.unsort",outdir,tmp4);
@@ -246,9 +294,14 @@ void make_index()
          (void)rewinddir(dirp2);
          (void)closedir(dirp2);
          sprintf(wdir_tmp3,"%s%s/index.sort",outdir,tmp4);
-         if(strcmp(IndexSortOrder,"A") == 0) sprintf(csort,"sort -n '%s' -o '%s'", tmp3, wdir_tmp3);
-         else sprintf(csort,"sort -n -r '%s' -o '%s'", tmp3, wdir_tmp3);
-         system(csort);
+         if(strcmp(IndexSortOrder,"A") == 0) sprintf(csort,"sort -n \"%s\" -o \"%s\"", tmp3, wdir_tmp3);
+         else sprintf(csort,"sort -n -r \"%s\" -o \"%s\"", tmp3, wdir_tmp3);
+         cstatus=system(csort);
+         if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
+            fprintf(stderr, "SARG: sort command return status %d\n",WEXITSTATUS(cstatus));
+            fprintf(stderr, "SARG: sort command: %s\n",csort);
+            exit(1);
+         }
          unlink(tmp3);
          if((fp_tmp2=fopen(wdir_tmp3,"r"))==NULL) {
             fprintf(stderr, "SARG: (index) %s: %s\n",text[45],wdir_tmp3);
@@ -262,20 +315,23 @@ void make_index()
          write_html_header(fp_ou2,"..");
          fprintf(fp_ou2,"<tr><th %s>%s/%s</th></tr>\n",hbc1,text[130],text[131]);
          while(fgets(wwork1,MAXLEN,fp_tmp2)!=NULL) {
-            wwork1[strlen(wwork1)-1]='\0';        
+            wwork1[strlen(wwork1)-1]='\0';
             strcpy(tmp5,wwork1);
             if(strstr(tmp5,"-") != 0) {
-               getword(warea,tmp5,'-');
-               name_month(warea);
+               if (getword_multisep(warea,sizeof(warea),tmp5,'-')<0) {
+                  printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",wdir_tmp3);
+                  exit(1);
+               }
+               name_month(warea,sizeof(warea));
                sprintf(tmp6,"%s-",warea);
-               name_month(tmp5);
+               name_month(tmp5,sizeof(tmp5));
                sprintf(nmonth,"%s%s",tmp6,tmp5);
             } else {
                strcpy(nmonth,tmp5);
-               name_month(nmonth);
+               name_month(nmonth,sizeof(nmonth));
             }
             fprintf(fp_ou2,"<tr><td class=\"data2\"><a href=\"%s\">%s %s</a></td></tr>\n",wwork1,tmp4,nmonth);
-         
+
             sprintf(val1,"%s%s/%s",outdir,tmp4,wwork1);
             sprintf(tmp5,"%s%s/%s/index.unsort",outdir,tmp4,wwork1);
             if((fp_ou3=fopen(tmp5,"w"))==NULL) {
@@ -293,9 +349,14 @@ void make_index()
             (void)closedir(dirp3);
             unlink(wdir_tmp3);
             sprintf(tmp6,"%s%s/%s/index.sort",outdir,tmp4,wwork1);
-            if(strcmp(IndexSortOrder,"A") == 0) sprintf(csort,"sort -n '%s' -o '%s'", tmp5, tmp6);
-            else sprintf(csort,"sort -n -r '%s' -o '%s'", tmp5, tmp6);
-            system(csort);
+            if(strcmp(IndexSortOrder,"A") == 0) sprintf(csort,"sort -n \"%s\" -o \"%s\"", tmp5, tmp6);
+            else sprintf(csort,"sort -n -r \"%s\" -o \"%s\"", tmp5, tmp6);
+            cstatus=system(csort);
+            if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
+               fprintf(stderr, "SARG: sort command return status %d\n",WEXITSTATUS(cstatus));
+               fprintf(stderr, "SARG: sort command: %s\n",csort);
+               exit(1);
+            }
             unlink(tmp5);
             sprintf(val2,"%s%s/%s/index.html",outdir,tmp4,wwork1);
             sprintf(val3,"%s/%s",tmp4,wwork1);
@@ -315,8 +376,8 @@ void make_index()
                fprintf(fp_ou3,"<tr><td class=\"data2\"><a href=\"%s\">%s %s %s</a></td></tr>\n",warea,tmp4,nmonth,warea);
             }
             if(fp_tmp3) fclose(fp_tmp3);
-            if(fp_ou3) fclose(fp_ou3);
             write_html_trailer(fp_ou3);
+            if(fp_ou3) fclose(fp_ou3);
             unlink(tmp6);
          }
          write_html_trailer(fp_ou2);
@@ -324,15 +385,23 @@ void make_index()
       }
       (void)rewinddir(dirp);
       (void)closedir(dirp);
-      write_html_trailer(fp_ou);
-      if(fp_ou) fclose(fp_ou);
       if(fp_tmp) fclose(fp_tmp);
       unlink(tmp6);
       unlink(wdir_tmp2);
    } else {
-      if(strcmp(IndexSortOrder,"A") == 0) sprintf(warea,"sort -t';' -k 7,7 -k 1,1 '%s' -o '%s'", wdir_tmp, wdir_tmp2);
-      else sprintf(warea,"sort -r -t';' -k 7,7 -k 1,1 '%s' -o '%s'", wdir_tmp, wdir_tmp2);
-      system(warea);
+      if(strcmp(IndexSortOrder,"A") == 0) sprintf(warea,"sort -t\";\" -k 7,7 -k 1,1 \"%s\" -o \"%s\"", wdir_tmp, wdir_tmp2);
+      else sprintf(warea,"sort -r -t\";\" -k 7,7 -k 1,1 \"%s\" -o \"%s\"", wdir_tmp, wdir_tmp2);
+      cstatus=system(warea);
+      if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
+         fprintf(stderr, "SARG: sort command return status %d\n",WEXITSTATUS(cstatus));
+         fprintf(stderr, "SARG: sort command: %s\n",warea);
+         exit(1);
+      }
+      if((fp_tmp2=fopen(wdir_tmp2,"r"))==NULL) {
+         fprintf(stderr, "SARG: (index) %s: %s\n",text[45],wdir_tmp2);
+         fprintf(stderr, "SARG: sort command: %s\n",warea);
+         exit(1);
+      }
       unlink(wdir_tmp);
       if((fp_ou=fopen(wdir,"w"))==NULL) {
          fprintf(stderr, "SARG: (index) %s: %s\n",text[45],wdir);
@@ -340,28 +409,37 @@ void make_index()
       }
       write_html_header(fp_ou,".");
       fprintf(fp_ou,"<tr><th %s>%s</th><th %s>%s</th><th %s>%s</th><th %s>%s</th><th %s>%s</th></tr>\n",hbc1,text[101],hbc1,text[102],hbc1,text[103],hbc1,text[93],hbc1,text[96]);
-      if((fp_tmp2=fopen(wdir_tmp2,"r"))==NULL) {
-         fprintf(stderr, "SARG: (index) %s: %s\n",text[45],wdir_tmp2);
-         exit(1);
-      }
       while(fgets(buf,MAXLEN,fp_tmp2)!=NULL) {
-         getword(period,buf,';');
-         getword(period,buf,';');
-         getword(data,buf,';');
-         getword(tuser,buf,';');
-         getword(tbytes,buf,';');
-         getword(media,buf,';');
+         if (getword_multisep(period,sizeof(period),buf,';')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",wdir_tmp2);
+            exit(1);
+         }
+         if (getword_multisep(period,sizeof(period),buf,';')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",wdir_tmp2);
+            exit(1);
+         }
+         if (getword_multisep(data,sizeof(data),buf,';')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",wdir_tmp2);
+            exit(1);
+         }
+         if (getword_multisep(tuser,sizeof(tuser),buf,';')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",wdir_tmp2);
+            exit(1);
+         }
+         if (getword_multisep(tbytes,sizeof(tbytes),buf,';')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",wdir_tmp2);
+            exit(1);
+         }
+         if (getword_multisep(media,sizeof(media),buf,';')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",wdir_tmp2);
+            exit(1);
+         }
          fprintf(fp_ou,"<tr><td class=\"data2\"><a href='%s/%s'>%s</a></td><td class=\"data2\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s</td></tr>\n",period,ReplaceIndex,period,data,tuser,tbytes,media);
       }
       if(fp_tmp2) fclose(fp_tmp2);
       unlink(wdir_tmp2);
    }
- 
-   fputs("</table></center>",fp_ou);
 
-   zdate(ftime, DateFormat);
-
-   show_info(fp_ou);
-
-   fputs("</body>\n</html>\n",fp_ou);
+   write_html_trailer(fp_ou);
+   if(fp_ou) fclose(fp_ou);
 }
