@@ -1,6 +1,6 @@
 /*
  * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2008
+ *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
  *
  * SARG donations:
@@ -24,16 +24,15 @@
  */
 
 #include "include/conf.h"
+#include "include/defs.h"
 
 
-void index_only()
+void index_only(const char *dirname,int debug)
 {
-
    DIR *dirp;
    struct dirent *direntp;
    char remove[MAXLEN];
   
-
    dirp = opendir(dirname);
    while ( (direntp = readdir( dirp )) != NULL ){
       if(strcmp(direntp->d_name,".") == 0 || strcmp(direntp->d_name,"..") == 0 || strcmp(direntp->d_name, "index.html") == 0)
@@ -43,7 +42,6 @@ void index_only()
       unlink(remove);
    }
 
-   (void)rewinddir( dirp );
    (void)closedir( dirp );
 
    return;
