@@ -1,6 +1,6 @@
 /*
  * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2008
+ *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
  *
  * SARG donations:
@@ -24,32 +24,49 @@
  */
 
 #include "include/conf.h"
+#include "include/defs.h"
 
-void language_load(char *language)
+void language_load(const char *language)
 {
-
+   char lfile[FILENAME_MAX];
    FILE *fp_text;
+   char buf[MAXLEN];
    int record=0;
 
-   sprintf(warea,"%s/languages/%s",SYSCONFDIR,language);
-
-   if((fp_text=fopen(warea,"r"))==NULL) {
-     fprintf(stderr, "SARG: (language) Cannot open language file: %s\n",warea);
-     exit(1);
+   if (snprintf(lfile,sizeof(lfile),LANGDIR"/%s",language)>=sizeof(lfile)) {
+      fprintf(stderr, "SARG: (language) language name is too long: %s/%s\n",LANGDIR,language);
+      exit(1);
    }
 
-   while(fgets(buf,MAXLEN,fp_text)!=NULL) {
-      getword(warea,buf,'"');
-      getword(warea,buf,'"');
+   if((fp_text=fopen(lfile,"r"))==NULL) {
+     fprintf(stderr, "SARG: (language) Cannot open language file: %s\n",lfile);
+     exit(1);
+   }
+   if(langcode)
+      printf("Reading language from %s\n",lfile);
+
+   while(record<sizeof(text)/sizeof(text[0]) && fgets(buf,sizeof(buf),fp_text)!=NULL) {
+      if (getword(warea,sizeof(warea),buf,'"')<0 || getword(warea,sizeof(warea),buf,'"')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",lfile);
+         exit(1);
+      }
       strcpy(text[record],warea);
 
       if(langcode)
-      printf("%d %s\n",record,warea);
+         printf("%d %s\n",record,warea);
 
       record++;
    }
 
    fclose(fp_text);
 
+   if(langcode) {
+      /*
+      The listing of the language strings is only used when adding new string so there is
+      no need to let sarg run any further than this point.
+      */
+      exit(0);
+   }
+
    return;
 }
