@@ -1,6 +1,6 @@
 /*
  * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2008
+ *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
  *
  * SARG donations:
@@ -24,8 +24,9 @@
  */
 
 #include "include/conf.h"
+#include "include/defs.h"
 
-void mklastlog()
+void mklastlog(const char *outdir)
 {
 
    FILE *fp_in, *fp_ou;
@@ -38,6 +39,7 @@ void mklastlog()
    time_t t;
    struct tm *local;
    struct stat statb;
+   int cstatus;
 
    if(strcmp(LastLog,"0") == 0)
       return;
@@ -58,8 +60,7 @@ void mklastlog()
       t=statb.st_ctime;
       local = localtime(&t);
       strftime(ftime, 127, "%Y%m%d%H%M%S", local);
-      sprintf(buf,"%s %s\n",ftime,direntp->d_name);
-      fputs(buf,fp_ou);
+      fprintf(fp_ou,"%s %s\n",ftime,direntp->d_name);
       ftot++;
    }
 
@@ -67,8 +68,13 @@ void mklastlog()
    (void)closedir( dirp );
    fclose(fp_ou);
    
-   sprintf(buf,"sort -n -k 1,1 -o '%slastlog' '%s'",outdir,temp);
-   system(buf);
+   sprintf(buf,"sort -n -k 1,1 -o \"%slastlog\" \"%s\"",outdir,temp);
+   cstatus=system(buf);
+   if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
+      fprintf(stderr, "SARG: sort command return status %d\n",WEXITSTATUS(cstatus));
+      fprintf(stderr, "SARG: sort command: %s\n",buf);
+      exit(1);
+   }
 
    unlink(temp);
 
@@ -88,17 +94,26 @@ void mklastlog()
    }
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-      getword(warea,buf,' ');
+      if (getword(warea,sizeof(warea),buf,' ')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",temp);
+         exit(1);
+      }
       buf[strlen(buf)-1]='\0';
      
       if(ftot) {
-         if(debug) {
-            sprintf(msg,"%s: %s",text[81],buf);
-            debuga(msg);
-         }
+         if(debug)
+            debuga("%s: %s",text[81],buf);
 //         sprintf(temp,"%s%s",outdir,buf);
-         sprintf(temp,"rm -r %s%s",outdir,buf);
-         system(temp);
+         if (snprintf(temp,sizeof(temp),"rm -r \"%s%s\"",outdir,buf)>=sizeof(temp)) {
+            fprintf(stderr,"SARG: Directory name too long: %s%s\n",outdir,buf);
+            exit(1);
+         }
+         cstatus=system(temp);
+         if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
+            fprintf(stderr, "SARG: command return status %d\n",WEXITSTATUS(cstatus));
+            fprintf(stderr, "SARG: command: %s\n",temp);
+            exit(1);
+         }
          unlink(temp);
          ftot--;
       }
