@@ -24,11 +24,12 @@
  */
 
 #include "include/conf.h"
+#include "include/defs.h"
 
-void getdata(char *, FILE *);
+int getdata(char *, FILE *);
 void datashow(char *);
-void getlog();
-void header();
+static void getlog(void);
+static void header(void);
 
 char dat[128];
 char tim[128];
@@ -36,59 +37,24 @@ char typ[128];
 char ouser[MAXLEN]="";
 char ourl[MAXLEN]="";
 
-void
-realtime(int argc, char *argv[])
+void realtime(void)
 {
 
    getlog();
 
 }
 
-void getlog()
+static void getlog(void)
 {
-   FILE *tmp, *fp, *fp_usr;
+   FILE *tmp, *fp;
    char template1[255]="/var/tmp/sargtpl1.XXXXXX";
    char template2[255]="/var/tmp/sargtpl2.XXXXXX";
    char cmd[512];
-   char buf[512];
-   int  fd1,fd2,nreg;
+   char buf[MAXLEN];
+   int  fd1,fd2;
+   int cstatus;
 
-   if(UserTabFile[0] != '\0') {
-      if(debug) {
-         sprintf(msg,"%s: %s",text[86],UserTabFile);
-         debuga(msg);
-      }
-      if((fp_usr=fopen(UserTabFile,"r"))==NULL) {
-        fprintf(stderr, "SARG: (realtime) %s: %s - %s\n",text[45],UserTabFile,strerror(errno));
-        exit(1);
-      }
-      nreg = lseek(fileno(fp_usr), 0, SEEK_END);
-      lseek(fileno(fp_usr), 0, 0);
-      if((userfile=(char *) malloc(nreg+100))==NULL){
-         fprintf(stderr, "SARG ERROR: %s",text[87]);
-         exit(1);
-      }
-      bzero(userfile,nreg+100);
-      strncat(userfile,":",1);
-      z1=0;
-      z2=1;
-      while(fgets(buf,MAXLEN,fp_usr)!=NULL){
-        buf[strlen(buf)-1]='\0';
-        if(strstr(buf,"\r") != 0) buf[strlen(buf)-1]='\0';
-        getword(bufy,buf,' ');
-        for(z1=0; z1<=strlen(bufy); z1++) {
-           userfile[z2]=bufy[z1];
-           z2++;
-        }
-        strncat(userfile,":",1);
-        for(z1=0; z1<=strlen(buf); z1++) {
-           userfile[z2]=buf[z1];
-           z2++;
-        }
-        strncat(userfile,":",1);
-      }
-      fclose(fp_usr);
-   }
+   read_usertab(UserTabFile);
 
    fd1 = mkstemp(template1);
    fd2 = mkstemp(template2);
@@ -101,42 +67,89 @@ void getlog()
    sprintf(cmd,"tail -%d %s",realtime_access_log_lines,AccessLog);
    fp = popen(cmd, "r");
    while(fgets(buf,sizeof(buf),fp) != NULL )
-      getdata(buf,tmp);
+      if (getdata(buf,tmp)<0) {
+         fprintf(stderr,"SARG: Maybe a broken record or garbage was returned by %s.\n",cmd);
+         exit(1);
+      }
    pclose(fp);
    fclose(tmp);
 
-   sprintf(cmd,"sort -r -k 1,1 -k 2,2 -o %s %s",template2,template1);
-   system(cmd);
+   sprintf(cmd,"sort -r -k 1,1 -k 2,2 -o \"%s\" \"%s\"",template2,template1);
+   cstatus=system(cmd);
+   if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
+      fprintf(stderr, "SARG: sort command return status %d\n",WEXITSTATUS(cstatus));
+      fprintf(stderr, "SARG: sort command: %s\n",cmd);
+      exit(1);
+   }
    unlink(template1);
    datashow(template2);
 }
 
-void getdata(char *rec, FILE *ftmp)
+int getdata(char *rec, FILE *ftmp)
 {
    time_t tt;
    struct tm *t;
 
-   getword3(dat,rec,' ');
-   getword3(warea,rec,' ');
+   if (getword3(dat,sizeof(dat),rec,' ')<0) {
+      fprintf(stderr,"SARG: The time stamp at column 1 is too long.\n");
+      return(-1);
+   }
+   if (getword3(warea,sizeof(warea),rec,' ')<0) {
+      fprintf(stderr,"SARG: The connection duration at column 2 is too long.\n");
+      return(-1);
+   }
    while(strcmp(warea,"") == 0 && strlen(rec) > 0)
-      getword3(warea,rec,' ');
-   getword3(ip,rec,' ');
-   getword3(warea,rec,' ');
-   getword3(warea,rec,' ');
-   getword3(typ,rec,' ');
+      if (getword3(warea,sizeof(warea),rec,' ')<0) {
+         return(-1);
+      }
+   if (getword3(ip,sizeof(ip),rec,' ')<0) {
+      fprintf(stderr,"SARG: The IP address at column 3 is too long.\n");
+      return(-1);
+   }
+   if (getword3(warea,sizeof(warea),rec,' ')<0) {
+      fprintf(stderr,"SARG: The status at column 4 is too long.\n");
+      return(-1);
+   }
+   if (getword3(warea,sizeof(warea),rec,' ')<0) {
+      fprintf(stderr,"SARG: The size at column 5 is too long.\n");
+      return(-1);
+   }
+   if (getword3(typ,sizeof(typ),rec,' ')<0) {
+      fprintf(stderr,"SARG: The action at column 6 is too long.\n");
+      return(-1);
+   }
    if(strncmp(typ,"CONNECT",7) == 0) {
-      getword3(url,rec,' ');
-      getword3(user,rec,' ');
-    }else {
-      getword3(url,rec,'/');
-      getword3(url,rec,'/');
-      getword3(url,rec,'/');
-      getword3(user,rec,' ');
-      getword3(user,rec,' ');
+      if (getword3(url,sizeof(url),rec,' ')<0) {
+         return(-1);
+      }
+      if (getword3(user,sizeof(user),rec,' ')<0) {
+         return(-1);
+      }
+   }else {
+      if (getword3(url,sizeof(url),rec,'/')<0) {
+         fprintf(stderr,"SARG: The URL at column 7 is too long.\n");
+         return(-1);
+      }
+      if (getword3(url,sizeof(url),rec,'/')<0) {
+         fprintf(stderr,"SARG: The URL at column 7 is too long.\n");
+         return(-1);
+      }
+      if (getword3(url,sizeof(url),rec,'/')<0) {
+         fprintf(stderr,"SARG: The URL at column 7 is too long.\n");
+         return(-1);
+      }
+      if (getword3(user,sizeof(user),rec,' ')<0) {
+         fprintf(stderr,"SARG: The data at column 8 is too long.\n");
+         return(-1);
+      }
+      if (getword3(user,sizeof(user),rec,' ')<0) {
+         fprintf(stderr,"SARG: The user at column 9 is too long.\n");
+         return(-1);
+      }
    }
 
    if(strncmp(user,"-",1) == 0 && strcmp(RealtimeUnauthRec,"ignore") == 0)
-      return;
+      return(0);
 
    tt=atoi(dat);
    t=localtime(&tt);
@@ -145,8 +158,8 @@ void getdata(char *rec, FILE *ftmp)
    else if(strncmp(DateFormat,"e",1) == 0)
       strftime(tbuf, 127, "%d-%m-%Y %H:%M", t);
 
-   sprintf(warea,"%s %s %s %s %s\n",tbuf,ip,user,url,typ);
-   fputs(warea,ftmp);
+   fprintf(ftmp,"%s %s %s %s %s\n",tbuf,ip,user,url,typ);
+   return(0);
 }
 
 void datashow(char *tmp)
@@ -163,13 +176,31 @@ void datashow(char *tmp)
 
    while(fgets(buf, MAXLEN, fin)) {
       buf[strlen(buf)-1]='\0';
-      getword3(dat,buf,' ');
-      getword3(tim,buf,' ');
-      getword3(ip,buf,' ');
-      getword3(user,buf,' ');
+      if (getword3(dat,sizeof(dat),buf,' ')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp);
+         exit(1);
+      }
+      if (getword3(tim,sizeof(tim),buf,' ')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp);
+         exit(1);
+      }
+      if (getword3(ip,sizeof(ip),buf,' ')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp);
+         exit(1);
+      }
+      if (getword3(user,sizeof(user),buf,' ')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp);
+         exit(1);
+      }
       if(strlen(dat) < 3 || strlen(user) < 1) continue;
-      getword3(url,buf,' ');
-      getword3(typ,buf,' ');
+      if (getword3(url,sizeof(url),buf,' ')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp);
+         exit(1);
+      }
+      if (getword3(typ,sizeof(typ),buf,' ')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp);
+         exit(1);
+      }
       if(strstr(RealtimeTypes,typ) == 0)
          continue;
 
@@ -178,24 +209,11 @@ void datashow(char *tmp)
 
       strcpy(u2,user);
       if(strcmp(Ip2Name,"yes") == 0)
-         ip2name(u2);
-      if(UserTabFile[0] != '\0') {
-         sprintf(warea,":%s:",u2);
-         if((str=(char *) strstr(userfile,warea)) != (char *) NULL ) {
-            z1=0;
-            str2=(char *) strstr(str+1,":");
-            str2++;
-            bzero(name, MAXLEN);
-            while(str2[z1] != ':') {
-               name[z1]=str2[z1];
-               z1++;
-            }
-         } else strcpy(name,u2);
-      } else strcpy(name,u2);
+         ip2name(u2,sizeof(u2));
+      get_usertab_name(u2,name,sizeof(name));
 
-      if(dotinuser && strstr(name,"_")) {
-         str2=(char *)subs(name,"_",".");
-         strcpy(name,str2);
+      if(dotinuser && strchr(name,'_')) {
+         subs(name,sizeof(name),"_",".");
       }
 
       printf("<tr><td class=\"data\">%s %s</td><td class=\"data3\">%s</td><td class=\"data3\">%s</td><td class=\"data3\">%s</td><td class=\"data2\"><a href=\"http://%s\">%s</td></tr>\n",dat,tim,ip,name,typ,url,url);
@@ -203,14 +221,14 @@ void datashow(char *tmp)
       strcpy(ourl,url);
    }
 
-   puts("</table>\n</html>\n");
+   puts("</table>\n</div>\n</body>\n</html>\n");
    fclose(fin);
    unlink(tmp);
    fflush(NULL);
-   
+
 }
 
-void header()
+static void header(void)
 {
    puts("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\"");
    puts(" \"http://www.w3.org/TR/html4/loose.dtd\">\n");
@@ -223,7 +241,7 @@ void header()
    css(stdout);
    puts("</head>\n");
    printf(buf,"<body style=\"font-family:%s;font-size:%s;background-color:%s;background-image:url(%s)\">\n",FontFace,TitleFontSize,BgColor,BgImage);
-   puts("<center><table cellpadding=\"1\" cellspacing=\"1\">\n");
+   puts("<div align=\"center\"><table cellpadding=\"1\" cellspacing=\"1\">\n");
    printf("<tr><th class=\"title2\" colspan=\"10\">SARG %s</th></tr>\n",text[134]);
    printf("<tr><th class=\"text\" colspan=\"10\">%s: %d s</th></tr>\n",text[136],realtime_refresh);
    printf("<tr><th class=\"header3\">%s</th><th class=\"header3\">%s</th><th class=\"header3\">%s</th><th class=\"header3\">%s</th><th class=\"header\">%s</th></tr>\n",text[110],text[111],text[98],text[135],text[91]);
