@@ -1,6 +1,6 @@
 /*
  * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2008
+ *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
  *
  * SARG donations:
@@ -24,10 +24,11 @@
  */
 
 #include "include/conf.h"
+#include "include/defs.h"
 
 extern numlist hours;
 
-void report_day(char *user, int *iprel, int *ipuser)
+void report_day(const char *user)
 {
 
    FILE *fp_in, *fp_ou;
@@ -70,24 +71,22 @@ void report_day(char *user, int *iprel, int *ipuser)
    fprintf(fp_ou, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n<html>\n<head>\n  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\">\n",CharSet);
    css(fp_ou);
    fputs("</head>\n",fp_ou);
-   if(strlen(FontFace) > 0) fprintf(fp_ou,"<font face=%s>\n",FontFace);
-   fprintf(fp_ou,"<body bgcolor=%s text=%s background='%s'>\n",BgColor,TxColor,BgImage);
-   if(strlen(LogoImage) > 0) fprintf(fp_ou, "<center><table cellpadding=\"0\" cellspacing=\"0\">\n<tr><th class=\"logo\"><img src='%s' border=0 align=absmiddle width=%s height=%s>&nbsp;%s</th></tr>\n<tr><td height=\"5\"></td></tr>\n</table>\n",LogoImage,Width,Height,LogoText);
+   //if(strlen(FontFace) > 0) fprintf(fp_ou,"<font face=%s>\n",FontFace);
+   fprintf(fp_ou,"<body bgcolor=\"%s\" text=\"%s\" background=\"%s\">\n",BgColor,TxColor,BgImage);
+   write_logo_image(fp_ou);
 
    if(strcmp(IndexTree,"date") == 0)
       show_sarg(fp_ou, "../../../..");
    else
       show_sarg(fp_ou, "../..");
 
-   fputs("<center><table cellpadding=0 cellspacing=0>\n",fp_ou);
-   sprintf(html,"<tr><th class=\"title\" colspan=2>%s</th></tr>\n",Title);
-   fputs(html,fp_ou);
-   
-   fputs("</table></center>\n",fp_ou);
-   fputs("<center><table cellpadding=0 cellspacing=0>\n",fp_ou);
+   fputs("<div align=\"center\"><table cellpadding=\"0\" cellspacing=\"0\">\n",fp_ou);
+   fprintf(fp_ou,"<tr><th class=\"title\" colspan=\"2\">%s</th></tr>\n",Title);
 
-   sprintf(html,"<tr><td class=\"header3\" colspan=\"2\">%s:&nbsp;%s</td></tr>\n",text[89],period);
-   fputs(html,fp_ou);
+   fputs("</table></div>\n",fp_ou);
+   fputs("<div align=\"center\"><table cellpadding=\"0\" cellspacing=\"0\">\n",fp_ou);
+
+   fprintf(fp_ou,"<tr><td class=\"header3\" colspan=\"2\">%s:&nbsp;%s</td></tr>\n",text[89],period);
 
    strcpy(wuser,user);
    if(strstr(wuser,"_") != 0)
@@ -96,37 +95,23 @@ void report_day(char *user, int *iprel, int *ipuser)
    if(strcmp(Ip2Name,"yes") == 0)
       if((str=(char *) strstr(name, ".")) != (char *) NULL) {
          if((str=(char *) strstr(str+1, ".")) != (char *) NULL)
-            ip2name(wuser);
+            ip2name(wuser,sizeof(wuser));
    }
 
-   if(UserTabFile[0] != '\0') {
-      sprintf(warea,":%s:",wuser);
-      if((str=(char *) strstr(userfile,warea)) != (char *) NULL ) {
-         z1=0;
-         str2=(char *) strstr(str+1,":");
-         str2++;
-         bzero(name, MAXLEN);
-         while(str2[z1] != ':') {
-            name[z1]=str2[z1];
-            z1++;
-         }
-      } else strcpy(name,wuser);
-   } else strcpy(name,user);
+   get_usertab_name(wuser,name,sizeof(name));
 
-   if(dotinuser && strstr(name,"_")) {
-      str2=(char *)subs(name,"_",".");
-      strcpy(name,str2);
+   if(dotinuser && strchr(name,'_')) {
+      subs(name,sizeof(name),"_",".");
    }
-     
-   sprintf(html,"<tr><th class=\"header3\" colspan=\"2\">%s:&nbsp;%s</th></tr>\n",text[90],name);
-   fputs(html,fp_ou);
+
+   fprintf(fp_ou,"<tr><th class=\"header3\" colspan=\"2\">%s:&nbsp;%s</th></tr>\n",text[90],name);
 
    fputs("<tr><td></td></tr><tr><td></td></tr>\n",fp_ou);
    fputs("<tr><td></td></tr><tr><td></td></tr></table>\n",fp_ou);
 
-   fputs("<table cellpadding=0 cellspacing=2>\n", fp_ou);
+   fputs("<table cellpadding=\"0\" cellspacing=\"2\">\n", fp_ou);
 
-   fputs( "<tr><td border=0></td>\n", fp_ou );
+   fputs( "<tr><td></td>\n", fp_ou );
 
    if(strcmp(datetimeby,"bytes") == 0)
      strcpy( html, text[93] );
@@ -134,42 +119,43 @@ void report_day(char *user, int *iprel, int *ipuser)
      strcpy( html, "H:M:S" );
 
    for( i = 0; i < hours.len; i++ )
-     fprintf( fp_ou,
-	"<td class=\"header3\">%02dH<br>%s</td>\n", hours.list[ i ], html );
+     fprintf( fp_ou, "<td class=\"header3\">%02dH<br>%s</td>\n", hours.list[ i ], html );
    fprintf( fp_ou,
      "<td class=\"header3\">%s<br>%s</td></tr>\n", text[107], html );
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-      getword(data,buf,' ');
+      if (getword(data,sizeof(data),buf,' ')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",wdirname);
+         exit(1);
+      }
       if(!count) {
          strcpy(odata,data);
          count++;
       }
 
-      getword(hour,buf,' ');
-      getword(elap,buf,' ');
+      if (getword(hour,sizeof(hour),buf,' ')<0 || getword(elap,sizeof(elap),buf,' ')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",wdirname);
+         exit(1);
+      }
       elap[strlen(elap)-1]='\0';
 
       if(strcmp(data,odata) != 0) {
          strcpy(oelap,elap);
 
-	 for( i = 0; i < hours.len; i++ )
-	   sprintf(c[ hours.list[ i ] ],"%s",fixtime(v[ hours.list[ i ] ]));
+         for( i = 0; i < hours.len; i++ )
+            sprintf(c[ hours.list[ i ] ],"%s",fixtime(v[ hours.list[ i ] ]));
 
-	 for( i = 8; i <= 17; i++ )
-	   if(strcmp(c[ hours.list[ i ] ],"00:00:00") == 0)
-		bzero(c[ hours.list[ i ] ],12);
+         for( i = 8; i <= 17; i++ )
+            if(strcmp(c[ hours.list[ i ] ],"00:00:00") == 0)
+               bzero(c[ hours.list[ i ] ],12);
+
+         fprintf(fp_ou, "<tr><td class=\"data\">%s</td>\n", odata );
+         for( i = 0; i < hours.len; i++ )
+            fprintf(fp_ou,"<td class=\"data\">%s</td>\n", c[ hours.list[ i ] ] );
+         fprintf( fp_ou, "<td class=\"data\">%s</td>\n", fixtime(tt) );
 
-         fprintf( fp_ou,
-       "<tr><td class=\"data\">%s</td>\n", odata );
-	 for( i = 0; i < hours.len; i++ )
-	   fprintf( fp_ou,
-	     "<td class=\"data\">%s</td>\n", c[ hours.list[ i ] ] );
-	 fprintf( fp_ou,
-	   "<td class=\"data\">%s</td>\n", fixtime(tt) );
-        
          tt=0;
-	 for( i = 0; i < hours.len; i++ ) v[ hours.list[ i ] ]=0;
+         for( i = 0; i < hours.len; i++ ) v[ hours.list[ i ] ]=0;
          strcpy(odata,data);
          strcpy(elap,oelap);
       }
@@ -188,30 +174,25 @@ void report_day(char *user, int *iprel, int *ipuser)
 
    for( i = 0; i < hours.len; i++ )
      if(strcmp(c[ hours.list[ i ] ],"00:00:00") == 0) bzero(c[ hours.list[ i ] ],12);
- 
-   fprintf( fp_ou,
-     "<tr><td class=\"data\">%s</td>\n", data );
+
+   fprintf(fp_ou, "<tr><td class=\"data\">%s</td>\n", data );
    for( i = 0; i < hours.len; i++ )
-     fprintf( fp_ou,
-       "<td class=\"data\">%s</td>\n", c[ hours.list[ i ] ] );
-   fprintf( fp_ou,
-     "<td class=\"data\">%s</td></tr>\n", fixtime(tt) );
+     fprintf(fp_ou, "<td class=\"data\">%s</td>\n", c[ hours.list[ i ] ] );
+   fprintf(fp_ou, "<td class=\"data\">%s</td></tr>\n", fixtime(tt) );
 
    for( i = 0; i < hours.len; i++ )
      sprintf(c[ hours.list[ i ] ],"%s",fixtime(t[ hours.list[ i ] ]));
 
-   fprintf( fp_ou,
-     "<tr><td class=\"header\">%s</td>\n", text[107] );
+   fprintf(fp_ou, "<tr><td class=\"header\">%s</td>\n", text[107] );
    for( i = 0; i < hours.len; i++ )
-     fprintf( fp_ou,
-       "<td class=\"header2\">%s</td>\n", c[ hours.list[ i ] ] );
-   fprintf( fp_ou,
-     "<td class=\"header2\">%s</td></tr>\n", fixtime(ttt) );
+     fprintf(fp_ou, "<td class=\"header2\">%s</td>\n", c[ hours.list[ i ] ] );
+   fprintf(fp_ou, "<td class=\"header2\">%s</td></tr>\n", fixtime(ttt) );
 
-   fputs("</body>\n</html>\n",fp_ou);
+   fputs("</table></div>\n",fp_ou);
 
    show_info(fp_ou);
 
+   fputs("</body>\n</html>\n",fp_ou);
    fclose(fp_in);
    fclose(fp_ou);
    return;
