@@ -1,6 +1,6 @@
 /*
  * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2008
+ *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
  *
  * SARG donations:
@@ -26,7 +26,15 @@
 #include "include/conf.h"
 #include "include/defs.h"
 
-void gerarel()
+static void maketmp(const char *user, const char *dirname, int debug, int indexonly);
+static void maketmp_hour(const char *user, const char *dirname, int indexonly);
+static void gravatmp_hora(const char *dirname, const char *user, const char *data, const char *hora, const char *elap, const char *accbytes, int indexonly);
+static void gravatmpf(const char *oldaccuser, const char *dirname, const char *oldurl, long long int nacc, long long int nbytes, const char *oldmsg, long long int nelap, int indexonly, long long int incache, long long int oucache);
+static void gravaporuser(const char *user, const char *dirname, const char *url, const char *ip, const char *data, const char *hora, const char *tam, const char *elap, int indexonly);
+static void gravager(char *dirname, const char *user, long long int nacc, const char *url, long long int nbytes, const char *ip, const char *hora, const char *dia, long long int nelap, long long int incache, long long int oucache);
+static void grava_SmartFilter(const char *dirname, const char *user, const char *ip, const char *data, const char *hora, const char *url, const char *smart);
+
+void gerarel(void)
 {
 
    FILE *fp_in;
@@ -34,13 +42,13 @@ void gerarel()
    char accdia[11], acchora[9], accuser[MAXLEN], accip[MAXLEN], accurl[MAXLEN], accbytes[12], accelap[10];
    char oldaccdia[11], oldacchora[9], oldaccip[MAXLEN], wdir[MAXLEN], per1[MAXLEN];
    char wdirname[MAXLEN], oldurl[MAXLEN], oldaccuser[MAXLEN];
-   char olduser[MAXLEN], oldmsg[50], acccode[50], oldaccelap[10], oldacccode[50], user[MAXLEN];
-   char ipantes[MAXLEN], nameantes[MAXLEN], wdname[MAXLEN], wname2[MAXLEN]; 
+   char olduser[MAXLEN], oldmsg[50], acccode[MAXLEN/2 - 1], oldaccelap[10], oldacccode[MAXLEN/2 - 1], user[MAXLEN];
+   char ipantes[MAXLEN], nameantes[MAXLEN];
    char accsmart[MAXLEN];
-   char wcrc[50];
-   char crc2[50];
-   long long int nbytes=0; 
-   long long int nelap=0; 
+   char wcrc[MAXLEN/2 -1];
+   char crc2[MAXLEN/2 -1];
+   long long int nbytes=0;
+   long long int nelap=0;
    long long int nacc=0;
    long long int rtotal=0;
    long long int incache=0;
@@ -48,6 +56,9 @@ void gerarel()
    char *s;
    DIR *dirp;
    struct dirent *direntp;
+   const char logext[]=".log";
+   int dlen;
+   char siteind[MAXLEN];
 
    ipantes[0]='\0';
    nameantes[0]='\0';
@@ -62,59 +73,72 @@ void gerarel()
       debugaz("dirname",dirname);
       debugaz("wdir",wdir);
    }
- 
-   strcpy(wdirname,dirname);
-   gperiod();
 
-   if(strlen(UserAgentLog) > 0 && email[0] == '\0') useragent();
+   gperiod(dirname,period);
+
+   if(UserAgentLog[0] != '\0' && email[0] == '\0') useragent();
 
    olduser[0]='\0';
    strncat(tmp,"/sarg",5);
 
    dirp = opendir(tmp);
    while ((direntp = readdir( dirp )) != NULL ) {
-      if((strstr(direntp->d_name,".log") == 0) ||
+      dlen=strlen(direntp->d_name)-(sizeof(logext)-1);
+      if (dlen<0) continue;
+      if((strcmp(direntp->d_name+dlen,logext) != 0) ||
          (strncmp(direntp->d_name,"download.log",12) == 0) ||
          (strncmp(direntp->d_name,"denied.log",10) == 0) ||
-	 (strncmp(direntp->d_name,"authfail.log.unsort",19) == 0))
+         (strncmp(direntp->d_name,"authfail.log.unsort",19) == 0))
          continue;
-         sprintf(tmp3,"%s/%s",tmp,direntp->d_name);
-         if((fp_in=fopen(tmp3,"r"))==NULL){
-            fprintf(stderr, "SARG: (report) %s: %s\n",text[45],tmp);
+      if (snprintf(tmp3,sizeof(tmp3),"%s/%s",tmp,direntp->d_name)>=sizeof(tmp3)) {
+         fprintf(stderr, "SARG: (report) directory entry too long: %s/%s\n",tmp,direntp->d_name);
          exit(1);
       }
-    
-      strcpy(wdname,direntp->d_name);
-      strip_prefix:
-      getword(wname2,wdname,'.');
-      strcat(user,wname2);
-   
-      if(strcmp(wdname,"log") !=0) {
-         strcat(user,".");
-         goto strip_prefix;
+      if((fp_in=MY_FOPEN(tmp3,"r"))==NULL){
+         fprintf(stderr, "SARG: (report) %s: %s\n",text[45],tmp);
+         exit(1);
       }
- 
+
+      if (dlen>0) {
+         if (dlen>=sizeof(user)) continue;
+         strncpy(user,direntp->d_name,dlen);
+         user[dlen]=0;
+      } else {
+         user[0]='\0';
+      }
+
       strcpy(wdirname,dirname);
       maketmp(user,tmp,debug,indexonly);
       maketmp_hour(user,tmp,indexonly);
-   
+
+      strcpy(u2,user);
+      if(strcmp(Ip2Name,"yes") == 0)
+         ip2name(u2,sizeof(u2));
+      get_usertab_name(u2,name,sizeof(name));
+
+      if(dotinuser && strchr(name,'_')) {
+         subs(name,sizeof(name),"_",".");
+      }
+
       ttopen=0;
       bzero(html_old, MAXLEN);
-   
+
       while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-         getword(accdia,buf,' ');
-         getword(acchora,buf,' ');
-         getword(accuser,buf,' ');
-         getword(accip,buf,' ');
-         getword(accurl,buf,' ');
-         getword(accbytes,buf,' ');
-         getword(acccode,buf,' ');
+         if (getword(accdia,sizeof(accdia),buf,' ')<0 || getword(acchora,sizeof(acchora),buf,' ')<0 ||
+             getword(accuser,sizeof(accuser),buf,' ')<0 || getword(accip,sizeof(accip),buf,' ')<0 ||
+             getword(accurl,sizeof(accurl),buf,' ')<0 || getword(accbytes,sizeof(accbytes),buf,' ')<0 ||
+             getword(acccode,sizeof(acccode),buf,' ')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp3);
+            exit(1);
+         }
          if(strncmp(acccode,"TCP_DENIED/407",14) == 0) continue;
-         getword(accelap,buf,' ');
-         getword(accsmart,buf,' ');
-         getword(accsmart,buf,'"');
-   
-         if(strlen(accsmart) > 0) {
+         if (getword(accelap,sizeof(accelap),buf,' ')<0 || getword(accsmart,sizeof(accsmart),buf,' ')<0 ||
+             getword(accsmart,sizeof(accsmart),buf,'"')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp3);
+            exit(1);
+         }
+
+         if(accsmart[0] != '\0') {
             smartfilter++;
             strcpy(wdirname,dirname);
             grava_SmartFilter(wdirname,accuser,accip,accdia,acchora,accurl,accsmart);
@@ -123,11 +147,11 @@ void gerarel()
          if(strcmp(Ip2Name,"yes") == 0) {
             if(strcmp(accip,ipantes) != 0) {
                strcpy(ipantes,accip);
-               ip2name(accip);
+               ip2name(accip,sizeof(accip));
                strcpy(nameantes,accip);
             } else strcpy(accip,nameantes);
          }
-     
+
          strcpy(wdirname,dirname);
          gravatmp_hora(wdirname,accuser,accdia,acchora,accelap,accbytes,indexonly);
 
@@ -153,15 +177,14 @@ void gerarel()
                if(strstr(oldacccode,"DENIED") != 0)
                   sprintf(oldmsg,"%s",text[46]);
                gravatmp(oldaccuser,wdirname,oldurl,nacc,nbytes,oldmsg,nelap,indexonly,incache,oucache);
-               gravager(wdirname,oldaccuser,nacc,oldurl,nbytes,oldaccip,oldacchora,oldaccdia,nelap,
-				incache,oucache);
+               gravager(wdirname,oldaccuser,nacc,oldurl,nbytes,oldaccip,oldacchora,oldaccdia,nelap,incache,oucache);
                nacc=0;
                nbytes=0;
                nelap=0;
                incache=0;
                oucache=0;
             }
-         } else {     
+         } else {
             if(strcmp(oldurl,accurl) != 0 || strcmp(oldaccuser,accuser) != 0){
                strcpy(oldmsg,"OK");
                if(strstr(oldacccode,"DENIED") != 0)
@@ -175,132 +198,112 @@ void gerarel()
                nelap=0;
                incache=0;
                oucache=0;
-   	    if(strcmp(oldaccuser,accuser) != 0)
-   	       ind2=0;
+               if(strcmp(oldaccuser,accuser) != 0)
+                  ind2=0;
             }
-      }
-      nacc++;
-      nbytes+=my_atoll(accbytes);
-      nelap+=my_atoll(accelap);
+         }
+         nacc++;
+         nbytes+=my_atoll(accbytes);
+         nelap+=my_atoll(accelap);
 
-      if(strstr(ReportType,"site_user_time_date") != 0) {
-         if(!ttopen) {
-            ind2++;
-	    strcpy(siteind,accurl);
-	    str=siteind;
-	    for(z1=0; z1<strlen(str); z1++) {
-	    if(str[z1]=='?' || str[z1]=='-' || str[z1]=='.' || str[z1]==':' || str[z1]=='/' || str[z1]=='\\')
-	                     str[z1]='_';
-            }
-	    sprintf(arqtt,"%s/%s",dirname,accuser);
-	    if(access(arqtt, R_OK) != 0)
-               my_mkdir(arqtt);
-	    sprintf(arqtt,"%s/%s/tt%s-%s.html",dirname,accuser,accuser,siteind);
-            if(strlen(arqtt) > 255) {
-               strncpy(val7,arqtt,255);
-               bzero(arqtt,MAXLEN);
-               strcpy(arqtt,val7);
-            }            
-            if ((fp_tt = fopen(arqtt, "w")) == 0) {
-               fprintf(stderr, "SARG: (report) %s: %s\n",text[45],arqtt);
-               exit(1);
-            }
-	    ttopen=1;
+         if(strstr(ReportType,"site_user_time_date") != 0) {
+            if(!ttopen) {
+               ind2++;
+               strcpy(siteind,accurl);
+               for(str=siteind; *str; str++) {
+                  if(*str=='?' || *str=='-' || *str=='.' || *str==':' || *str=='/' || *str=='\\' || *str=='*' ||
+                     *str=='\'' || *str=='\"' || *str=='$')
+                     *str='_';
+               }
+               sprintf(arqtt,"%s/%s",dirname,accuser);
+               if(access(arqtt, R_OK) != 0)
+                  my_mkdir(arqtt);
+               sprintf(arqtt,"%s/%s/tt%s-%s.html",dirname,accuser,accuser,siteind);
+               if(strlen(arqtt) > 255) {
+                  arqtt[255]='\0';
+               }
+               if ((fp_tt = fopen(arqtt, "w")) == 0) {
+                  fprintf(stderr, "SARG: (report) %s: %s\n",text[45],arqtt);
+                  exit(1);
+               }
+               ttopen=1;
 
-   	    if(strcmp(Privacy,"yes") == 0)
-               sprintf(httplink,"<font size=%s color=%s><href=http://%s>%s",	\
-	    	          FontSize,PrivacyStringColor,PrivacyString,PrivacyString);
-	     else
-               sprintf(httplink,"<font size=%s><a href=http://%s>%s</A>",FontSize,accurl,accurl);
+               /*
+               if(strcmp(Privacy,"yes") == 0)
+                  sprintf(httplink,"<font size=%s color=%s><href=http://%s>%s",	\
+                              FontSize,PrivacyStringColor,PrivacyString,PrivacyString);
+               else
+                  sprintf(httplink,"<font size=%s><a href=\"http://%s\">%s</a>",FontSize,accurl,accurl);
+               */
 
-            sprintf(ltext110,"%s",text[110]);
-            if(ltext110){
+               sprintf(ltext110,"%s",text[110]);
                for(s=ltext110; *s; ++s)
                   *s=tolower(*s);
+
+               fprintf(fp_tt, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n<html>\n<head>\n  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\">\n",CharSet);
+               css(fp_tt);
+               fputs("</head>\n",fp_tt);
+               fprintf(fp_tt,"<body bgcolor=\"%s\" text=\"%s\" background=\"%s\">\n",BgColor,TxColor,BgImage);
+               write_logo_image(fp_tt);
+
+               if(strcmp(IndexTree,"date") == 0)
+                  show_sarg(fp_tt, "../../../..");
+               else
+                  show_sarg(fp_tt, "../..");
+
+               fputs("<div align=\"center\"><table cellpadding=\"0\" cellspacing=\"0\">\n",fp_tt);
+               fprintf(fp_tt,"<tr><th class=\"title\" colspan=\"2\">%s</th></tr>\n",Title);
+
+               fprintf(fp_tt,"<tr><td class=\"header\" colspan=\"2\">%s:&nbsp;%s</td></tr>\n",text[89],period);
+               fprintf(fp_tt,"<tr><td class=\"header\" colspan=\"2\">%s:&nbsp;%s</td></tr>\n",text[90],name);
+               fprintf(fp_tt,"<tr><td class=\"header\" colspan=\"2\">%s:&nbsp;%s, %s</td></tr>\n",text[104],UserSortField,UserSortOrder);
+               fprintf(fp_tt,"<tr><th class=\"header3\" colspan=\"2\">%s</th></tr>\n",text[32]);
+
+               fputs("</table></div>\n",fp_tt);
+               fputs("<div align=\"center\"><table cellpadding=\"0\" cellspacing=\"2\">\n",fp_tt);
+               fputs("<tr><td></td><td></td></tr>",fp_tt);
+               bzero(tmp4, MAXLEN);
+               strncpy(tmp4,text[110],4);
+               fprintf(fp_tt,"<tr><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th></tr>\n",text[91],tmp4,text[110]+5);
             }
 
-	    fprintf(fp_tt, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n<html>\n<head>\n  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\">\n",CharSet);
-            css(fp_tt);
-            fputs("</head>\n",fp_tt);
-            fprintf(fp_tt,"<body bgcolor=%s text=%s background='%s'>\n",BgColor,TxColor,BgImage);
-	    if(strlen(LogoImage) > 0) fprintf(fp_tt, "<center><table cellpadding=\"0\" cellspacing=\"0\">\n<tr><th class=\"logo\"><img src='%s' border=0 align=absmiddle width=%s height=%s>&nbsp;%s</th></tr>\n<tr><td height=\"5\"></td></tr>\n</table>\n",LogoImage,Width,Height,LogoText);
+            sprintf(html,"<tr><td class=\"data\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s</td></tr>\n",accurl,accdia,acchora);
 
-            if(strcmp(IndexTree,"date") == 0)
-               show_sarg(fp_tt, "../../../..");
-            else
-               show_sarg(fp_tt, "../..");
+            if(strcmp(html,html_old) != 0)
+               fputs(html,fp_tt);
+            strcpy(html_old, html);
+         } else bzero(ltext110, 50);
 
-            fputs("<center><table cellpadding=0 cellspacing=0>\n",fp_tt);
-            fprintf(fp_tt,"<tr><th class=\"title\" colspan=\"2\">%s</th></tr>\n",Title);
+         strcpy(wcrc,acccode);
+         if (getword(crc2,sizeof(crc2),wcrc,'/')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp3);
+            exit(1);
+         }
 
-            strcpy(u2,user);
-            if(strcmp(Ip2Name,"yes") == 0)
-               ip2name(u2);
-            if(UserTabFile[0] != '\0') {
-               sprintf(warea,":%s:",u2);
-               if((str=(char *) strstr(userfile,warea)) != (char *) NULL ) {
-                  z1=0;
-                  str2=(char *) strstr(str+1,":");
-                  str2++;
-                  bzero(name, MAXLEN);
-                  while(str2[z1] != ':') {
-                     name[z1]=str2[z1];
-                     z1++;
-                  }
-               } else strcpy(name,u2);
-            } else strcpy(name,u2);
+         if(strstr(crc2,"MISS") != 0)
+            oucache+=my_atoll(accbytes);
+         else incache+=my_atoll(accbytes);
 
-            if(dotinuser && strstr(name,"_")) {
-               str2=(char *)subs(name,"_",".");
-               strcpy(name,str2);
-            }
+         strcpy(oldurl,accurl);
 
-	    fprintf(fp_tt,"<tr><td class=\"header\" colspan=\"2\">%s:&nbsp;%s</td></tr>\n",text[89],period);
-	    fprintf(fp_tt,"<tr><td class=\"header\" colspan=\"2\">%s:&nbsp;%s</td></tr>\n",text[90],name);
-	    fprintf(fp_tt,"<tr><td class=\"header\" colspan=\"2\">%s:&nbsp;%s, %s</td></tr>\n",text[104],UserSortField,UserSortOrder);
-	    fprintf(fp_tt,"<tr><th class=\"header3\" colspan=2>%s</th></tr>\n",text[32]);
+         if(strcmp(accuser,oldaccuser) != 0) {
+            strcpy(wdirname,dirname);
+            day_totalize(tmp,oldaccuser,indexonly);
+         }
 
-            fputs("</table></center>\n",fp_tt);
-            fputs("<center><table cellpadding=\"0\" cellspacing=\"2\">\n",fp_tt);
-            fputs("<tr><td></td><td></td></tr>",fp_tt);
-            bzero(tmp4, MAXLEN);
-            strncpy(tmp4,text[110],4);
-            fprintf(fp_tt,"<tr><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th></tr>\n",text[91],tmp4,text[110]+5);
-	 }
+         strcpy(oldaccuser,accuser);
+         strcpy(oldacccode,acccode);
+         strcpy(oldaccelap,accelap);
+         strcpy(oldaccip,accip);
+         strcpy(oldaccdia,accdia);
+         strcpy(oldacchora,acchora);
 
-         sprintf(html,"<tr><td class=\"data\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s</td></tr>\n",accurl,accdia,acchora);
-
-	 if(strcmp(html,html_old) != 0)
-            fputs(html,fp_tt);
-	 strcpy(html_old, html);
-      } else bzero(ltext110, 50);
-
-      strcpy(wcrc,acccode);
-      getword(crc2,wcrc,'/');
-
-      if(strstr(crc2,"MISS") != 0)
-         oucache+=my_atoll(accbytes);
-       else incache+=my_atoll(accbytes);
-
-      strcpy(oldurl,accurl);
-
-      if(strcmp(accuser,oldaccuser) != 0) {
-         strcpy(wdirname,dirname);
-         day_totalize(tmp,oldaccuser,indexonly);
       }
-
-      strcpy(oldaccuser,accuser);
-      strcpy(oldacccode,acccode);
-      strcpy(oldaccelap,accelap);
-      strcpy(oldaccip,accip);
-      strcpy(oldaccdia,accdia);
-      strcpy(oldacchora,acchora);
-
+      bzero(user,MAXLEN);
+      fclose(fp_in);
+      unlink(tmp3);
    }
-   bzero(user,MAXLEN);
-   fclose(fp_in);
-   unlink(tmp3);
-}
+   closedir(dirp);
 
    strcpy(oldmsg,"OK");
    if(strstr(oldacccode,"DENIED") != 0)
@@ -319,7 +322,7 @@ void gerarel()
    strcpy(wdirname,dirname);
    totalger(wdirname, debug, outdir);
 
-   if(strlen(email) == 0) {
+   if(email[0] == '\0') {
       if(strstr(ReportType,"downloads") != 0) download_report();
 
       if(strlen(DansGuardianConf) > 0) {
@@ -365,14 +368,16 @@ void gerarel()
       index_only(wdirname, debug);
    }
 
+   /*
+   2009-10-13(Frederic) This piece of code is never called so it is commented out for good.
    if(strlen(email) < 0)
       removetmp(dirname);
+   */
 
    return;
 }
 
-
-void maketmp(char *user, char *dirname, int debug, int indexonly)
+static void maketmp(const char *user, const char *dirname, int debug, int indexonly)
 {
 
    FILE *fp_ou;
@@ -387,8 +392,7 @@ void maketmp(char *user, char *dirname, int debug, int indexonly)
    strcat(wdirname,user);
 
    if(debug){
-      sprintf(msg,"%s: %s",text[49],wdirname);
-      debuga(msg);
+      debuga("%s: %s",text[49],wdirname);
    }
 
    strcat(wdirname,".utmp");
@@ -402,7 +406,7 @@ void maketmp(char *user, char *dirname, int debug, int indexonly)
 }
 
 
-void maketmp_hour(char *user, char *dirname, int indexonly)
+static void maketmp_hour(const char *user, const char *dirname, int indexonly)
 {
 
    FILE *fp_ou;
@@ -427,10 +431,15 @@ void maketmp_hour(char *user, char *dirname, int indexonly)
 }
 
 
-void gravatmp(char *oldaccuser, char *dirname, char *oldurl, long long int nacc, long long int nbytes, char *oldmsg, long long int nelap, int indexonly, long long int incache, long long int oucache)
+void gravatmp(const char *oldaccuser, const char *dirname, const char *oldurl, long long int nacc, long long int nbytes, const char *oldmsg, long long int nelap, int indexonly, long long int incache, long long int oucache)
 {
 
    FILE *fp_ou;
+   char val1[16];
+   char val2[16];
+   char val3[16];
+   char val4[16];
+   char val5[16];
 
    char wdirname[MAXLEN];
 
@@ -442,10 +451,10 @@ void gravatmp(char *oldaccuser, char *dirname, char *oldurl, long long int nacc,
    strcat(wdirname,oldaccuser);
    strcat(wdirname,".utmp");
 
-   if((fp_ou=fopen(wdirname,"a"))==NULL){
+   if((fp_ou=MY_FOPEN(wdirname,"a"))==NULL){
       fprintf(stderr, "SARG: (report) %s: %s\n",text[45],wdirname);
       exit(1);
-   } 
+   }
 
    my_lltoa(nacc,val1,15);
    my_lltoa(nbytes,val2,15);
@@ -468,11 +477,10 @@ void gravatmp(char *oldaccuser, char *dirname, char *oldurl, long long int nacc,
 }
 
 
-void gravatmp_hora(char *dirname, char *user, char *data, char *hora, char *elap, char *bytes, int indexonly)
+static void gravatmp_hora(const char *dirname, const char *user, const char *data, const char *hora, const char *elap, const char *bytes, int indexonly)
 {
 
    FILE *fp_ou;
-
    char wdirname[MAXLEN];
 
    if(indexonly || (strstr(ReportType,"users_sites") == 0)) return;
@@ -482,7 +490,7 @@ void gravatmp_hora(char *dirname, char *user, char *data, char *hora, char *elap
    strcat(wdirname,user);
    strcat(wdirname,".htmp");
 
-   if((fp_ou=fopen(wdirname,"a"))==NULL){
+   if((fp_ou=MY_FOPEN(wdirname,"a"))==NULL){
       fprintf(stderr, "SARG: (report-2) %s: %s - %s\n",text[45],wdirname,strerror(errno));
       exit(1);
    }
@@ -496,7 +504,7 @@ void gravatmp_hora(char *dirname, char *user, char *data, char *hora, char *elap
 }
 
 
-void gravaporuser(char *user, char *dirname, char *url, char *ip, char *data, char *hora, char *tam, char *elap, int indexonly)
+static void gravaporuser(const char *user, const char *dirname, const char *url, const char *ip, const char *data, const char *hora, const char *tam, const char *elap, int indexonly)
 {
 
    FILE *fp_ou;
@@ -510,10 +518,10 @@ void gravaporuser(char *user, char *dirname, char *url, char *ip, char *data, ch
    strcat(wdirname,user);
    strcat(wdirname,".ip");
 
-   if((fp_ou=fopen(wdirname,"a"))==NULL){
+   if((fp_ou=MY_FOPEN(wdirname,"a"))==NULL){
       fprintf(stderr, "SARG: (report) %s: %s\n",text[45],wdirname);
       exit(1);
-   } 
+   }
 
    fprintf(fp_ou,"%s %s %s %s %s %s\n",ip,url,data,hora,tam,elap);
 
@@ -524,7 +532,7 @@ void gravaporuser(char *user, char *dirname, char *url, char *ip, char *data, ch
 }
 
 
-void gravatmpf(char *oldaccuser, char *dirname, char *oldurl, long long int nacc, long long int nbytes, char *oldmsg, long long int nelap, int indexonly, long long int incache, long long int oucache)
+static void gravatmpf(const char *oldaccuser, const char *dirname, const char *oldurl, long long int nacc, long long int nbytes, const char *oldmsg, long long int nelap, int indexonly, long long int incache, long long int oucache)
 {
 
    FILE *fp_ou;
@@ -538,7 +546,7 @@ void gravatmpf(char *oldaccuser, char *dirname, char *oldurl, long long int nacc
    strcat(wdirname,oldaccuser);
    strcat(wdirname,".utmp");
 
-   if((fp_ou=fopen(wdirname,"a"))==NULL){
+   if((fp_ou=MY_FOPEN(wdirname,"a"))==NULL){
       fprintf(stderr, "SARG: (report) %s: %s\n",text[45],wdirname);
       exit(1);
    }
@@ -556,24 +564,24 @@ void gravatmpf(char *oldaccuser, char *dirname, char *oldurl, long long int nacc
 
    if(fp_tt) {
       fputs("</table>\n",fp_tt);
-      fputs("</html>\n",fp_tt);
+      fputs("</body>\n</html>\n",fp_tt);
       fclose(fp_tt);
    }
 
    return;
-     
+
 }
 
 
-void gravager(char *dirname, char *user, long long int nacc, char *url, long long int nbytes, char *ip, char *hora, char *dia, long long int nelap, long long int incache, long long int oucache)
+static void gravager(char *dirname, const char *user, long long int nacc, const char *url, long long int nbytes, const char *ip, const char *hora, const char *dia, long long int nelap, long long int incache, long long int oucache)
 {
 
    FILE *fp_ou;
 
    strcat(dirname,"/");
    strcat(dirname,"sarg-general");
-   
-   if((fp_ou=fopen(dirname,"a"))==NULL){
+
+   if((fp_ou=MY_FOPEN(dirname,"a"))==NULL){
       fprintf(stderr, "SARG: (report) %s: %s\n",text[45],dirname);
       exit(1);
    }
@@ -590,7 +598,7 @@ void gravager(char *dirname, char *user, long long int nacc, char *url, long lon
 
 }
 
-void grava_SmartFilter(char *dirname, char *user, char *ip, char *data, char *hora, char *url, char *smart)
+static void grava_SmartFilter(const char *dirname, const char *user, const char *ip, const char *data, const char *hora, const char *url, const char *smart)
 {
 
    FILE *fp_ou;
@@ -599,7 +607,7 @@ void grava_SmartFilter(char *dirname, char *user, char *ip, char *data, char *ho
 
    sprintf(wdirname,"%s/smartfilter.unsort",dirname);
 
-   if((fp_ou=fopen(wdirname,"a"))==NULL){
+   if((fp_ou=MY_FOPEN(wdirname,"a"))==NULL){
       fprintf(stderr, "SARG: (report) %s: %s\n",text[45],wdirname);
       exit(1);
    }
