@@ -1,6 +1,6 @@
 /*
  * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2008
+ *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
  *
  * SARG donations:
@@ -24,8 +24,9 @@
  */
 
 #include "include/conf.h"
+#include "include/defs.h"
 
-void splitlog(char *arq, char *df, int dfrom, int duntil, char *convert)
+void splitlog(const char *arq, char *df, int dfrom, int duntil, char *convert)
 {
 
    FILE *fp_in;
@@ -39,15 +40,18 @@ void splitlog(char *arq, char *df, int dfrom, int duntil, char *convert)
    struct tm *t;
 
    if(arq[0] == '\0')
-      strcpy(arq,"/usr/local/squid/logs/access.log");
+      arq="/var/log/squid/access.log";
 
-   if((fp_in=fopen(arq,"r"))==NULL) {
+   if((fp_in=MY_FOPEN(arq,"r"))==NULL) {
       fprintf(stderr, "SARG: (splitlog) %s: %s\n",text[8],arq);
       exit(1);
    }
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-      getword(data,buf,' ');
+      if (getword(data,sizeof(data),buf,' ')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",arq);
+         exit(1);
+      }
       tt=atoi(data);
       t=localtime(&tt);
 
