@@ -1,6 +1,6 @@
 /*
  * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2008
+ *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
  *
  * SARG donations:
@@ -24,18 +24,19 @@
  */
 
 #include "include/conf.h"
+#include "include/defs.h"
 
-void topsites()
+void topsites(void)
 {
 
    FILE *fp_in, *fp_ou;
-      
+
    char url[MAXLEN];
    char ourl[MAXLEN];
    char nacc[20];
    char nbytes[20];
    char ntime[20];
-   char ntemp[20];
+   char ntemp[255];
    char ttnacc[20];
    char ttnbytes[20];
    char ttntime[20];
@@ -54,6 +55,7 @@ void topsites()
    long long int tntime=0;
    long long int twork1=0, twork2=0, twork3=0;
    int regs=0;
+   int cstatus;
 
    if(strcmp(Privacy,"yes") == 0)
       return;
@@ -74,14 +76,23 @@ void topsites()
       exit(1);
    }
 
-   fgets(period,sizeof(period),fp_in);
+   if (!fgets(period,sizeof(period),fp_in)) {
+      fprintf(stderr,"SARG: (topuser) read error in %s\n",per);
+      exit(1);
+   }
    fclose(fp_in);
 
-   sprintf(csort,"sort -k 4,4 -o '%s' '%s'",general2,general);
-   system(csort);
+   sprintf(csort,"sort -k 4,4 -o \"%s\" \"%s\"",general2,general);
+   cstatus=system(csort);
+   if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
+      fprintf(stderr, "SARG: sort command return status %d\n",WEXITSTATUS(cstatus));
+      fprintf(stderr, "SARG: sort command: %s\n",csort);
+      exit(1);
+   }
 
    if((fp_in=fopen(general2,"r"))==NULL) {
      fprintf(stderr, "SARG: (topsite) %s: %s\n",text[8],general2);
+     fprintf(stderr, "SARG: sort command: %s\n",csort);
      exit(1);
    }
 
@@ -91,20 +102,53 @@ void topsites()
    }
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-      getword(url,buf,' ');
+      if (getword(url,sizeof(url),buf,' ')<0) {
+         printf("SARG: Maybe you have an invalid user in your %s file of the topsites.\n",general2);
+         exit(1);
+      }
       if(strcmp(url,"TOTAL") == 0) {
-         getword(ttnacc,buf,' ');
-         getword(ttnbytes,buf,' ');
-         getword(ttntime,buf,' ');
+         if (getword(ttnacc,sizeof(ttnacc),buf,' ')<0) {
+            printf("SARG: Maybe you have an invalid total number of access in your %s file of the topsites.\n",general2);
+            exit(1);
+         }
+         if (getword(ttnbytes,sizeof(ttnbytes),buf,' ')<0) {
+            printf("SARG: Maybe you have an invalid total number of bytes in your %s file of the topsites.\n",general2);
+            exit(1);
+         }
+         if (getword(ttntime,sizeof(ttntime),buf,' ')<0) {
+            printf("SARG: Maybe you have an invalid total time in your %s file of the topsites.\n",general2);
+            exit(1);
+         }
          continue;
       }
-      getword(nacc,buf,' ');
-      getword(nbytes,buf,' ');
-      getword(url,buf,' ');
-      getword(ntemp,buf,' ');
-      getword(ntemp,buf,' ');
-      getword(ntemp,buf,' ');
-      getword(ntime,buf,' ');
+      if (getword(nacc,sizeof(nacc),buf,' ')<0) {
+         printf("SARG: Maybe you have an invalid number of access in your %s file of the topsites.\n",general2);
+         exit(1);
+      }
+      if (getword(nbytes,sizeof(nbytes),buf,' ')<0) {
+         printf("SARG: Maybe you have an invalid number of bytes in your %s file of the topsites.\n",general2);
+         exit(1);
+      }
+      if (getword(url,sizeof(url),buf,' ')<0) {
+         printf("SARG: Maybe you have an invalid url in your %s file of the topsites.\n",general2);
+         exit(1);
+      }
+      if (getword(ntemp,sizeof(ntemp),buf,' ')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in column 5 in your %s file of the topsites.\n",general2);
+         exit(1);
+      }
+      if (getword(ntemp,sizeof(ntemp),buf,' ')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in column 6 in your %s file of the topsites.\n",general2);
+         exit(1);
+      }
+      if (getword(ntemp,sizeof(ntemp),buf,' ')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in column 7 in your %s file of the topsites.\n",general2);
+         exit(1);
+      }
+      if (getword(ntime,sizeof(ntime),buf,' ')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in column 8 in your %s file of the topsites.\n",general2);
+         exit(1);
+      }
 
       if(!regs) {
          strcpy(ourl,url);
@@ -115,8 +159,7 @@ void topsites()
          my_lltoa(tnacc,val1,15);
          my_lltoa(tnbytes,val2,15);
          my_lltoa(tntime,val3,15);
-         sprintf(buf,"%s %s %s %s\n",val1,val2,val3,ourl);
-         fputs(buf, fp_ou);
+         fprintf(fp_ou,"%s %s %s %s\n",val1,val2,val3,ourl);
          strcpy(ourl,url);
          tnacc=0;
          tnbytes=0;
@@ -131,8 +174,7 @@ void topsites()
    my_lltoa(tnacc,val1,15);
    my_lltoa(tnbytes,val2,15);
    my_lltoa(tntime,val3,15);
-   sprintf(buf,"%s %s %s %s\n",val1,val2,val3,ourl);
-   fputs(buf, fp_ou);
+   fprintf(fp_ou,"%s %s %s %s\n",val1,val2,val3,ourl);
 
    fclose(fp_in);
    fclose(fp_ou);
@@ -150,17 +192,22 @@ void topsites()
    if(strcmp(TopsitesSortType,"d") == 0)
       strcpy(sortt,"-r");
 
-   sprintf(csort,"sort %s -k %s -o '%s' '%s'",sortt,sortf,sites,general3);
-   system(csort);
+   sprintf(csort,"sort %s -k %s -o \"%s\" \"%s\"",sortt,sortf,sites,general3);
+   cstatus=system(csort);
+   if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
+      fprintf(stderr, "SARG: sort command return status %d\n",WEXITSTATUS(cstatus));
+      fprintf(stderr, "SARG: sort command: %s\n",csort);
+      exit(1);
+   }
+   if((fp_in=fopen(sites,"r"))==NULL) {
+      fprintf(stderr, "SARG: (topsite) %s: %s\n",text[8],sites);
+      fprintf(stderr, "SARG: sort command: %s\n",csort);
+      exit(1);
+   }
 
    unlink(general2);
    unlink(general3);
 
-   if((fp_in=fopen(sites,"r"))==NULL) {
-     fprintf(stderr, "SARG: (topsite) %s: %s\n",text[8],sites);
-     exit(1);
-   }
-
    if((fp_ou=fopen(report,"w"))==NULL) {
      fprintf(stderr, "SARG: (topsite) %s: %s\n",text[8],report);
      exit(1);
@@ -171,31 +218,27 @@ void topsites()
    fprintf(fp_ou, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n<html>\n<head>\n  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\">\n",CharSet);
    css(fp_ou);
    fputs("</head>\n",fp_ou);
-   fprintf(fp_ou,"<body bgcolor=%s text=%s background='%s'>\n",BgColor,TxColor,BgImage);
-   if(strlen(LogoImage) > 0) fprintf(fp_ou, "<center><table cellpadding=\"0\" cellspacing=\"0\">\n<tr><th class=\"logo\"><img src='%s' border=0 align=absmiddle width=%s height=%s>&nbsp;%s</th></tr>\n<tr><td height=\"5\"></td></tr>\n</table>\n",LogoImage,Width,Height,LogoText);
+   fprintf(fp_ou,"<body bgcolor=\"%s\" text=\"%s\" background=\"%s\">\n",BgColor,TxColor,BgImage);
+   write_logo_image(fp_ou);
 
    if(strcmp(IndexTree,"date") == 0)
       show_sarg(fp_ou,"../../..");
    else
       show_sarg(fp_ou, "..");
 
-   fputs("<center><table cellpadding=\"0\" cellspacing=\"0\">\n",fp_ou);
-   sprintf(url,"<tr><th class=\"title\">%s</th></tr>\n",Title);
-   fputs(url,fp_ou);
+   fputs("<div align=\"center\"><table cellpadding=\"0\" cellspacing=\"0\">\n",fp_ou);
+   fprintf(fp_ou,"<tr><th class=\"title\">%s</th></tr>\n",Title);
 
-   sprintf(url,"<tr><td class=\"header3\">%s: %s</td></tr>\n",text[89],period);
-   fputs(url,fp_ou);
-   sprintf(url,"<tr><th class=\"header3\">%s %s %s</th></tr>\n",text[83],TopSitesNum,text[84]);
-   fputs(url,fp_ou);
-   fputs("</table></center>\n",fp_ou);
+   fprintf(fp_ou,"<tr><td class=\"header3\">%s: %s</td></tr>\n",text[89],period);
+   fprintf(fp_ou,"<tr><th class=\"header3\">%s %s %s</th></tr>\n",text[83],TopSitesNum,text[84]);
+   fputs("</table></div>\n",fp_ou);
 
-   fputs("<center><table cellpadding=\"1\" cellspacing=\"2\">\n",fp_ou);
+   fputs("<div align=\"center\"><table cellpadding=\"1\" cellspacing=\"2\">\n",fp_ou);
    fputs("<tr><td></td></tr>\n",fp_ou);
-   sprintf(url,"<tr><th class=\"header\">%s</th><th class=\"header\">%s</th> \
+   fprintf(fp_ou,"<tr><th class=\"header\">%s</th><th class=\"header\">%s</th> \
 		<th class=\"header\">%s</th><th class=\"header\">%s</th> \
 		<th class=\"header\">%s</th></tr>\n", \
 		text[100],text[91],text[92],text[93],text[99]);
-   fputs(url,fp_ou);
 
    regs=1;
    ntopsites = 0;
@@ -203,38 +246,41 @@ void topsites()
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
       if(regs>atoi(TopSitesNum))
          break;
-      getword(nacc,buf,' ');
+      if (getword(nacc,sizeof(nacc),buf,' ')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",sites);
+         exit(1);
+      }
       if (atoi(nacc) == 0) continue;
-
-      getword(nbytes,buf,' ');
-      getword(ntime,buf,' ');
-      getword(url,buf,' ');
+      if (getword(nbytes,sizeof(nbytes),buf,' ')<0 ||
+          getword(ntime,sizeof(ntime),buf,' ')<0 || getword(url,sizeof(url),buf,' ')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",sites);
+         exit(1);
+      }
 
       twork1=my_atoll(nacc);
       twork2=my_atoll(nbytes);
       twork3=my_atoll(ntime);
 
-      sprintf(wwork1,"%s",fixnum(twork1,1));
-      sprintf(wwork2,"%s",fixnum(twork2,1));
-      sprintf(wwork3,"%s",fixtime(twork3,1));
+      strcpy(wwork1,fixnum(twork1,1));
+      strcpy(wwork2,fixnum(twork2,1));
+      strcpy(wwork3,fixtime(twork3));
 
       if(strlen(BlockIt) > 0)
          sprintf(BlockImage,"<a href=\"%s%s?url=%s\"><img src=\"../images/sarg-squidguard-block.png\" border=\"0\"></a>&nbsp;",wwwDocumentRoot,BlockIt,url);
       else BlockImage[0]='\0';
 
 
-      sprintf(ourl,"<tr><td class=\"data\">%d</td><td class=\"data2\">%s<a href=\"http://%s\"><font class=\"link\">%s</font></td><td class=\"data\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s</td></tr>\n",regs,BlockImage,url,url,wwork1,wwork2,wwork3);
-      fputs(ourl,fp_ou);
+      fprintf(fp_ou,"<tr><td class=\"data\">%d</td><td class=\"data2\">%s<a href=\"http://%s\"><font class=\"link\">%s</font></a></td><td class=\"data\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s</td></tr>\n",regs,BlockImage,url,url,wwork1,wwork2,wwork3);
       regs++;
    }
 
 
-   fputs("</table></center>\n",fp_ou);
+   fputs("</table>\n</div>\n",fp_ou);
 
    show_info(fp_ou);
 
    fputs("</body>\n</html>\n",fp_ou);
-   
+
    fclose(fp_in);
    fclose(fp_ou);
 
