@@ -1,6 +1,6 @@
 /*
  * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2008
+ *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
  *
  * SARG donations:
@@ -24,14 +24,17 @@
  */
 
 #include "include/conf.h"
+#include "include/defs.h"
 
-int topuser()
+void topuser(void)
 {
 
    FILE *fp_in = NULL, *fp_ou = NULL, *fp_top1 = NULL, *fp_top2 = NULL, *fp_top3 = NULL;
    long long int ttnbytes=0, ttnacc=0, tnacc=0;
    long long int tnbytes=0, ttnelap=0, tnelap=0;
    long long int tnincache=0, tnoucache=0, ttnincache=0, ttnoucache=0;
+   long long int twork;
+   long long int twork2;
    float perc=0.00;
    float perc2=0.00;
    float inperc=0.00, ouperc=0.00;
@@ -47,6 +50,16 @@ int topuser()
    int  totuser=0;
    int  topcount=0;
    char *s;
+   int cstatus;
+   char warea[MAXLEN];
+   char user2[MAXLEN];
+   char name[MAXLEN];
+   char href1[MAXLEN];
+   char href2[MAXLEN];
+   char href3[MAXLEN];
+   char href1end[5];
+   char href2end[5];
+   char href3end[5];
 
    ipantes[0]='\0';
    nameantes[0]='\0';
@@ -64,42 +77,83 @@ int topuser()
 
    ntopuser = 0;
    if((fp_in=fopen(wger,"r"))==NULL) {
-    fprintf(stderr, "SARG: (topuser) %s: %s\n",text[45],wger);
-    exit(1);
+      fprintf(stderr, "SARG: (topuser) %s: %s\n",text[45],wger);
+      exit(1);
    }
 
    if((fp_top2=fopen(top2,"w"))==NULL) {
-    fprintf(stderr, "SARG: (topuser) %s: %s\n",text[45],top2);
-    exit(1);
+      fprintf(stderr, "SARG: (topuser) %s: %s\n",text[45],top2);
+      exit(1);
    }
 
-   fscanf(fp_in,"%s%s%s%s%s%s%s%s%s%s",user,nacc,nbytes,url,ip,time,date,elap,incac,oucac);
+   olduser[0]='\0';
+   totuser=0;
 
-   strcpy(olduser,user);
-   totuser=1;
-
-   while(!feof(fp_in)) {
+   while(fgets(warea,sizeof(warea),fp_in)) {
+      if (getword(user,sizeof(user),warea,' ')<0) {
+         printf("SARG: Maybe you have a broken user in your %s file.\n",wger);
+         exit(1);
+      }
+      if(strcmp(user,"TOTAL") == 0) {
+         continue;
+      }
+      if (getword(nacc,sizeof(nacc),warea,' ')<0) {
+         printf("SARG: Maybe you have a broken number of access in your %s file.\n",wger);
+         exit(1);
+      }
+      if (getword(nbytes,sizeof(nbytes),warea,' ')<0) {
+         printf("SARG: Maybe you have a broken number of bytes in your %s file.\n",wger);
+         exit(1);
+      }
+      if (getword(url,sizeof(url),warea,' ')<0) {
+         printf("SARG: Maybe you have a broken url in your %s file.\n",wger);
+         exit(1);
+      }
+      if (getword(ip,sizeof(ip),warea,' ')<0) {
+         printf("SARG: Maybe you have a broken user's IP in your %s file.\n",wger);
+         exit(1);
+      }
+      if (getword(time,sizeof(time),warea,' ')<0) {
+         printf("SARG: Maybe you have a broken time in your %s file.\n",wger);
+         exit(1);
+      }
+      if (getword(date,sizeof(date),warea,' ')<0) {
+         printf("SARG: Maybe you have a broken date in your %s file.\n",wger);
+         exit(1);
+      }
+      if (getword(elap,sizeof(elap),warea,' ')<0) {
+         printf("SARG: Maybe you have a broken download duration in your %s file.\n",wger);
+         exit(1);
+      }
+      if (getword(incac,sizeof(incac),warea,' ')<0) {
+         printf("SARG: Maybe you have a broken in cache download in your %s file.\n",wger);
+         exit(1);
+      }
+      if (getword(oucac,sizeof(oucac),warea,' ')<0) {
+         printf("SARG: Maybe you have a broken not in cache download in your %s file.\n",wger);
+         exit(1);
+      }
       if(strcmp(olduser,user) != 0) {
-         if(strcmp(user,"TOTAL") != 0)
-            totuser++;
+         totuser++;
 
-         my_lltoa(tnbytes,val1,15);
-         my_lltoa(tnacc,val2,15);
-         my_lltoa(tnelap,val3,15);
-         my_lltoa(tnincache,val4,15);
-         my_lltoa(tnoucache,val5,15);
-         sprintf(preg,"%s %s %s %s %s %s\n",olduser,val1,val2,val3,val4,val5);
-         fputs(preg,fp_top2);
+         if (olduser[0] != '\0') {
+            my_lltoa(tnbytes,val1,15);
+            my_lltoa(tnacc,val2,15);
+            my_lltoa(tnelap,val3,15);
+            my_lltoa(tnincache,val4,15);
+            my_lltoa(tnoucache,val5,15);
+            fprintf(fp_top2,"%s %s %s %s %s %s\n",olduser,val1,val2,val3,val4,val5);
 
-	 strcpy(olduser,user);
-         ttnbytes+=tnbytes;
-	 ttnacc+=tnacc;
-	 ttnelap+=tnelap;
-         ttnincache+=tnincache;
-         ttnoucache+=tnoucache;
+            ttnbytes+=tnbytes;
+            ttnacc+=tnacc;
+            ttnelap+=tnelap;
+            ttnincache+=tnincache;
+            ttnoucache+=tnoucache;
+         }
+         strcpy(olduser,user);
          tnbytes=0;
          tnacc=0;
-         tnelap=0; 
+         tnelap=0;
          tnincache=0;
          tnoucache=0;
       }
@@ -109,33 +163,29 @@ int topuser()
       tnelap+=my_atoll(elap);
       tnincache+=my_atoll(incac);
       tnoucache+=my_atoll(oucac);
-
-      fscanf(fp_in,"%s%s%s%s%s%s%s%s%s%s",user,nacc,nbytes,url,ip,time,date,elap,incac,oucac);
-
-      if(strcmp(user,"TOTAL") == 0)
-         continue;
    }
 
-   my_lltoa(tnbytes,val1,15);
-   my_lltoa(tnacc,val2,15);
-   my_lltoa(tnelap,val3,15);
-   my_lltoa(tnincache,val4,15);
-   my_lltoa(tnoucache,val5,15);
-   sprintf(preg,"%s %s %s %s %s %s\n",olduser,val1,val2,val3,val4,val5);
-   fputs(preg,fp_top2);
+   if (olduser[0] != '\0') {
+      my_lltoa(tnbytes,val1,15);
+      my_lltoa(tnacc,val2,15);
+      my_lltoa(tnelap,val3,15);
+      my_lltoa(tnincache,val4,15);
+      my_lltoa(tnoucache,val5,15);
+      fprintf(fp_top2,"%s %s %s %s %s %s\n",olduser,val1,val2,val3,val4,val5);
 
-   ttnbytes+=tnbytes;
-   ttnacc+=tnacc;
-   ttnelap+=tnelap;
-   ttnincache+=tnincache;
-   ttnoucache+=tnoucache;
+      ttnbytes+=tnbytes;
+      ttnacc+=tnacc;
+      ttnelap+=tnelap;
+      ttnincache+=tnincache;
+      ttnoucache+=tnoucache;
+
+   }
 
    my_lltoa(ttnbytes,val1,15);
    my_lltoa(ttnacc,val2,15);
    my_lltoa(ttnelap,val3,15);
    my_lltoa(ttnincache,val4,15);
    my_lltoa(ttnoucache,val5,15);
-   sprintf(preg,"TOTAL %s %s %s %s %s\n",val1,val2,val3,val4,val5);
    if (fp_in) fclose(fp_in);
    if (fp_top2) fclose(fp_top2);
 
@@ -154,18 +204,42 @@ int topuser()
    if(strcmp(TopuserSortOrder,"normal") == 0)
       order[0]='\0';
 
-   sprintf(csort,"sort -n -T %s %s -k %s -o '%s' '%s'", TempDir, order, sfield, top1, top2);
-   system(csort);
+   sprintf(csort,"sort -n -T \"%s\" %s -k %s -o \"%s\" \"%s\"", TempDir, order, sfield, top1, top2);
+   cstatus=system(csort);
+   if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
+      fprintf(stderr, "SARG: sort command return status %d\n",WEXITSTATUS(cstatus));
+      fprintf(stderr, "SARG: sort command: %s\n",csort);
+      exit(1);
+   }
+   if((fp_top1=fopen(top1,"a"))==NULL) {
+      fprintf(stderr, "SARG: (topuser) %s: %s\n",text[45],top1);
+      fprintf(stderr, "SARG: sort command: %s\n",csort);
+      exit(1);
+   }
 
    unlink(top2);
 
-   if((fp_top1=fopen(top1,"a"))==NULL) {
-    fprintf(stderr, "SARG: (topuser) %s: %s\n",text[45],top1);
-    exit(1);
-   }
-   fputs(preg,fp_top1);
+   fprintf(fp_top1,"TOTAL %s %s %s %s %s\n",val1,val2,val3,val4,val5);
    fclose(fp_top1);
 
+   /*
+   * get period
+   */
+
+   strcpy(arqper,dirname);
+   strcat(arqper,"/sarg-period");
+
+   if ((fp_in = fopen(arqper, "r")) == 0) {
+      fprintf(stderr, "SARG: (topuser) %s: %s\n",text[45],arqper);
+      exit(1);
+   }
+
+   if (!fgets(period,sizeof(period),fp_in)) {
+      fprintf(stderr,"SARG: (topuser) Read error in %s\n",arqper);
+      exit(1);
+   }
+   fclose(fp_in);
+
    if((fp_top1=fopen(top1,"r"))==NULL) {
       fprintf(stderr, "SARG: (topuser) %s: %s\n",text[45],top1);
       exit(1);
@@ -176,32 +250,16 @@ int topuser()
       exit(1);
    }
 
- /*
- * get period
- */
-
-   strcpy(arqper,dirname);
-   strcat(arqper,"/sarg-period");
-
-   if ((fp_in = fopen(arqper, "r")) == 0) {
-      fprintf(stderr, "SARG: (topuser) %s: %s\n",text[45],arqper);
-      exit(1);
-   }
-
-   fgets(period,sizeof(period),fp_in);
-   fclose(fp_in);
-
    fprintf(fp_top3, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n<html>\n<head>\n  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\">\n",CharSet);
-   fputs("</head>\n",fp_top3);
    css(fp_top3);
-   fprintf(fp_top3,"<body class=\"body\">");
-   if(strlen(LogoImage) > 0) fprintf(fp_top3, "<center><table cellpadding=\"0\" cellspacing=\"0\">\n<tr><th class=\"logo\"><img src='%s' border=0 align=absmiddle width=%s height=%s>&nbsp;%s</th></tr>\n<tr><td height=\"5\"></td></tr>\n</table>\n",LogoImage,Width,Height,LogoText);
+   fprintf(fp_top3,"</head>\n<body class=\"body\">");
+   write_logo_image(fp_top3);
 
    if(strcmp(IndexTree,"date") == 0)
       show_sarg(fp_top3, "../../..");
    else
       show_sarg(fp_top3, "..");
-   fputs("<center><table cellpadding=\"0\" cellspacing=\"0\">\n",fp_top3);
+   fputs("<div align=\"center\"><table cellpadding=\"0\" cellspacing=\"0\">\n",fp_top3);
    fprintf(fp_top3,"<tr><th align=\"center\" class=\"title\">%s</th></tr>\n",Title);
 
    fprintf(fp_top3,"<tr><td class=\"header3\">%s: %s</td></tr>\n",text[89],period);
@@ -213,18 +271,18 @@ int topuser()
    fprintf(fp_top3,"<tr><td class=\"header3\">%s</td></tr>\n",wheader);
    fprintf(fp_top3,"<tr><th class=\"header3\">%s</th></tr>\n",text[137]);
 
-   fputs("</table></center>\n",fp_top3);
-   fputs("<center><table cellpadding=\"1\" cellspacing=\"2\">\n",fp_top3);
+   fputs("</table></div>\n",fp_top3);
+   fputs("<div align=\"center\"><table cellpadding=\"1\" cellspacing=\"2\">\n",fp_top3);
    fputs("<tr><td><br></td><td></td></tr>\n",fp_top3);
 
-   if(strstr(ReportType,"topsites") != 0 && strcmp(Privacy,"yes") != 0) fprintf(fp_top3,"<tr><td class=\"link\" colspan=11><a href=\"topsites.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[119]);
-   if(strstr(ReportType,"sites_users") != 0 && strcmp(Privacy,"yes") != 0) fprintf(fp_top3,"<tr><td class=\"link\" colspan=11><a href=\"siteuser.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[85]);
-   if(dansguardian_count) fprintf(fp_top3,"<tr><td class=\"link\" colspan=11><a href=\"dansguardian.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[128]);
+   if(strstr(ReportType,"topsites") != 0 && strcmp(Privacy,"yes") != 0) fprintf(fp_top3,"<tr><td class=\"link\" colspan=\"11\"><a href=\"topsites.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[119]);
+   if(strstr(ReportType,"sites_users") != 0 && strcmp(Privacy,"yes") != 0) fprintf(fp_top3,"<tr><td class=\"link\" colspan=\"11\"><a href=\"siteuser.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[85]);
+   if(dansguardian_count) fprintf(fp_top3,"<tr><td class=\"link\" colspan=\"11\"><a href=\"dansguardian.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[128]);
    if(squidguard_count) fprintf(fp_top3,"<tr><td class=\"link\" colspan=11><a href=\"squidguard.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[122]);
-   if (strstr(ReportType,"downloads") != 0 && download_count && strcmp(Privacy,"yes") != 0 && ndownload) fprintf(fp_top3,"<tr><td class=\"link\" colspan=11><a href=\"download.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[125]);
-   if (strstr(ReportType,"denied") != 0 && denied_count && strcmp(Privacy,"yes") != 0) fprintf(fp_top3,"<tr><td class=\"link\" colspan=11><a href=\"denied.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[118]);
-   if (strstr(ReportType,"auth_failures") != 0 && authfail_count && strcmp(Privacy,"yes") != 0) fprintf(fp_top3,"<tr><td class=\"link\" colspan=11><a href=\"authfail.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[117]);
-   if(smartfilter) fprintf(fp_top3,"<tr><td class=\"link\" colspan=11><a href=\"smartfilter.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[116]);
+   if (strstr(ReportType,"downloads") != 0 && download_count && strcmp(Privacy,"yes") != 0 && ndownload) fprintf(fp_top3,"<tr><td class=\"link\" colspan=\"11\"><a href=\"download.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[125]);
+   if (strstr(ReportType,"denied") != 0 && denied_count && strcmp(Privacy,"yes") != 0) fprintf(fp_top3,"<tr><td class=\"link\" colspan=\"11\"><a href=\"denied.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[118]);
+   if (strstr(ReportType,"auth_failures") != 0 && authfail_count && strcmp(Privacy,"yes") != 0) fprintf(fp_top3,"<tr><td class=\"link\" colspan=\"11\"><a href=\"authfail.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[117]);
+   if(smartfilter) fprintf(fp_top3,"<tr><td class=\"link\" colspan=\"11\"><a href=\"smartfilter.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[116]);
    fputs("<tr><td></td></tr>\n",fp_top3);
 
    if (strstr(ReportType,"topuser") == 0) {
@@ -232,79 +290,111 @@ int topuser()
       if (fp_top3) fclose (fp_top3);
       return;
    }
-   strcpy(val1,text[100]);
-   strcpy(val2,text[98]);
-   strcpy(val3,text[92]);
-   strcpy(val4,text[93]);
-   sprintf(val11,"%%%s",text[93]);
-   sprintf(val5,"%s-%s-%s",text[113],text[114],text[112]);
-   strcpy(val6,text[94]);
-   strcpy(val7,text[95]);
-   sprintf(val8,"%%%s",text[99]);
-   strcpy(val9,"colspan=2");
-   bzero(val10, 255);
- 
-   strcpy(hbc1,"class=\"header\"");
-   strcpy(hbc2,"class=\"header\"");
-   strcpy(hbc3,"class=\"header\"");
-   strcpy(hbc4,"class=\"header\"");
-   strcpy(hbc5,"class=\"header3\"");
-   strcpy(hbc6,"class=\"header\"");
-   strcpy(hbc7,"class=\"header\"");
-   strcpy(hbc8,"class=\"header\"");
-   strcpy(hbc9,"class=\"header\"");
+
    strcpy(hbc10,"class=\"header\"");
 
    if(strstr(TopUserFields,"NUM") == 0) {
       bzero(val1, 255);
       bzero(hbc1, 30);
+   } else {
+      strcpy(val1,text[100]);
+      strcpy(hbc1,"class=\"header\"");
    }
    if(strstr(TopUserFields,"USERID") == 0) {
       bzero(val2, 255);
       bzero(hbc2, 30);
+   } else {
+      strcpy(val2,text[98]);
+      strcpy(hbc2,"class=\"header\"");
    }
    if(strstr(TopUserFields,"CONNECT") == 0) {
       bzero(val3, 255);
       bzero(hbc3, 30);
+   } else {
+      strcpy(val3,text[92]);
+      strcpy(hbc3,"class=\"header\"");
    }
    if(strstr(TopUserFields,"BYTES") == 0) {
       bzero(val4, 255);
       bzero(hbc4, 30);
+   } else {
+      strcpy(val4,text[93]);
+      strcpy(hbc4,"class=\"header\"");
    }
    if(strstr(TopUserFields,"SETYB") == 0) {
       bzero(val11, 255);
       bzero(hbc9, 30);
+   } else {
+      sprintf(val11,"%%%s",text[93]);
+      strcpy(hbc9,"class=\"header\"");
    }
    if(strstr(TopUserFields,"IN-CACHE-OUT") == 0) {
       bzero(val5, 255);
       bzero(hbc5, 30);
       bzero(val9, 255);
       strcpy(val10,"<td></td>");
+   } else {
+      sprintf(val5,"%s-%s-%s",text[113],text[114],text[112]);
+      strcpy(hbc5,"class=\"header3\"");
+      strcpy(val9,"colspan=\"2\"");
+      bzero(val10, 255);
    }
    if(strstr(TopUserFields,"USED_TIME") == 0) {
       bzero(val6, 255);
       bzero(hbc6, 30);
+   } else {
+      strcpy(val6,text[94]);
+      strcpy(hbc6,"class=\"header\"");
    }
-   if(strstr(TopUserFields,"MILISEC") == 0) { 
+   if(strstr(TopUserFields,"MILISEC") == 0) {
       bzero(val7, 255);
       bzero(hbc7, 30);
+   } else {
+      strcpy(val7,text[95]);
+      strcpy(hbc7,"class=\"header\"");
    }
    if(strstr(TopUserFields,"%TIME") == 0) {
       bzero(val8, 255);
       bzero(hbc8, 30);
+   } else {
+      sprintf(val8,"%%%s",text[99]);
+      strcpy(hbc8,"class=\"header\"");
    }
 
    fprintf(fp_top3,"<tr><th %s>%s</th><th %s></th><th %s>%s</th><th %s>%s</th><th %s>%s</th><th %s>%s</th><th %s %s>%s%s</th><th %s>%s</th><th %s>%s</th><th %s>%s</th></tr>\n",hbc1,val1,hbc1,hbc2,val2,hbc3,val3,hbc4,val4,hbc9,val11,hbc5,val9,val5,val10,hbc6,val6,hbc7,val7,hbc8,val8);
 
    ntopuser = 0;
 
-   fscanf(fp_top1,"%s%s%s%s%s%s",user,nbytes,nacc,elap,incac,oucac);
-   while(!feof(fp_top1)) {
+   while(fgets(warea,sizeof(warea),fp_top1)) {
+      if (getword(user,sizeof(user),warea,' ')<0) {
+         printf("SARG: Maybe you have a broken user in your %s file.\n",top1);
+         exit(1);
+      }
+      if (getword(nbytes,sizeof(nbytes),warea,' ')<0) {
+         printf("SARG: Maybe you have a broken number of bytes in your %s file.\n",top1);
+         exit(1);
+      }
+      if (getword(nacc,sizeof(nacc),warea,' ')<0) {
+         printf("SARG: Maybe you have a broken number of access in your %s file.\n",top1);
+         exit(1);
+      }
+      if (getword(elap,sizeof(elap),warea,' ')<0) {
+         printf("SARG: Maybe you have a broken elpased time in your %s file.\n",wger);
+         exit(1);
+      }
+      if (getword(incac,sizeof(incac),warea,' ')<0) {
+         printf("SARG: Maybe you have a broken in-cache size in your %s file.\n",wger);
+         exit(1);
+      }
+      if (getword(oucac,sizeof(oucac),warea,' ')<0) {
+         printf("SARG: Maybe you have a broken out-of-cache size in your %s file.\n",wger);
+         exit(1);
+      }
       if(atoi(nacc) < 1) {
-         fscanf(fp_top1,"%s%s%s%s%s%s",user,nbytes,nacc,elap,incac,oucac);
          continue;
-      } else ntopuser = 1;
-      if(atoi(TopUsersNum) > 0 && topcount >= atoi(TopUsersNum)) goto final;
+      }
+      ntopuser = 1;
+      if(TopUsersNum > 0 && topcount >= TopUsersNum) goto final;
       strcpy(user2,user);
       tnbytes=my_atoll(nbytes);
 
@@ -328,24 +418,27 @@ int topuser()
          ouperc=((ouperc * 100) / tnbytes);
       } else ouperc = 0;
 
-      strcpy(href," ");
-      strcpy(href2," ");
-      strcpy(href3," ");
-
       if(strcmp(user,"TOTAL") != 0){
-         sprintf(href,"<a href='%s/",user);
-         sprintf(href2,"<a href='%s/d",user);
-         sprintf(href3,"<a href='%s/graph_day.png'>",user);
-         strcat(href,user);
-         strcat(href2,user);
-         strcat(href,".html'>");
-         strcat(href2,".html'>");
+         sprintf(href1,"<a href=\"%s/%s.html\">",user,user);
+         strcpy(href1end,"</a>");
+         sprintf(href2,"<a href=\"%s/d%s.html\">",user,user);
+         strcpy(href2end,"</a>");
+         sprintf(href3,"<a href=\"%s/graph_day.png\">",user);
+         strcpy(href3end,"</a>");
+         posicao++;
+      } else {
+         strcpy(href1," ");
+         strcpy(href1end,"");
+         strcpy(href2," ");
+         strcpy(href2end,"");
+         strcpy(href3," ");
+         strcpy(href3end,"");
       }
 
-      if(strcmp(Graphs,"yes") != 0)
+      if(strcmp(Graphs,"yes") != 0) {
          bzero(href3,MAXLEN);
-
-      posicao++;
+         strcpy(href3end,"");
+      }
 
       tnelap=my_atoll(elap);
 
@@ -354,56 +447,46 @@ int topuser()
          if(strcmp(Ip2Name,"yes") == 0) {
             if(strcmp(user2,ipantes) != 0) {
                strcpy(ipantes,user2);
-               ip2name(user2);
+               ip2name(user2,sizeof(user2));
                strcpy(nameantes,user2);
             } else strcpy(user2,nameantes);
          }
       }
-    
+
       if(strstr(ReportType,"date_time") != 0) {
          sprintf(ltext110,"%s",text[110]);
-         if(ltext110){
-            for(s=ltext110; *s; ++s)
-               *s=tolower(*s);
-         }
+         for(s=ltext110; *s; ++s)
+            *s=tolower(*s);
       } else {
          bzero(href2, MAXLEN);
+         href2end[0]='\0';
          bzero(ltext110, 50);
          sprintf(val1,"%s/d%s.html",dirname,user);
-	 unlink(val1);
+         unlink(val1);
       }
 
-//      if(UserTabFile[0] != '\0' && strstr(user2,".") != 0) {
-      if(UserTabFile[0] != '\0' && strcmp(user2,"TOTAL") != 0) {
-         sprintf(warea,":%s:",user2);
-         if((str=(char *) strstr(userfile,warea)) != (char *) NULL ) {
-            z1=0;
-            str2=(char *) strstr(str+1,":");
-            str2++;
-            bzero(name, MAXLEN);
-            while(str2[z1] != ':') {
-               name[z1]=str2[z1];
-               z1++;
-            }
-         } else strcpy(name,user2);
-      } else strcpy(name,user2);
+      if(strcmp(user2,"TOTAL") != 0) {
+         get_usertab_name(user2,name,sizeof(name));
 
-      if((strcmp(Ip2Name,"yes") == 0) && 
-        ((str=(char *) strstr(name, ".")) != (char *) NULL) && 
-	((str=(char *) strstr(str+1, ".")) != (char *) NULL))
-               ip2name(name);
+         if((strcmp(Ip2Name,"yes") == 0) &&
+            ((str=(char *) strstr(name, ".")) != (char *) NULL) &&
+            ((str=(char *) strstr(str+1, ".")) != (char *) NULL))
+            ip2name(name,sizeof(name));
+      } else {
+         name[0]='\0';
+      }
 
       twork=my_atoll(nacc);
       my_lltoa(twork,nacc,0);
-      sprintf(wwork1,"%s",fixnum(twork,1));
-      sprintf(wwork2,"%s",fixnum(tnbytes,1));
-      sprintf(wwork3,"%s",fixnum2(tnelap,1));
+      strcpy(wwork1,fixnum(twork,1));
+      strcpy(wwork2,fixnum(tnbytes,1));
+      strcpy(wwork3,fixnum2(tnelap,1));
 
       sprintf(val1,"%d",posicao);
 #ifdef HAVE_GD
-      sprintf(val2,"%s<img src=\"%s/graph.png\" border=\"0\" title=\"%s\"></a>&nbsp;%s<img src=\"%s/datetime.png\" border=\"0\" title=\"%s %s\">\n",href3,ImageFile,text[126],href2,ImageFile,ltext110,text[55]);
+      sprintf(val2,"%s<img src=\"%s/graph.png\" border=\"0\" title=\"%s\" alt=\"G\">%s&nbsp;%s<img src=\"%s/datetime.png\" border=\"0\" title=\"%s %s\" alt=\"T\">%s\n",href3,ImageFile,text[126],href3end,href2,ImageFile,ltext110,text[55],href2end);
 #else
-      sprintf(val2,"%s<img src=\"%s/datetime.png\" border=\"0\" title=\"%s\">\n",href2,ImageFile,ltext110);
+      sprintf(val2,"%s<img src=\"%s/datetime.png\" border=\"0\" title=\"%s\" alt=\"T\">%s\n",href2,ImageFile,ltext110,href2end);
 #endif
 
       sprintf(val3,"%3.2f%%",perc);
@@ -435,7 +518,7 @@ int topuser()
          bzero(wwork1, 255);
          bzero(hbc3, 30);
       }
-      if(strstr(TopUserFields,"BYTES") == 0) { 
+      if(strstr(TopUserFields,"BYTES") == 0) {
          bzero(wwork2, 255);
          bzero(hbc4, 30);
       }
@@ -465,16 +548,16 @@ int topuser()
          bzero(hbc10, 30);
       }
 
-      if(strstr(ReportType,"users_sites") == 0)
-         href[0]='\0';
-
-      if(dotinuser && strstr(name,"_")) {
-         str2=(char *)subs(name,"_",".");
-         strcpy(name,str2);
-         free(str2);
+      if(strstr(ReportType,"users_sites") == 0) {
+         href1[0]='\0';
+         href1end[0]='\0';
       }
 
-      sprintf(preg,"<tr><td %s>%s</td><td %s>%s</td><td %s>%s%s</td><td %s>%s</td><td %s>%s</td><td %s>%s</td><td %s>%s</td><td %s>%s</td><td %s>%s</td><td %s>%s<td %s>%s</td></tr>\n",hbc1,val1,hbc2,val2,hbc2,href,name,hbc3,wwork1,hbc4,wwork2,hbc5,val3,hbc6,val4,hbc7,val5,hbc8,val6,hbc9,wwork3,hbc10,val7);
+      if(dotinuser && strchr(name,'_')) {
+         subs(name,sizeof(name),"_",".");
+      }
+
+      sprintf(preg,"<tr><td %s>%s</td><td %s>%s</td><td %s>%s%s%s</td><td %s>%s</td><td %s>%s</td><td %s>%s</td><td %s>%s</td><td %s>%s</td><td %s>%s</td><td %s>%s<td %s>%s</td></tr>\n",hbc1,val1,hbc2,val2,hbc2,href1,name,href1end,hbc3,wwork1,hbc4,wwork2,hbc5,val3,hbc6,val4,hbc7,val5,hbc8,val6,hbc9,wwork3,hbc10,val7);
 
       if(strstr(user,"TOTAL") != 0) {
 
@@ -488,16 +571,16 @@ int topuser()
             ouperc=atol(oucac) / ouperc;
          } else ouperc = 0;
 
-	 sprintf(wwork1,"%s",fixnum(ttnacc,1));
-	 sprintf(wwork2,"%s",fixnum(ttnbytes,1));
-	 sprintf(wwork3,"%s",fixnum2(ttnelap,1));
+         sprintf(wwork1,"%s",fixnum(ttnacc,1));
+         sprintf(wwork2,"%s",fixnum(ttnbytes,1));
+         sprintf(wwork3,"%s",fixnum2(ttnelap,1));
 
          strcpy(hbc1,"class=\"header2\"");
          strcpy(hbc2,"class=\"header2\"");
          strcpy(hbc3,"class=\"header2\"");
          strcpy(hbc4,"class=\"header2\"");
          strcpy(hbc5,"class=\"header2\"");
-         strcpy(hbc6,"class=\"header2\""); 
+         strcpy(hbc6,"class=\"header2\"");
          strcpy(hbc7,"class=\"header2\"");
          strcpy(hbc8,"class=\"header2\"");
          strcpy(hbc9,"class=\"header2\"");
@@ -532,17 +615,15 @@ int topuser()
          }
 
          if(strstr(ReportType,"date_time") != 0) {
-	    if(strstr(TopUserFields,"TOTAL") != 0)
+           if(strstr(TopUserFields,"TOTAL") != 0)
                sprintf(preg,"<tr><td></td><td></td><th %s>%s</th><th %s>%s</th><th %s>%15s</th><td></td><th %s>%s</th><th %s>%s</th><th %s>%s</th><th %s>%s</th></tr>\n",hbc10,text[107],hbc1,wwork1,hbc2,wwork2,hbc3,val4,hbc4,val5,hbc5,val6,hbc6,wwork3);
-	 } else if(strstr(TopUserFields,"TOTAL") != 0)
+        } else if(strstr(TopUserFields,"TOTAL") != 0)
             sprintf(preg,"<tr><td></td><td></td><th %s>%s</th><th %s>%s</th><th %s>%15s</th><td></td><th %s>%s</th><th %s>%s</th><th %s>%s</th><th %s>%s</th></tr>\n",hbc10,text[107],hbc1,wwork1,hbc2,wwork2,hbc3,val4,hbc4,val5,hbc5,val6,hbc6,wwork3);
       }
 
       fputs(preg,fp_top3);
 
       topcount++;
-
-      fscanf(fp_top1,"%s%s%s%s%s%s",user,nbytes,nacc,elap,incac,oucac);
    }
 
    if(ttnbytes) tnbytes=ttnbytes / totuser;
@@ -550,9 +631,9 @@ int topuser()
 
    twork=ttnacc/totuser;
    twork2=ttnelap/totuser;
-   sprintf(wwork1,"%s",fixnum(twork,1));
-   sprintf(wwork2,"%s",fixnum(tnbytes,1));
-   sprintf(wwork3,"%s",fixnum2(twork2,1));
+   strcpy(wwork1,fixnum(twork,1));
+   strcpy(wwork2,fixnum(tnbytes,1));
+   strcpy(wwork3,fixnum2(twork2,1));
 
    if(strstr(TopUserFields,"CONNECT") == 0) {
       bzero(wwork1, 255);
@@ -578,17 +659,19 @@ int topuser()
    }
 
    if(ntopuser) {
-      if((strstr(ReportType,"date_time") != 0 && strstr(TopUserFields,"AVERAGE") != 0)) fprintf(fp_top3,"<tr><td></td><th></th><th %s>%s</th><th %s>%s</th><th %s>%15s</th><td></td><td></td><td></td><th %s>%s</th><th %s>%s</th></tr>\n",hbc10,text[96],hbc1,wwork1,hbc2,wwork2,hbc3,buildtime(ttnelap/totuser),hbc4,wwork3);
-      else if(strstr(TopUserFields,"AVERAGE") != 0) fprintf(fp_top3,"<tr><td></td><th></th><td></td><th %s>%s</th><th %s>%s</th><th %s>%15s</th><td></td><td></td><td></td><th %s>%s</th><th %s>%s</th></tr>\n",hbc10,text[96],hbc1,wwork1,hbc2,wwork2,hbc3,buildtime(ttnelap/totuser),hbc4,wwork3);
+      if((strstr(ReportType,"date_time") != 0 && strstr(TopUserFields,"AVERAGE") != 0))
+         fprintf(fp_top3,"<tr><td></td><th></th><th %s>%s</th><th %s>%s</th><th %s>%15s</th><td></td><td></td><td></td><th %s>%s</th><th %s>%s</th></tr>\n",hbc10,text[96],hbc1,wwork1,hbc2,wwork2,hbc3,buildtime(ttnelap/totuser),hbc4,wwork3);
+      else if(strstr(TopUserFields,"AVERAGE") != 0)
+         fprintf(fp_top3,"<tr><td></td><th></th><td></td><th %s>%s</th><th %s>%s</th><th %s>%15s</th><td></td><td></td><td></td><th %s>%s</th><th %s>%s</th></tr>\n",hbc10,text[96],hbc1,wwork1,hbc2,wwork2,hbc3,buildtime(ttnelap/totuser),hbc4,wwork3);
    }
 
-   if(strlen(UserAgentLog) > 0) {
+   if(UserAgentLog[0] != '\0') {
       fputs("<tr><td></td></tr>\n",fp_top3);
       fputs("<tr><td></td></tr>\n",fp_top3);
-      fputs("<td align=\"left\" colspan=8><font size=-1><a href='useragent.html'>Useragent</a> Report</td>\n",fp_top3);
+      fputs("<td align=\"left\" colspan=\"8\"><font size=-1><a href=\"useragent.html\">Useragent</a> Report</td>\n",fp_top3);
    }
 
-   fputs("</table></center>",fp_top3);
+   fputs("</table></div>",fp_top3);
 
    show_info(fp_top3);
 
