@@ -1,6 +1,6 @@
 /*
  * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2008
+ *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
  *
  * SARG donations:
@@ -24,8 +24,9 @@
  */
 
 #include "include/conf.h"
+#include "include/defs.h"
 
-void day_totalize(const char *tmp, char *user, int indexonly)
+void day_totalize(const char *tmp, const char *user, int indexonly)
 {
 
    FILE *fp_in, *fp_ou;
@@ -46,6 +47,7 @@ void day_totalize(const char *tmp, char *user, int indexonly)
    int  regs=0;
    long long int telap=0;
    long long int tused=0;
+   int cstatus;
 
    if(indexonly) return;
    if(strstr(ReportType,"users_sites") == 0) return;
@@ -54,16 +56,21 @@ void day_totalize(const char *tmp, char *user, int indexonly)
    sprintf(arqout,"%s/%s.day",tmp,user);
    sprintf(sortout,"%s/%s.sort",tmp,user);
 
-   sprintf(csort,"sort -k 1,1 -k 2,2 -o '%s' '%s'",sortout,wdirname);
-   system(csort);
+   sprintf(csort,"sort -k 1,1 -k 2,2 -o \"%s\" \"%s\"",sortout,wdirname);
+   cstatus=system(csort);
+   if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
+      fprintf(stderr, "SARG: sort command return status %d\n",WEXITSTATUS(cstatus));
+      fprintf(stderr, "SARG: sort command: %s\n",csort);
+      exit(1);
+   }
+   if((fp_in=fopen(sortout,"r"))==NULL) {
+      fprintf(stderr, "SARG: (totday) %s: %s\n",text[8],sortout);
+      fprintf(stderr, "SARG: sort command: %s\n",csort);
+      exit(1);
+   }
 
    unlink(wdirname);
 
-   if((fp_in=fopen(sortout,"r"))==NULL) {
-     fprintf(stderr, "SARG: (totday) %s: %s\n",text[8],sortout);
-     exit(1);
-   }
-
    if((fp_ou=fopen(arqout,"w"))==NULL) {
      fprintf(stderr, "SARG: (totday) %s: %s\n",text[8],arqout);
      exit(1);
@@ -73,11 +80,12 @@ void day_totalize(const char *tmp, char *user, int indexonly)
       if(strstr(buf,"\n") != 0)
          buf[strlen(buf)-1]='\0';
          
-      getword(data,buf,' ');
-      getword(hora,buf,':');
-      getword(min,buf,':');
-      getword(elap,buf,' ');
-      strcpy(elap,buf);
+      if (getword(data,sizeof(data),buf,' ')<0 || getword(hora,sizeof(hora),buf,':')<0 ||
+          getword(min,sizeof(min),buf,':')<0 || getword(elap,sizeof(elap),buf,' ')<0 ||
+          getword(elap,sizeof(elap),buf,0)<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",sortout);
+         exit(1);
+      }
       sprintf(hm,"%s%s",hora,min);
 
       if(!regs) {
@@ -93,8 +101,7 @@ void day_totalize(const char *tmp, char *user, int indexonly)
             tused=telap;
 
          my_lltoa(telap,val1,15);
-         sprintf(buf,"%s %s %s\n",odata,ohora,val1);
-         fputs(buf, fp_ou);
+         fprintf(fp_ou,"%s %s %s\n",odata,ohora,val1);
          strcpy(odata,data);
          strcpy(ohora,hora);
          strcpy(ohm,hm);
@@ -115,8 +122,7 @@ void day_totalize(const char *tmp, char *user, int indexonly)
       tused=telap;
 
    my_lltoa(telap,val1,15);
-   sprintf(buf,"%s %s %s\n",data,hora,val1);
-   fputs(buf, fp_ou);
+   fprintf(fp_ou,"%s %s %s\n",data,hora,val1);
 
    fclose(fp_in);
    fclose(fp_ou);
