@@ -1,6 +1,6 @@
 /*
  * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2008
+ *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
  *
  * SARG donations:
@@ -24,8 +24,9 @@
  */
 
 #include "include/conf.h"
+#include "include/defs.h"
 
-void usage(char *prog)
+void usage(const char *prog)
 {
   fprintf(stderr, "%s: %s [%s...]\n", prog,text[39],text[40]);
   fprintf(stderr, "%5s-a %s\n"," ",text[23]);
@@ -45,8 +46,8 @@ void usage(char *prog)
   fprintf(stderr, "%5s-t %s [HH, HH:MM]\n"," ",text[31]);
   fprintf(stderr, "%5s-u %s\n"," ",text[32]);
   fprintf(stderr, "%5s-w %s\n"," ",text[34]);
-  fprintf(stderr, "%5s-x %s\n"," ",text[36]);
-  fprintf(stderr, "%5s-z %s\n"," ",text[35]);
+  fprintf(stderr, "%5s-x %s\n"," ",text[35]);
+  fprintf(stderr, "%5s-z %s\n"," ",text[36]);
   fprintf(stderr, "%5s-convert %s\n"," ",text[76]);
   fprintf(stderr, "%5s-split %s\n"," ",text[77]);
   fprintf(stderr, "\n\t%s-%s %s Pedro Lineu Orso -    pedro.orso@gmail.com\n",PGM,VERSION,text[78]);
