@@ -1,6 +1,6 @@
 /*
  * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2008
+ *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
  *
  * SARG donations:
@@ -24,8 +24,9 @@
  */
 
 #include "include/conf.h"
+#include "include/defs.h"
 
-void useragent()
+void useragent(void)
 {
 
    FILE *fp_in = NULL, *fp_ou = NULL, *fp_ht = NULL;
@@ -38,6 +39,7 @@ void useragent()
    char idate[MAXLEN], fdate[MAXLEN];
    int  agentot=0, agentot2=0, agentdif=0, cont=0;
    unsigned long totregsl=0;
+   int cstatus;
 
    ip[0]='\0';
    data[0]='\0';
@@ -64,28 +66,32 @@ void useragent()
    }
 
    if(debug) {
-      sprintf(msg,"%s: %s",text[66],UserAgentLog);
-      debuga(msg);
+      debuga("%s: %s",text[66],UserAgentLog);
    }
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
       totregsl++;
-      getword(ip,buf,' ');
-      getword(data,buf,'[');
-      getword(data,buf,' ');
+      if (getword(ip,sizeof(ip),buf,' ')<0 || getword(data,sizeof(data),buf,'[')<0 ||
+          getword(data,sizeof(data),buf,' ')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",UserAgentLog);
+         exit(1);
+      }
       if(totregsl == 1)
          strcpy(idate,data);
-      getword(agent,buf,'"');
-      getword(agent,buf,'"');
-
+      if (getword(agent,sizeof(agent),buf,'"')<0 || getword(agent,sizeof(agent),buf,'"')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",UserAgentLog);
+         exit(1);
+      }
       strcpy(warea,agent);
       strup(warea);
       if(strstr(warea,"SCRIPT") != 0 || strstr(warea,"ONLOAD") != 0)
          baddata();
 
       if(strlen(buf)) {
-         getword(user,buf,' ');
-         getword(user,buf,'\n');
+         if (getword(user,sizeof(user),buf,' ')<0 || getword(user,sizeof(user),buf,'\n')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",UserAgentLog);
+            exit(1);
+         }
       }
 
       if(user[0] == '-')
@@ -93,16 +99,14 @@ void useragent()
       if(strlen(user) == 0)
          strcpy(user,ip);
 
-      sprintf(buf,"%s\\%s\\%s\\%s\\\n",ip,data,agent,user);
-      fputs(buf,fp_ou);
+      fprintf(fp_ou,"%s\\%s\\%s\\%s\\\n",ip,data,agent,user);
       user[0]='\0';
    }
 
    strcpy(fdate,data);
 
    if(debug) {
-      sprintf(msg, "   %s: %ld",text[10],totregsl);
-      debuga(msg);
+      debuga("   %s: %ld",text[10],totregsl);
    }
 
    fclose(fp_in);
@@ -113,19 +117,23 @@ void useragent()
    }
 
    if(debug) {
-      sprintf(msg,"%s: %s",text[54],tmp2);
-      debuga(msg);
+      debuga("%s: %s",text[54],tmp2);
    }
 
-   sprintf(csort,"sort -n -t '\\' -k 4,4 -k 3,3 -k 2,2 -k 1,1 -o '%s' '%s'",tmp2,tmp3);
-   system(csort);
-
-   unlink(tmp3);
-
-   if((fp_in=fopen(tmp2,"r"))==NULL) {
-      fprintf(stderr, "SARG: (useragent) %s: %s\n",text[45],tmp2);
+   sprintf(csort,"sort -n -t \"\\\" -k 4,4 -k 3,3 -k 2,2 -k 1,1 -o \"%s\" \"%s\"",tmp2,tmp3);
+   cstatus=system(csort);
+   if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
+      fprintf(stderr, "SARG: sort command return status %d\n",WEXITSTATUS(cstatus));
+      fprintf(stderr, "SARG: sort command: %s\n",csort);
       exit(1);
    }
+   if((fp_in=fopen(tmp2,"r"))==NULL) {
+      fprintf(stderr, "SARG: (useragent) %s: %s\n",text[45],tmp2);
+      fprintf(stderr, "SARG: sort command: %s\n",csort);
+      exit(1);
+   }
+
+   unlink(tmp3);
 
    if((fp_ht=fopen(hfile,"w"))==NULL) {
       fprintf(stderr, "SARG: (useragent) %s: %s\n",text[45],hfile);
@@ -133,61 +141,61 @@ void useragent()
    }
  
    if(debug)
-      debuga(text[72]);
+      debuga("%s",text[72]);
 
    fprintf(fp_ht, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n<html>\n<head>\n  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\">\n",CharSet);
    fputs("</head>\n",fp_ht);
    if(strlen(FontFace) > 0) fprintf(fp_ht,"<font face=%s>\n",FontFace);
    fprintf(fp_ht,"<body bgcolor=%s text=%s background='%s'>\n",BgColor,TxColor,BgImage);
-   if(strlen(LogoImage) > 0) fprintf(fp_ht, "<center><table cellpadding=\"0\" cellspacing=\"0\">\n<tr><th class=\"logo\"><img src='%s' border=0 align=absmiddle width=%s height=%s>&nbsp;%s</th></tr>\n<tr><td height=\"5\"></td></tr>\n</table>\n",LogoImage,Width,Height,LogoText);
+   write_logo_image(fp_ht);
 
    if(strcmp(IndexTree,"date") == 0)
       show_sarg(fp_ht, "../../..");
    else
       show_sarg(fp_ht,"..");
-   fputs("<center><table cellpadding=0 cellspacing=0>\n",fp_ht);
-   sprintf(html,"<tr><th align=center colspan=2><b><font color=%s size=+1>%s</font></b></th></tr>\n",TiColor,text[105]);
-   fputs(html,fp_ht);
+   fputs("<div align=\"center\"><table cellpadding=\"0\" cellspacing=\"0\">\n",fp_ht);
+   fprintf(fp_ht,"<tr><th align=\"center\" colspan=\"2\"><b><font color=\"%s\" size=\"+1\">%s</font></b></th></tr>\n",TiColor,text[105]);
 
    fputs("<tr><td></td></tr><tr><td></td></tr>",fp_ht);
-   fputs("</table></center>\n",fp_ht);
+   fputs("</table></div>\n",fp_ht);
 
-   fputs("<center><table cellpadding=0 cellspacing=0>\n",fp_ht);
+   fputs("<div align=\"center\"><table cellpadding=\"0\" cellspacing=\"0\">\n",fp_ht);
 
-   sprintf(html,"<tr><td align=right bgcolor=%s><font size=%s>%s:</font><td align=left bgcolor=%s><font size=%s>%s - %s</font></td></td></tr>\n",HeaderBgColor,FontSize,text[89],TxBgColor,FontSize,idate,fdate);
-   fputs(html,fp_ht);
+   fprintf(fp_ht,"<tr><td align=\"right\" bgcolor=\"%s\"><font size=\"%s\">%s:</font><td align=\"left\" bgcolor=\"%s\"><font size=\"%s\">%s - %s</font></td></td></tr>\n",HeaderBgColor,FontSize,text[89],TxBgColor,FontSize,idate,fdate);
 
-   fputs("</table></center>\n",fp_ht);
-   fputs("<center><table cellpadding=0 cellspacing=0>\n",fp_ht);
+   fputs("</table></div>\n",fp_ht);
+   fputs("<div align=\"center\"><table cellpadding=\"0\" cellspacing=\"0\">\n",fp_ht);
    fputs("<tr><td></td><td></td></tr>",fp_ht);
 
-   sprintf(html,"<tr><th align=left bgcolor=%s><font size=%s color=%s>%s</font></th><th bgcolor=%s align=left><font size=%s color=%s>%s</font></th></tr>\n",HeaderBgColor,FontSize,HeaderColor,text[98],HeaderBgColor,FontSize,HeaderColor,text[106]);
-   fputs(html,fp_ou);
+   fprintf(fp_ou,"<tr><th align=left bgcolor=%s><font size=%s color=%s>%s</font></th><th bgcolor=%s align=left><font size=%s color=%s>%s</font></th></tr>\n",HeaderBgColor,FontSize,HeaderColor,text[98],HeaderBgColor,FontSize,HeaderColor,text[106]);
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-      getword(ip,buf,'\\');
+      if (getword(ip,sizeof(ip),buf,'\\')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp2);
+         exit(1);
+      }
 
       if(strcmp(Ip2Name,"yes") == 0) {
          if(strcmp(ip,ipbefore) != 0) {
             strcpy(ipbefore,ip);
-            ip2name(ip);
+            ip2name(ip,sizeof(ip));
             strcpy(namebefore,ip);
          } else strcpy(ip,namebefore);
       }
 
-      getword(data,buf,'\\');
-      getword(agent,buf,'\\');
-      getword(user,buf,'\\');
+      if (getword(data,sizeof(data),buf,'\\')<0 || getword(agent,sizeof(agent),buf,'\\')<0 ||
+          getword(user,sizeof(user),buf,'\\')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp2);
+         exit(1);
+      }
 
       if(strcmp(user,user_old) != 0) {
-         sprintf(html,"<tr><td align=left bgcolor=%s><font size=%s>%s</td><td align=left bgcolor=%s><font size=%s>%s</td></tr>\n",TxBgColor,FontSize,user,TxBgColor,FontSize,agent);
-         fputs(html,fp_ht);
+         fprintf(fp_ht,"<tr><td align=left bgcolor=%s><font size=%s>%s</td><td align=left bgcolor=%s><font size=%s>%s</td></tr>\n",TxBgColor,FontSize,user,TxBgColor,FontSize,agent);
          strcpy(user_old,user);
          strcpy(agent_old,agent);
       } else {
          if(strcmp(agent,agent_old) != 0) {
-            sprintf(html,"<tr><td></td><td align=left bgcolor=%s><font size=%s>%s</td></tr>\n",TxBgColor,FontSize,agent);
-            fputs(html,fp_ht);
+            fprintf(fp_ht,"<tr><td></td><td align=left bgcolor=%s><font size=%s>%s</td></tr>\n",TxBgColor,FontSize,agent);
             strcpy(agent_old,agent);
          }
       }
@@ -197,15 +205,20 @@ void useragent()
    fclose(fp_in);
    fclose(fp_ou);
 
-   sprintf(csort,"sort -n -t '\\' -k 3,3 -o '%s' '%s'",tmp3,tmp2);
-   system(csort);
-
-   unlink(tmp2);
-
-   if((fp_in=fopen(tmp3,"r"))==NULL) {
-      fprintf(stderr, "SARG: (useragent) %s: %s\n",text[45],tmp3);
+   sprintf(csort,"sort -n -t \"\\\" -k 3,3 -o \"%s\" \"%s\"",tmp3,tmp2);
+   cstatus=system(csort);
+   if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
+      fprintf(stderr, "SARG: sort command return status %d\n",WEXITSTATUS(cstatus));
+      fprintf(stderr, "SARG: sort command: %s\n",csort);
       exit(1);
    }
+   if((fp_in=fopen(tmp3,"r"))==NULL) {
+      fprintf(stderr, "SARG: (useragent) %s: %s\n",text[45],tmp3);
+      fprintf(stderr, "SARG: sort command: %s\n",csort);
+      exit(1);
+   }
+
+   unlink(tmp2);
 
    if((fp_ou=fopen(tmp2,"w"))==NULL) {
       fprintf(stderr, "SARG: (useragent) %s: %s\n",text[45],tmp2);
@@ -215,9 +228,11 @@ void useragent()
    agent_old[0]='\0';
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-      getword(ip,buf,'\\');
-      getword(data,buf,'\\');
-      getword(agent,buf,'\\');
+      if (getword(ip,sizeof(ip),buf,'\\')<0 || getword(data,sizeof(data),buf,'\\')<0 ||
+          getword(agent,sizeof(agent),buf,'\\')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp);
+         exit(1);
+      }
 
       if(!cont) {
          cont++;
@@ -228,16 +243,14 @@ void useragent()
 
       if(strcmp(agent,agent_old) != 0) {
          agentdif++;
-         sprintf(html,"%06d %s\n",agentot,agent_old);
-         fputs(html,fp_ou);
+         fprintf(fp_ou,"%06d %s\n",agentot,agent_old);
          strcpy(agent_old,agent);
          agentot2+=agentot;
          agentot=1;
       }
    }
    agentdif++;
-   sprintf(html,"%06d %s\n",agentot,agent);
-   fputs(html,fp_ou);
+   fprintf(fp_ou,"%06d %s\n",agentot,agent);
    agentot2+=agentot;
 
    fclose(fp_in);
@@ -245,15 +258,20 @@ void useragent()
 
    unlink(tmp3);
 
-   sprintf(csort,"sort -n -r -k 1,1 -o '%s' '%s'",tmp3,tmp2);
-   system(csort);
-
-   unlink(tmp2);
-
-   if((fp_in=fopen(tmp3,"r"))==NULL) {
-      fprintf(stderr, "SARG: (useragent) %s: %s\n",text[45],tmp3);
+   sprintf(csort,"sort -n -r -k 1,1 -o \"%s\" \"%s\"",tmp3,tmp2);
+   cstatus=system(csort);
+   if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
+      fprintf(stderr, "SARG: sort command return status %d\n",WEXITSTATUS(cstatus));
+      fprintf(stderr, "SARG: sort command: %s\n",csort);
       exit(1);
    }
+   if((fp_in=fopen(tmp3,"r"))==NULL) {
+      fprintf(stderr, "SARG: (useragent) %s: %s\n",text[45],tmp3);
+      fprintf(stderr, "SARG: sort command: %s\n",csort);
+      exit(1);
+   }
+
+   unlink(tmp2);
 
    if((fp_ht=fopen(hfile,"a"))==NULL) {
       fprintf(stderr, "SARG: (useragent) %s: %s\n",text[45],hfile);
@@ -262,25 +280,25 @@ void useragent()
 
    fputs("<br><br>\n",fp_ht);
 
-   fputs("<center><table cellpadding=0 cellspacing=0>\n",fp_ht);
-   sprintf(html,"<tr><th align=left bgcolor=%s><font size=%s color=%s>%s</font></th><th bgcolor=%s align=left><font size=%s color=%s>%s</font></th><th align=center bgcolor=%s><font size=%s color=%s>%%</font></th></tr>\n",HeaderBgColor,FontSize,HeaderColor,text[106],HeaderBgColor,FontSize,HeaderColor,text[107],HeaderBgColor,FontSize,HeaderColor);
-   fputs(html,fp_ht);
+   fputs("<div align=\"center\"><table cellpadding=\"0\" cellspacing=\"0\">\n",fp_ht);
+   fprintf(fp_ht,"<tr><th align=\"left\" bgcolor=\"%s\"><font size=\"%s\" color=\"%s\">%s</font></th><th bgcolor=\"%s\" align=\"left\"><font size=\"%s\" color=\"%s\">%s</font></th><th align=\"center\" bgcolor=\"%s\"><font size=\"%s\" color=\"%s\">%%</font></th></tr>\n",HeaderBgColor,FontSize,HeaderColor,text[106],HeaderBgColor,FontSize,HeaderColor,text[107],HeaderBgColor,FontSize,HeaderColor);
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
       buf[strlen(buf)-1]='\0';
-      getword(tagent,buf,' ');
+      if (getword(tagent,sizeof(tagent),buf,' ')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp);
+         exit(1);
+      }
       perc=atoi(tagent) * 100;
       perc=perc / agentot2;
 
-      sprintf(html,"<tr><td align=left bgcolor=%s><font size=%s>%s</td><td align=right bgcolor=%s><font size=%s>%d</td><td align=right bgcolor=%s><font size=%s>%3.2f</td></tr>\n",TxBgColor,FontSize,buf,TxBgColor,FontSize,atoi(tagent),TxBgColor,FontSize,perc);
-      fputs(html,fp_ht);
+      fprintf(fp_ht,"<tr><td align=left bgcolor=%s><font size=%s>%s</td><td align=right bgcolor=%s><font size=%s>%d</td><td align=right bgcolor=%s><font size=%s>%3.2f</td></tr>\n",TxBgColor,FontSize,buf,TxBgColor,FontSize,atoi(tagent),TxBgColor,FontSize,perc);
    }
 
    fputs("</table></html>\n",fp_ht);
    if(strcmp(ShowSargInfo,"yes") == 0) {
       zdate(ftime, DateFormat);
-      sprintf(html,"<br><br><center><font size=-2>%s <a href='%s'>%s-%s</a> %s %s</font></center>\n",text[108],URL,PGM,VERSION,text[109],ftime);
-      fputs(html,fp_ht);
+      fprintf(fp_ht,"<br><br><div align=\"center\"><font size=\"-2\">%s <a href=\"%s\">%s-%s</a> %s %s</font></div>\n",text[108],URL,PGM,VERSION,text[109],ftime);
    }
 
    fputs("</table>\n</body>\n</html>\n",fp_ht);
